/*
    This file contains 2 functions.  The first is called by
    either "qdedge" or "tredge".  The second is called by
    the main function to find the edge corresponding to nodes
    on a boundary edge.

		Updated 11/20/09

    SLFCEM source file
    Version:  1.1
    Copyright (C) 2009  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if QUADE
#include "../quad/quad_em/qdeconst.h"
#include "../quad/quad_em/qdestruct.h"
#endif
#if TRIE
#include "../tri/tri_em/treconst.h"
#include "../tri/tri_em/trestruct.h"
#endif


int edge_record( int *node_edge_connect, int *edge_connect, int *numed2,
	int i, int edge_num, int node00, int node11 )
{
/*
    This library function does the actual generation of 
    "node_edge_connect" as well "edge_connect".  "edge_connect"
    is continously generated while "node_edge_connect" only
    records when a particular edge's nodal connectivity hasn't
    already been recorded.

		Updated 11/11/09
*/

	int j, new_edge_flag;

	new_edge_flag = 1;
	for( j = 0; j < *numed2; ++j )
	{
		if( *(node_edge_connect + nped*j) == node00 )
		{
		    if( *(node_edge_connect + nped*j + 1) == node11 )
		    {
                                /* Edge has already been recorded. */
			*(edge_connect+epel*i + edge_num) = j;
			new_edge_flag = 0;
			break;
		    }
		}
	}
	if( new_edge_flag )
	{
                        /* Record new edge. */
		*(edge_connect+epel*i + edge_num) = (*numed2-1);
		*(node_edge_connect+nped*(*numed2-1))     = node00;
		*(node_edge_connect+nped*(*numed2-1) + 1) = node11;
		++*numed2;
	}

	return 1;
}


int edge_id( BOUND bc, double *EMedge, int *node_edge_connect, int numed2)
{
/*
    This library function identifies the edge based on node00 and node11
    and uses it to set the boundary condition variables.

		Updated 11/11/09
*/

	int i, j, dum, node00, node11;

	for( i = 0; i <  bc.num_edge[0]; ++i )
	{
	    for( j = 0; j < numed2; ++j )
	    {
		node00 =  *(node_edge_connect + nped*j);
		node11 =  *(node_edge_connect + nped*j + 1);

		if( bc.edge_node[nped*i] == node00 )
		{
		    if( bc.edge_node[nped*i + 1] == node11 )
		    {
			bc.edge[i] = j;
			*(EMedge + Vndof*bc.edge[i]) = bc.EMedge[Vndof*i];
			break;
		    }
		}
	    }
	}

/* This is the prescribed electric field edges for the elements. */

/*
        printf( "\n\n");
	for( i = 0; i <  bc.num_edge[0]; ++i )
	{
	    printf( "%4d %4d %16.8e\n", i, bc.edge[i], *(EMedge + Vndof*bc.edge[i]));
	}
*/

	return 1;
}


int edge_id2( int *edge_list, int i, int *node_edge_connect, int numed2, int node00, int node11 )
{
/*
    This library function identifies the edge based on node00 and node11.

		Updated 11/27/09
*/

	int j;

	for( j = 0; j < numed2; ++j )
	{
		if( *(node_edge_connect + nped*j) == node00 )
		{
		    if( *(node_edge_connect + nped*j + 1) == node11 )
		    {
			edge_list[i] = j;
			printf( "This is i =%4d for edge %4d\n", i, edge_list[i]);
			break;
		    }
		}
	}

	return 1;
}

