/*
    This library function reads in data for a finite element
    program which does analysis on a quadrilateral element

		Updated 12/28/09

    SLFCEM source file
    Version:  1.1
    Copyright (C) 1999-2009  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#if QUADE
#include "../quad/quad_em/qdeconst.h"
#include "../quad/quad_em/qdestruct.h"
#endif
#if TRIE
#include "../tri/tri_em/treconst.h"
#include "../tri/tri_em/trestruct.h"
#endif


extern int dof, EMdof, nmat, nmode, numed, numel, numnp, modal_flag, static_flag;
extern int edge_connect_read_flag;
extern double sigma, low_mode;

int Ereader( BOUND bc, int *connect, int *node_edge_connect, int *edge_connect,
	double *coord, int *el_matl, double *charge, MATL *matl, char *name,
	FILE *o1, double *EMedge, double *Volt)
{
	int i, j, dum, dum1, dum2, dum3, dum4, dum5, name_length, file_loc;
	char *ccheck, one_char;
	char buf[ BUFSIZ ];
	char text;
	FILE *o4;

	printf( "number of edges:%d elements:%d nodes:%d materials:%d modes:%d dof:%d EMdof:%d\n",
		numed, numel, numnp, nmat, nmode, dof, EMdof);
	printf( " sigma:%16.8e low mode:%16.8e\n", sigma, low_mode);
	fgets( buf, BUFSIZ, o1 );
	printf( "\n");

	for( i = 0; i < nmat; ++i )
	{
	   fscanf( o1, "%d ",&dum);
	   printf( "material (%3d) permittivity, permeability, oper. fq.",dum);
	   fscanf( o1, " %lf %lf %lf\n", &matl[dum].er, &matl[dum].ur, &matl[dum].ko);
	   printf( " %7.3e %7.3e %7.3e\n", matl[dum].er, matl[dum].ur, matl[dum].ko);

	   matl[dum].lam = 2.0*pi/matl[dum].ko;                /* Calculate wavelength */
	   matl[dum].fq  = matl[dum].ko*speed_light/(2.0*pi);  /* Calculate operating frequency */
	}
	fgets( buf, BUFSIZ, o1 );
	printf( "\n");

	file_loc = ftell(o1);

	fscanf( o1,"%d %d %d",&dum, &dum1, &dum2);

/* Check if this data contains the edge connectivity.  */

	edge_connect_read_flag = 1;
	while(( one_char = (unsigned char) fgetc(o1)) != '\n')
	{
		if(one_char != ' ' )
		{
		    ungetc( one_char, o1);
		    fscanf( o1,"%d %d", &dum3, &dum4, &dum5);
		    edge_connect_read_flag = 0;
		    break;
		}
	}

	fseek(o1, file_loc, 0);

	if(edge_connect_read_flag)
	{
            for( i = 0; i < numed; ++i )
            {
                fscanf( o1,"%d ",&dum);
                printf( "connectivity for edge (%4d) ",dum);
                for( j = 0; j < nped; ++j )
                {
                    fscanf( o1, "%d",(node_edge_connect+nped*dum+j));
                    printf( "%4d ",*(node_edge_connect+nped*dum+j));
                }
                fscanf( o1,"\n");
                printf( "\n");
            }
            fgets( buf, BUFSIZ, o1 );
            printf( "\n");
	}

	for( i = 0; i < numel; ++i )
	{
	    fscanf( o1,"%d ",&dum);
	    printf( "connectivity for element (%4d) ",dum);
	    for( j = 0; j < npel; ++j )
	    {
		fscanf( o1, "%d",(connect+npel*dum+j));
		printf( "%4d ",*(connect+npel*dum+j));
	    }
	    printf( "    ");
	    if(edge_connect_read_flag)
	    {
		for( j = 0; j < epel; ++j )
		{
		    fscanf( o1, "%d",(edge_connect+epel*dum+j));
		    printf( "%4d ",*(edge_connect+epel*dum+j));
		}
	    }
	    fscanf( o1,"%d\n",(el_matl+dum));
	    printf( " with matl %3d\n",*(el_matl+dum));
	}
	fgets( buf, BUFSIZ, o1 );
	printf( "\n");

	for( i = 0; i < numnp; ++i )
	{
	   fscanf( o1,"%d ",&dum);
	   printf( "coordinate (%d) ",dum);
	   printf( "coordinates ");
	   for( j = 0; j < nsd; ++j )
	   {
		fscanf( o1, "%lf ",(coord+nsd*dum+j));
		printf( "%9.6e ",*(coord+nsd*dum+j));
	   }
	   fscanf( o1,"\n");
	   printf( "\n");
	}
	fgets( buf, BUFSIZ, o1 );
	printf( "\n");

	if(static_flag)
	{
		dum= 0;
		fscanf( o1,"%d",&bc.Volt[dum]);
		printf( "Volt (%4d) has a prescribed value of: ",bc.Volt[dum]);
		while( bc.Volt[dum] > -1 )
		{
		    fscanf( o1,"%lf\n%d",(Volt+Vndof*bc.Volt[dum]),
			    &bc.Volt[dum+1]);
		    printf( "%14.6e\n",*(Volt+Vndof*bc.Volt[dum]));
		    printf( "Volt (%4d) has a prescribed value of: ",
			    bc.Volt[dum+1]);
		    ++dum;
		}
		bc.num_Volt[0]=dum;
	}
	if(modal_flag)
	{
	    if(edge_connect_read_flag)
	    {
		dum= 0;
		fscanf( o1,"%d",&bc.edge[dum]);
		printf( "edge (%4d) has a prescribed value of: ",bc.edge[dum]);
		while( bc.edge[dum] > -1 )
		{
		    fscanf( o1,"%lf\n%d",(EMedge+edof*bc.edge[dum]),
			    &bc.edge[dum+1]);
		    printf( "%14.6e\n",*(EMedge+edof*bc.edge[dum]));
		    printf( "edge (%4d) has a prescribed value of: ",
			    bc.edge[dum+1]);
		    ++dum;
		}
		bc.num_edge[0]=dum;
	    }
	    else
	    {
		dum= 0;
		fscanf( o1,"%d",&bc.edge_node[0]);
		printf( "edge node1: %5d   ", bc.edge_node[0]);
		while( bc.edge_node[nped*dum] > -1 )
		{
		    fscanf( o1,"%d", &bc.edge_node[nped*dum+1]);
		    fscanf( o1,"%lf\n%d",(bc.EMedge + edof*dum),
			    &bc.edge_node[nped*(dum+1)]);
		    printf( "node2: %5d   has a prescribed value of: ", bc.edge_node[nped*dum+1]);
		    printf( "%14.6e\n",*(bc.EMedge + edof*dum));
		    printf( "edge node1: %5d   ", bc.edge_node[nped*(dum+1)]);

		    dum1 = bc.edge_node[nped*dum];
		    dum2 = bc.edge_node[nped*dum+1];
		    if(dum1 > dum2)
		    {
			    bc.edge_node[nped*dum+1] = dum1;
			    bc.edge_node[nped*dum] = dum2;
		    }

		    ++dum;
		}
		bc.num_edge[0]=dum;
	    }
	}

	fscanf( o1,"\n");
	fgets( buf, BUFSIZ, o1 );
	printf( "\n\n");

	dum= 0;
	printf("charge vector for node: ");
	fscanf( o1,"%d",&bc.charge[dum]);
	printf( "(%4d)",bc.charge[dum]);
	while( bc.charge[dum] > -1 )
	{
	   fscanf( o1,"%lf ",(charge+Vndof*bc.charge[dum]));
	   printf("%14.6e ",*(charge+Vndof*bc.charge[dum]));
	   fscanf( o1,"\n");
	   printf( "\n");
	   printf("charge vector for node: ");
	   ++dum;
	   fscanf( o1,"%d",&bc.charge[dum]);
	   printf( "(%4d)",bc.charge[dum]);
	}
	bc.num_charge[0]=dum;
	printf( "\n\n");

	return 1;
}

