/*
    This library function writes the resulting data for a finite element
    program which does analysis on a quadrilateral element 

		Updated 1/12/10

    SLFCEM source file
    Version:  1.1
    Copyright (C) 1999-2010  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if QUADE
#include "../quad/quad_em/qdeconst.h"
#include "../quad/quad_em/qdestruct.h"
#endif
#if TRIE
#include "../tri/tri_em/treconst.h"
#include "../tri/tri_em/trestruct.h"
#endif


extern int dof, EMdof, nmat, nmode, numed, numel, numnp, modal_flag, static_flag;
extern int edge_connect_read_flag;
extern double sigma, low_mode;

int Ewriter ( BOUND bc, int *connect, int *node_edge_connect, int *edge_connect,
	double *coord, int *el_matl, double *charge, int *id, MATL *matl, char *name,
	double *EMnode, double *EMedge, double *EMmag, double *Volt)
{
	int i,j,dum,check, node, name_length;
	char *ccheck;
	double fpointx, fpointy, fpointz, fdum, fdum2;
	char out[30];
	FILE *o3, *o4;

/* Every output file is named after the input file with
   an ".oqd" or ".otr" extension */

	name_length = strlen(name);
	if( name_length > 35) name_length = 35;

	memset(out,0,30*sizeof(char));

	ccheck = strncpy(out, name, name_length);
	if(!ccheck) printf( " Problems with strncpy \n");

#if QUADE
	ccheck = strncpy(out+name_length, ".oqd", 4);
	if(!ccheck) printf( " Problems with strncpy \n");
#endif
#if TRIE
	ccheck = strncpy(out+name_length, ".otr", 4);
	if(!ccheck) printf( " Problems with strncpy \n");
#endif

	o3 = fopen( out,"w" );

	fprintf( o3, "   numed numel numnp nmat nmode sigma low_mode");
	fprintf( o3, " (This is for the quad mesh file: %s)\n", name);
	fprintf( o3, "%4d %4d %4d %4d %4d %16.8e %16.8e\n",
		numed, numel, numnp, nmat, nmode, sigma, low_mode);
	fprintf( o3, "matl no., permeability, permittivity, operating fq\n");
	for( i = 0; i < nmat; ++i )
	{

	    fprintf( o3, "  %4d    %12.6e %12.6e %12.6e\n ",i,
		matl[i].er, matl[i].ur, matl[i].ko);
	}

        if(edge_connect_read_flag)
        {
            fprintf( o3, "edge no., node connectivity\n");
            for( i = 0; i < numed; ++i )
            {
                fprintf( o3, "%4d    ", i);
                for( j = 0; j < nped; ++j )
                {
                    fprintf( o3, "%4d ",*(node_edge_connect+nped*i+j));
                }
                fprintf( o3, "\n");
            }
        }

	fprintf( o3, "el no., connectivity, matl no. \n");
	for( i = 0; i < numel; ++i )
	{
	    fprintf( o3, "%6d ",i);
	    for( j = 0; j < npel; ++j )
	    {
		fprintf( o3, "%6d ",*(connect+npel*i+j));
	    }
	    fprintf( o3, "    ");
	    if(edge_connect_read_flag)
	    {
		for( j = 0; j < epel; ++j )
		{
		    fprintf( o3, "%4d ",*(edge_connect+epel*i+j));
		}
	    }
	    fprintf( o3, "   %3d\n",*(el_matl+i));
	}

	fprintf( o3, "node no., coordinates \n");

	for( i = 0; i < numnp; ++i )
	{
/*
	     fpointx = *(coord+nsd*i) + *(EMnode+ndof*i);
	     fpointy = *(coord+nsd*i+1) + *(EMnode+ndof*i+1);
*/
	     fpointx = *(coord+nsd*i);
	     fpointy = *(coord+nsd*i+1);
	     fprintf( o3, "%4d %14.6f %14.6f\n", i, fpointx, fpointy );
	}

	if( static_flag)
	{
	    fprintf( o3, "prescribed voltage: node, voltage\n");
	    for( i = 0; i < numnp; ++i )
	    {
		fprintf( o3, "%4d %14.6e\n", i, *(Volt+Vndof*i));
	    }
	    fprintf( o3, " -10 \n");
	}
	if( modal_flag)
	{
	    if(edge_connect_read_flag)
	    {
		fprintf( o3, "prescribed edge: edge value\n");
		for( i = 0; i < numed; ++i )
		{
		    fprintf( o3, "%4d %14.6e\n", i, *(EMedge+Vndof*i));
		}
	    }
	    else
	    {
		fprintf( o3, "prescribed edge: node1, node2 edge value\n");
		for( i = 0; i < numed; ++i )
		{
		    fprintf( o3, "%4d ",*(node_edge_connect+nped*i));
		    fprintf( o3, "%4d ",*(node_edge_connect+nped*i+1));
		    fprintf( o3, "%14.6e\n", *(EMedge+Vndof*i));
		}
	    }
	    fprintf( o3, " -10 \n");
	}

	if( static_flag)
	{
	    fprintf( o3, "node with point charged, point charge\n");
	    for( i = 0; i < bc.num_charge[0] ; ++i )
	    {
		node = bc.charge[i];
		*(id+Vndof*node) = -1;
	    }

	    for( i = 0; i < numnp; ++i )
	    {
		if( *(id+Vndof*i) < 0 )
		{
		    fprintf( o3,"%4d",i);
		    fprintf( o3," %16.8e\n", *(charge+Vndof*i));
		}
	    }
	}
	else
	{
	    fprintf( o3, "node with point charged, point charge\n");
	    for( i = 0; i < bc.num_charge[0]; ++i )
	    {
		fprintf( o3, "%4d %14.6e\n", i, *(charge+Vndof*bc.charge[i]));
	    }
	}
	fprintf( o3, " -10 \n");

	fprintf( o3, "Electric Field: node, x and y component, magnitude\n");
	for( i = 0; i < numnp; ++i )
	{
	    fprintf( o3, "%4d %16.8e %16.8e %16.8e\n",
		i, *(EMnode+ndof*i), *(EMnode+ndof*i+1), *(EMmag+Vndof*i));
	}
	fprintf( o3, " -10 \n");

	return 1;
}

