/*
    This library function writes the resulting data for a finite element
    program which does analysis on a trianglerilateral element 

		Updated 1/23/10

    SLFCEM source file
    Version:  1.1
    Copyright (C) 1999-2010  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if QUADE
#include "../quad/quad_em/qdeconst.h"
#include "../quad/quad_em/qdestruct.h"
#endif
#if TRIE
#include "../tri/tri_em/treconst.h"
#include "../tri/tri_em/trestruct.h"
#endif

extern int dof, nmat, nmode, numel, numnp;
extern int modal_flag, static_flag;

int writer ( BOUND bc, int *connect, double *coord, int *el_matl, double *charge,
	int *id, MATL *matl, char *name, double *EMnode, double *EMmag)
{
	int i,j,dum,check, node, name_length;
	char *ccheck;
	double fpointx, fpointy, fpointz;
	char out[30];
	FILE *o3, *o4;

/* Every output file is named after the input file with
   a ".ooo" extension */

	name_length = strlen(name);
	if( name_length > 25) name_length = 25;

	memset(out,0,30*sizeof(char));

	ccheck = strncpy(out, name, name_length);
	if(!ccheck) printf( " Problems with strncpy \n");

	ccheck = strncpy(out+name_length, ".ooo", 4);
	if(!ccheck) printf( " Problems with strncpy \n");

	o3 = fopen( out,"w" );

	fprintf( o3, "   numel numnp nmat nmode plane_stress_flag");
	fprintf( o3, " (This is for the mesh file: %s)\n", name);
	fprintf( o3, "    %4d %4d %4d %4d %4d\n ",
		numel,numnp,nmat,nmode, 0);
	fprintf( o3, "matl no., E modulus, Poisson Ratio, density, thickness\n");
	for( i = 0; i < nmat; ++i )
	{
	   fprintf( o3, "  %4d    %12.6e %12.6e %12.6e\n",i,
		matl[i].er, matl[i].ur, matl[i].ko);
	}

	fprintf( o3, "el no., connectivity, matl no. \n");
	for( i = 0; i < numel; ++i )
	{
	   fprintf( o3, "%6d ",i);
	   for( j = 0; j < npel; ++j )
	   {
		fprintf( o3, "%6d ",*(connect+npel*i+j));
	   }
	   fprintf( o3, "   %3d\n",*(el_matl+i));
	}

	fprintf( o3, "node no., coordinates \n");

	for( i = 0; i < numnp; ++i )
	{
/*
		fpointx = *(coord+nsd*i) + *(EMnode+ndof*i);
		fpointy = *(coord+nsd*i+1) + *(EMnode+ndof*i+1);
*/
		fpointx = *(coord+nsd*i);
		fpointy = *(coord+nsd*i+1);
		fprintf( o3, "%4d %14.6f %14.6f\n", i, fpointx, fpointy );
	}

	if( static_flag)
	{
	    fprintf( o3, "prescribed displacement x: node  disp value \n");
	    for( i = 0; i < numnp; ++i )
	    {
		fprintf( o3, "%4d %14.6e\n", i,
			*(charge+Vndof*i));
	    }
	    fprintf( o3, " -10 \n"); 

	    fprintf( o3, "prescribed displacement y: node  disp value \n");
	    for( i = 0; i < numnp; ++i )
	    {
		fprintf( o3, "%4d %14.6e\n", i, *(EMmag+i));
	    }
	    fprintf( o3, " -10 \n");
	}
	if( modal_flag)
	{
	    fprintf( o3, "prescribed displacement x: node  disp value \n");
	    for( i = 0; i < numnp; ++i )
	    {
		fprintf( o3, "%4d %14.6e\n", i,
			*(EMnode+ndof*i));
	    }
	    fprintf( o3, " -10 \n");

	    fprintf( o3, "prescribed displacement y: node  disp value \n");
	    for( i = 0; i < numnp; ++i )
	    {
		fprintf( o3, "%4d %14.6e\n", i,
			*(EMnode+ndof*i+1));
	    }
	    fprintf( o3, " -10 \n");
	}

	fprintf( o3, "node with point load and load vector in x,y \n");

#if 0
	if( static_flag)
	{
	    for( i = 0; i < bc.num_charge[0] ; ++i )
	    {
		node = bc.charge[i];
		*(id+ndof*node) = -1;
		*(id+ndof*node+1) = -1;
	    }
	}

	for( i = 0; i < numnp; ++i )
	{
		if( *(id+ndof*i) < 0 || *(id+ndof*i+1) < 0 )
		{
			fprintf( o3,"%4d",i);
			fprintf( o3," %16.4e  %16.4e ",
				*(charge+Vndof*i), *(charge+Vndof*i+1));
			fprintf( o3, "\n");
		}
	}
	fprintf( o3, " -10\n");
#endif

	for( i = 0; i < numnp; ++i )
	{
	    {
		fprintf( o3,"%4d",i);
		fprintf( o3," %16.4e  %16.4e ",
		    *(EMnode+ndof*i), *(EMnode+ndof*i+1));
		fprintf( o3, "\n");
	    }
	}
	fprintf( o3, " -10\n");

	return 1;
}

