/*
   This program draws the prescribed Voltage nodes and EMnode vectors
   for 2-D elements that have no rotational degrees of freedom.
 
                  Last Update 1/21/10

    SLFCEM source file
    Version:  1.1
    Copyright (C) 1999-2010  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
 */

#if WINDOWS
#include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#if QUADE
#include "../quad/quad_em/qdeconst.h"
#include "../quad/quad_em/qdestruct.h"
#endif
#if TRIE
#include "../tri/tri_em/treconst.h"
#include "../tri/tri_em/trestruct.h"
#endif

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

extern GLfloat yellow[4], orange[4], orangeRed[4], red[4], green[4], 
	violetRed[4], magenta[4], purple[4], blue[4],
	white[4], grey[4], black[4], brown[4];
extern double AxisLength_max;
extern int numnp;

void Volt_bc_node02D(int displaylistnum, BOUND bc, double *coord0)
{

/* draws prescribed Voltage nodes on undefromed configuration */

  int i,j, dum;
  double fdum, fpointx, fpointy, fpointz;
  GLfloat axes_ambuse[] =   { 0.5, 0.0, 0.0, 1.0 };
  fdum = AxisLength_max;
  glNewList(displaylistnum, GL_COMPILE);
  glPushAttrib(GL_LIGHTING_BIT);
    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, axes_ambuse);
    glPointSize(8);
    glBegin(GL_POINTS);
	glColor4fv(black);
	for( i = 0; i < bc.num_Volt[0]; ++i)
	{
		fpointx = *(coord0+nsd*bc.Volt[i]);
		fpointy = *(coord0+nsd*bc.Volt[i] + 1);
		fpointz = 0.0;
		glVertex3f( fpointx, fpointy, fpointz);
	}
    glEnd();
  glPopAttrib();
  glEndList();
}

void EMnode_vectors02D(int displaylistnum, BOUND bc, double *coord0,
	XYF *EMnode_vec )
{

/* draws EMnode vector on undefromed configuration */

  int i,j, dum;
  double fpointx, fpointy, fpointz, fx, fy, fz;
  GLfloat axes_ambuse[] =   { 0.5, 0.0, 0.0, 1.0 };
  glNewList(displaylistnum, GL_COMPILE);
  glPushAttrib(GL_LIGHTING_BIT);
    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, axes_ambuse);
    glLineWidth (2.0);
    glBegin(GL_LINES);
/* Draw the EMnode vectors axis */
	glColor4fv(white);
	for( i = 0; i < numnp; ++i)
	{
		fx = EMnode_vec[i].x; fy = EMnode_vec[i].y;
			fz = 0.0;
		fpointx = *(coord0+nsd*i);
		fpointy = *(coord0+nsd*i + 1);
		fpointz = 0.0;
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fx, fy, fz);
	}
    glEnd();
    glPointSize(2);
    glBegin(GL_POINTS);
	glColor4fv(red);
	for( i = 0; i < numnp; ++i)
	{
		fpointx = *(coord0+nsd*i);
		fpointy = *(coord0+nsd*i + 1);
		fpointz = 0.0;
		glVertex3f( fpointx, fpointy, fpointz);
	}
    glEnd();
  glPopAttrib();
  glEndList();
}

void EM_bc_edge02D( int displaylistnum, BOUND bc, double *coord0)
{

/* draws boundary Electric Field edges on undeformed configuration */

  int i,j, dum, node0, node1;
  double fpointx0, fpointy0, fpointz, fpointx1, fpointy1;
  fpointz = 0.0;
  GLfloat axes_ambuse[] =   { 0.5, 0.0, 0.0, 1.0 };
  glNewList(displaylistnum, GL_COMPILE);
  glPushAttrib(GL_LIGHTING_BIT);
    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, axes_ambuse);
    glLineWidth (4.0);
    glBegin(GL_LINES);
        glColor4fv(black);
/* Draw the EMedge lines */
        for( i = 0; i < bc.num_edge[0]; ++i)
        {
                node0 = bc.edge_node[nped*i];
                node1 = bc.edge_node[nped*i + 1];
                fpointx0 = *(coord0+nsd*node0);
                fpointy0 = *(coord0+nsd*node0 + 1);
                fpointx1 = *(coord0+nsd*node1);
                fpointy1 = *(coord0+nsd*node1 + 1);
                glVertex3f( fpointx0, fpointy0, fpointz);
                glVertex3f( fpointx1, fpointy1, fpointz);
        }
    glEnd();
  glPopAttrib();
  glEndList();
}

void Volt_bc_node2D(BOUND bc, double *coord)
{

/* draws prescribed Voltage nodes vector on deformed configuration */

  int i,j, dum;
  double fdum, fpointx, fpointy, fpointz;
  GLfloat axes_ambuse[] =   { 0.5, 0.0, 0.0, 1.0 };
  fdum = AxisLength_max;
    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, axes_ambuse);
    glPointSize(8);
    glBegin(GL_POINTS);
	glColor4fv(black);
	for( i = 0; i < bc.num_Volt[0]; ++i)
	{
		fpointx = *(coord+nsd*bc.Volt[i]);
		fpointy = *(coord+nsd*bc.Volt[i] + 1);
		fpointz = 0.0;
		glVertex3f( fpointx, fpointy, fpointz);
	}
    glEnd();
}

void EMnode_vectors2D(BOUND bc, double *coord, XYF *EMnode_vec )
{

/* draws EMnode vector on deformed configuration */

  int i,j, dum;
  double fpointx, fpointy, fpointz, fx, fy, fz;
  GLfloat axes_ambuse[] =   { 0.5, 0.0, 0.0, 1.0 };
    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, axes_ambuse);
    glLineWidth (2.0);
    glBegin(GL_LINES);
/* Draw the EMnode vectors axis */
	glColor4fv(white);
	for( i = 0; i < numnp; ++i)
	{
		fx = EMnode_vec[i].x; fy = EMnode_vec[i].y;
			fz = 0.0;
		fpointx = *(coord+nsd*i);
		fpointy = *(coord+nsd*i + 1);
		fpointz = 0.0;
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fx, fy, fz);
	}
    glEnd();
    glPointSize(2);
    glBegin(GL_POINTS);
	glColor4fv(red);
	for( i = 0; i < numnp; ++i)
	{
		fpointx = *(coord+nsd*i);
		fpointy = *(coord+nsd*i + 1);
		fpointz = 0.0;
		glVertex3f( fpointx, fpointy, fpointz);
	}
    glEnd();
}

void EM_bc_edge2D(BOUND bc, double *coord)
{

/* draws boundary Electric Field edges on deformed configuration */

  int i,j, dum, node0, node1;
  double fpointx0, fpointy0, fpointz, fpointx1, fpointy1;
  fpointz = 0.0;
  GLfloat axes_ambuse[] =   { 0.5, 0.0, 0.0, 1.0 };

    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, axes_ambuse);
    glLineWidth (4.0);
    glBegin(GL_LINES);
        glColor4fv(black);
/* Draw the EMedge lines */
        for( i = 0; i < bc.num_edge[0]; ++i)
        {
                node0 = bc.edge_node[nped*i];
                node1 = bc.edge_node[nped*i + 1];
                fpointx0 = *(coord+nsd*node0);
                fpointy0 = *(coord+nsd*node0 + 1);
                fpointx1 = *(coord+nsd*node1);
                fpointy1 = *(coord+nsd*node1 + 1);
                glVertex3f( fpointx0, fpointy0, fpointz);
                glVertex3f( fpointx1, fpointy1, fpointz);
        }
    glEnd();
}

