/*
    This program contains the control mouse routine for the FEM GUI
    for triangle elements.
  
                  Last Update 1/17/10

    SLFCEM source file
    Version:  1.1
    Copyright (C) 1999-2010  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
 */

#if WINDOWS
#include <windows.h>
#endif

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#if QUADE
#include "../quad/quad_em/qdeconst.h"
#include "../quad/quad_em/qdestruct.h"
#include "../quad/quadegr/qdestrcgr.h"
#endif
#if TRIE
#include "../tri/tri_em/treconst.h"
#include "../tri/tri_em/trestruct.h"
#include "../tri/triegr/trestrcgr.h"
#endif

#include "control.h"

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

/****** FEA globals ******/

extern int dof, nmat, numnp, numel;
extern int modal_flag, static_flag;
extern double *coord, *coord0;
extern double *EMnode;
extern int *connecter;
extern int *edge_connect, *node_edge_connect;
extern BOUND bc;
extern XYF *EMnode_vec, *EMnode_vec0;

/****** For drawing the Mesh Window ******/
extern double coord_rescale;

/* Global variables for the mesh color and nodal data */

extern int *el_matl_color;
extern MATL *matl_crtl;

/****** EXTERNAL VARIABLES ********/

extern NORM *norm, *norm0;

extern int ControlDiv_y[rowdim + 2], ControlDiv_x[rowdim + 2];
extern int boxMove_x, boxMove_y, boxTextMove_x, textMove_x, textMove_y[rowdim+2];
extern int textDiv_xa, textDiv_xb;
extern double matl_choicef, node_choicef, ele_choicef;

extern int ControlWindow, MeshWindow;
extern double step_sizex, step_sizey, step_sizez;
extern int control_height, control_width, mesh_height, mesh_width;
extern double xAngle, yAngle, zAngle;
extern double amplify_factor, amplify_step, amplify_step0;
extern double left_right, up_down, in_out;
extern double left_right0, up_down0, in_out0;
extern double ortho_left, ortho_right, ortho_top, ortho_bottom,
	ortho_left0, ortho_right0, ortho_top0, ortho_bottom0;
extern int ortho_redraw_flag;

extern int input_flag, post_flag, color_choice, matl_choice, node_choice, ele_choice;
extern int input_color_flag;
extern int Solid_flag, Perspective_flag, Render_flag, EMedgeColor_flag,
	EMfieldVec_flag, Material_flag, Node_flag, Element_flag,
	Axes_flag, Outline_flag, Transparent_flag;
extern int Before_flag, After_flag, Both_flag, Amplify_flag; 
extern int Boundary_flag;
extern int node_edge_quantities_flag;

extern GLfloat yellow[4], orange[4], orangeRed[4], red[4], green[4],
	violetRed[4], magenta[4], purple[4], blue[4],
	white[4], grey[4], black[4];

extern char RotateData[3][25];
extern char MoveData[3][25];
extern char AmplifyData[25];
extern char BoxData[2*boxnumber+2][25];
extern char BoxText[25];

extern int Color_flag[rowdim];
extern double EMnodex_div[boxnumber+1], EMnodey_div[boxnumber+1], EMmag_div[boxnumber+1],
	EMedge_div[boxnumber+1], charge_div[boxnumber+1], Volt_div[boxnumber+1];

void ScreenShot( int , int );

void EControlMouse(int button, int state, int x, int y)
{
	int i, check, dum1, dum2;
	double fpointx, fpointy, fpointz;

	memset(BoxText,0,25*sizeof(char));
	for( i = 0; i < 2*boxnumber+2; ++i)
	{
		memset(BoxData[i],0,25*sizeof(char));
	}

	if (button == GLUT_LEFT_BUTTON)
	{
		if ( x < textDiv_xa )
		{

/* These are for the View Option Keys */

			if ( y >= ControlDiv_y[3] && y < ControlDiv_y[4] )
			{
/* Solid Turned On */
				Solid_flag = 1;
			}
			if ( y >= ControlDiv_y[4] && y < ControlDiv_y[5] )
			{
/* Node ID Turned On increment up */
				EMfieldVec_flag = 0;
				Boundary_flag = 0;
				EMedgeColor_flag = 0;
				Element_flag = 0;
				Material_flag = 0;
				Node_flag = 1;
				node_choicef += .5;
				if ( node_choicef > 1.0 )
				{
					node_choicef = 0.0;
				}
				node_choice += (int)node_choicef;
				if ( node_choice > numnp - 1 )
				{
					node_choice = 0;
				}
				color_choice = 31;
				if ( Both_flag )
				{
					After_flag = 0;
					Before_flag = 1;
					Both_flag = 0;
				}
			}
			if ( y >= ControlDiv_y[5] && y < ControlDiv_y[6] )
			{
/* Element ID Turned On increment up */
				/*ScreenShot( control_width, control_height );*/

				EMfieldVec_flag = 0;
				Boundary_flag = 0;
				EMedgeColor_flag = 0;
				Element_flag = 1;
				Material_flag = 0;
				Node_flag = 0;
				ele_choicef += .5;
				if ( ele_choicef > 1.0 )
				{
					ele_choicef = 0.0;
				}
				ele_choice += (int)ele_choicef;
				if ( ele_choice > numel - 1 )
				{
					ele_choice = 0;
				}
				color_choice = 32;
				Solid_flag = 1;
				if ( Both_flag )
				{
					After_flag = 0;
					Before_flag = 1;
					Both_flag = 0;
				}
			}
			if ( y >= ControlDiv_y[6] && y < ControlDiv_y[7] )
			{
/* Material ID Turned On increment up */
				EMfieldVec_flag = 0;
				Boundary_flag = 0;
				EMedgeColor_flag = 0;
				Element_flag = 0;
				Node_flag = 0;
				Material_flag = 1;
				matl_choicef += .5;
				if ( matl_choicef > 1.0 )
				{
					matl_choicef = 0.0;
				}
				matl_choice += (int)matl_choicef;
				if ( matl_choice > nmat - 1 )
				{
					matl_choice = 0;
				}
				color_choice = 30;
				Solid_flag = 1;
				if ( Both_flag )
				{
					After_flag = 0;
					Before_flag = 1;
					Both_flag = 0;
				}
			}
			if ( y >= ControlDiv_y[7] && y < ControlDiv_y[8] )
			{
/* EMedge bounary edges Turned On */
				/*EMfieldVec_flag = 0;*/
				/*EMedgeColor_flag = 1;*/
				Boundary_flag = 1;
				Element_flag = 0;
				Material_flag = 0;
				Node_flag = 0;
				Outline_flag = 0;
				/*if ( Both_flag )
				{
					After_flag = 0;
					Before_flag = 1;
					Both_flag = 0;
				}*/
			}
			if ( y >= ControlDiv_y[8] && y < ControlDiv_y[9] )
			{
/* Electric Field Vectors Turned On */
				EMfieldVec_flag = 1;
				/*EMedgeColor_flag = 0;*/
				Element_flag = 0;
				Material_flag = 0;
				Node_flag = 0;
				/*if ( Both_flag )
				{
					After_flag = 0;
					Before_flag = 1;
					Both_flag = 0;
				}*/
			}
			if ( y >= ControlDiv_y[9] && y < ControlDiv_y[10] )
			{
/* Axes Turned On */
				Axes_flag = 1;
			}
/* These are for the Rotation Keys */
			if ( y >= ControlDiv_y[12] && y < ControlDiv_y[13] )
			{
/* Rotate -x */
				xAngle -= 5.0;
			}
			if ( y >= ControlDiv_y[13] && y < ControlDiv_y[14] )
			{
/* Rotate -y */
				yAngle -= 5.0;
			}
			if ( y >= ControlDiv_y[14] && y < ControlDiv_y[15] )
			{
/* Rotate -z */
				zAngle -= 5.0;
			}
			if ( y >= ControlDiv_y[15] && y < ControlDiv_y[16] )
			{
/* Reset Angles */
				xAngle = 0.0; yAngle = 0.0; zAngle = 0.0;
			}
/* These are for the Move Keys */
			if ( y >= ControlDiv_y[17] && y < ControlDiv_y[18] )
			{
/* Move -x */
				left_right -= step_sizex;
			}
			if ( y >= ControlDiv_y[18] && y < ControlDiv_y[19] )
			{
/* Move -y */
				up_down -= step_sizey;
			}
			if ( y >= ControlDiv_y[19] && y < ControlDiv_y[20] )
			{
/* Move -z */
				in_out -= step_sizez;
			}
			if ( y >= ControlDiv_y[20] && y < ControlDiv_y[21] )
			{
/* Reset Position */
				left_right = left_right0;
				up_down = up_down0;
				in_out = in_out0;

				ortho_right = ortho_right0;
				ortho_left = ortho_left0;
				ortho_top = ortho_top0;
				ortho_bottom = ortho_bottom0;
			}
/* These are for the Deformation Keys */
			if ( y >= ControlDiv_y[24] && y < ControlDiv_y[25] )
			{
/* Before Turned On */
				After_flag = 0;
				/*amplify_factor = 1.0;
				Amplify_flag = 0;*/
				Before_flag = 1;
				Both_flag = 0;
				/*Boundary_flag = 0;*/
			}
			if ( y >= ControlDiv_y[25] && y < ControlDiv_y[26] )
			{
/* After Turned On */
				After_flag = 1;
				Before_flag = 0;
				Both_flag = 0;
			}

			if ( y >= ControlDiv_y[26] && y < ControlDiv_y[27] )
			{
/* Both Before and After Turned On */
				After_flag = 1;
				/*EMfieldVec_flag = 0;*/
				Both_flag = 1;
				Before_flag = 1;
				/*Boundary_flag = 0;
				EMedgeColor_flag = 0;*/
				/*Material_flag = 0;
				Solid_flag = 0;*/
			}
#if 0
			if ( y >= ControlDiv_y[27] && y < ControlDiv_y[28] )
			{
/* Amplification increased */
			    if ( post_flag )
			    {
				After_flag = 1;
				amplify_step = amplify_step0;
				if( amplify_factor < 1.0 - SMALL2 )
					amplify_step = .1;
				amplify_factor += amplify_step;
				Amplify_flag = 1;
				/*EMfieldVec_flag = 0;
				EMedgeColor_flag = 0;*/
			    }
			}
#endif

/* These are for the Engineering Analysis Option Keys */
			if ( y >= ControlDiv_y[30] )
			{
/* Strains Turned On */
				if( post_flag)
				{
				   After_flag = 1;
				   Before_flag = 0;
				}
				Both_flag = 0;
				Element_flag = 0;
				Material_flag = 0;
				Node_flag = 0;
				Solid_flag = 1;
				Transparent_flag = 0;
/* Node and Edge Quantities */

/* Charge */
				node_edge_quantities_flag = 1;
				color_choice = 1;
			}
			if ( y >= ControlDiv_y[31] )
			{
/* Voltage */
				color_choice = 2;
			}
			if ( y >= ControlDiv_y[32] )
			{
/* Electric Field Magnitude */
				color_choice = 3;
			}
			if ( y >= ControlDiv_y[33] )
			{
/* Electric Field Edge */
				Before_flag = 1;
				if( post_flag)
				{
				   After_flag = 1;
				   Before_flag = 0;
				}
				Boundary_flag = 0;
				color_choice = 4;
			}
			if ( y >= ControlDiv_y[34] )
			{
/* Electric Field X */
				Before_flag = 1;
				if( post_flag)
				{
				   After_flag = 1;
				   Before_flag = 0;
				}
				Both_flag = 0;
				Element_flag = 0;
				Material_flag = 0;
				Node_flag = 0;
				Solid_flag = 1;

				color_choice = 5;
			}
			if ( y >= ControlDiv_y[35] )
			{
/* Electric Field Y */
				color_choice = 6;
			}
#if 0
			if ( y >= ControlDiv_y[36] )
			{
/* Electric Field Z */
				color_choice = 7;
			}
#endif
		}


		if ( x >= textDiv_xa && x < textDiv_xb )
		{
			if ( y >= ControlDiv_y[3] && y <= ControlDiv_y[4] )
			{
/* Solid Turned Off */
				Boundary_flag = 0;
				Element_flag = 0;
				Material_flag = 0;
				Node_flag = 0;
				Solid_flag = 0;
				Outline_flag = 1;
			}
			if ( y >= ControlDiv_y[4] && y < ControlDiv_y[5] )
			{
/* Node ID Turned On increment down */
				EMfieldVec_flag = 0;
				Boundary_flag = 0;
				EMedgeColor_flag = 0;
				Element_flag = 0;
				Material_flag = 0;
				Node_flag = 1;
				node_choicef += .5;
				if ( node_choicef > 1.0 )
				{
					node_choicef = 0.0;
				}
				node_choice -= (int)node_choicef;
				if ( node_choice < 0 )
				{
					node_choice = numnp-1;
				}
				color_choice = 31;
				if ( Both_flag )
				{
					After_flag = 0;
					Before_flag = 1;
					Both_flag = 0;
				}
			}
			if ( y >= ControlDiv_y[5] && y < ControlDiv_y[6] )
			{
/* Element ID Turned On increment down */
				/*ScreenShot( 350, 700);*/

				EMfieldVec_flag = 0;
				Boundary_flag = 0;
				EMedgeColor_flag = 0;
				Element_flag = 1;
				Material_flag = 0;
				Node_flag = 0;
				ele_choicef += .5;
				if ( ele_choicef > 1.0 )
				{
					ele_choicef = 0.0;
				}
				ele_choice -= (int)ele_choicef;
				if ( ele_choice < 0 )
				{
					ele_choice = numel-1;
				}
				color_choice = 32;
				Solid_flag = 1;
				if ( Both_flag )
				{
					After_flag = 0;
					Before_flag = 1;
					Both_flag = 0;
				}
			}
			if ( y >= ControlDiv_y[6] && y < ControlDiv_y[7] )
			{
/* Material ID Turned On increment down */
				EMfieldVec_flag = 0;
				Boundary_flag = 0;
				EMedgeColor_flag = 0;
				Element_flag = 0;
				Node_flag = 0;
				Material_flag = 1;
				matl_choicef += .5;
				if ( matl_choicef > 1.0 )
				{
					matl_choicef = 0.0;
				}
				matl_choice -= (int)matl_choicef;
				if ( matl_choice < 0 )
				{
					matl_choice = nmat-1;
				}
				color_choice = 30;
				Solid_flag = 1;
				if ( Both_flag )
				{
					After_flag = 0;
					Before_flag = 1;
					Both_flag = 0;
				}
			}
			if ( y >= ControlDiv_y[7] && y <= ControlDiv_y[8] )
			{
/* Boundary Voltage nodes or boundary Electric Field edges Turned Off */
				Boundary_flag = 0;
			}
			if ( y >= ControlDiv_y[8] && y < ControlDiv_y[9] )
			{
/* Electric Field Vectors Turned Off */
				EMfieldVec_flag = 0;
			}
			if ( y >= ControlDiv_y[9] && y < ControlDiv_y[10] )
			{
/* Axes Turned Off */
				Axes_flag = 0;
			}
/* These are for the Rotation Keys */
			if ( y >= ControlDiv_y[12] && y < ControlDiv_y[13] )
			{
/* Rotate +x */
				xAngle += 5.0;
			}
			if ( y >= ControlDiv_y[13] && y < ControlDiv_y[14] )
			{
/* Rotate +y */
				yAngle += 5.0;
			}
			if ( y >= ControlDiv_y[14] && y < ControlDiv_y[15] )
			{
/* Rotate +z */
				zAngle += 5.0;
			}
/* These are for the Move Keys */
			if ( y >= ControlDiv_y[17] && y < ControlDiv_y[18] )
			{
/* Move +x */
				left_right += step_sizex;
			}
			if ( y >= ControlDiv_y[18] && y < ControlDiv_y[19] )
			{
/* Move +y */
				up_down += step_sizey;
			}
			if ( y >= ControlDiv_y[19] && y < ControlDiv_y[20] )
			{
/* Move +z */
				in_out += step_sizez;
			}

#if 0
/* These are for the Deformation Keys */
			if ( y >= ControlDiv_y[27] && y < ControlDiv_y[28] )
			{
/* Amplification decreased */
			    if( post_flag )
			    {
				After_flag = 1;
				amplify_step = amplify_step0;
				if( amplify_factor < 1.0 + amplify_step0 - SMALL2 )
					amplify_step = .1;
				amplify_factor -= amplify_step;
				/*Amplify_flag = 1;*/
			    }
			}
#endif

/* The lines below are redundant and may not be needed.  I keep them here in case
   there are more quantities to look at
*/

/* These are for the Engineering Analysis Option Keys */
			if ( y >= ControlDiv_y[30] )
			{
				if( post_flag)
				{
				   After_flag = 1;
				   Before_flag = 0;
				}
				Both_flag = 0;
				Element_flag = 0;
				Material_flag = 0;
				Node_flag = 0;
				Solid_flag = 1;
				Transparent_flag = 0;
/* Node and Edge Quantities */

/* Charge */
				node_edge_quantities_flag = 1;
				color_choice = 1;
			}
			if ( y >= ControlDiv_y[31] )
			{
/* Voltage */
				color_choice = 2;
			}
			if ( y >= ControlDiv_y[32] )
			{
/* Electric Field Magnitude */
				color_choice = 3;
			}
			if ( y >= ControlDiv_y[33] )
			{
/* Electric Field Edge */
				Before_flag = 1;
				if( post_flag)
				{
				   After_flag = 1;
				   Before_flag = 0;
				}
				Boundary_flag = 0;
				color_choice = 4;
			}
			if ( y >= ControlDiv_y[34] )
			{
				Before_flag = 1;
				if( post_flag)
				{
				   After_flag = 1;
				   Before_flag = 0;
				}
				Both_flag = 0;
				Element_flag = 0;
				Material_flag = 0;
				Node_flag = 0;
				Solid_flag = 1;
/* Electric Field X */
				color_choice = 5;
			}
			if ( y >= ControlDiv_y[35] )
			{
/* Electric Field Y */
				color_choice = 6;
			}
#if 0
			if ( y >= ControlDiv_y[36] )
			{
/* Electric Field Z */
				color_choice = 7;
			}
#endif
		}
	}

	sprintf( RotateData[0], "%8.2f ", xAngle);
	sprintf( RotateData[1], "%8.2f ", yAngle);
	sprintf( RotateData[2], "%8.2f ", zAngle);

	sprintf( MoveData[0], "%8.2f ", left_right);
	sprintf( MoveData[1], "%8.2f ", up_down);
	sprintf( MoveData[2], "%8.2f ", in_out);

	sprintf( AmplifyData, "%10.3e ", amplify_factor);

	for(i = 29; i < rowdim ; ++i)
	{
		Color_flag[i] = 0;
	}

	switch (color_choice) {
		case 1:
			strncpy(BoxText, "charge", 6);
			Color_flag[29] = 1;
                        node_edge_quantities_flag = 1;
			sprintf( BoxData[0], "%10.3e ", charge_div[8]);
			sprintf( BoxData[2], "%10.3e ", charge_div[7]);
			sprintf( BoxData[4], "%10.3e ", charge_div[6]);
			sprintf( BoxData[6], "%10.3e ", charge_div[5]);
			sprintf( BoxData[8], "%10.3e ", charge_div[4]);
			sprintf( BoxData[10], "%10.3e ", charge_div[3]);
			sprintf( BoxData[12], "%10.3e ", charge_div[2]);
			sprintf( BoxData[14], "%10.3e ", charge_div[1]);
			sprintf( BoxData[16], "%10.3e ", charge_div[0]);
		break;
		case 2:
			strncpy(BoxText, "Voltage", 7);
			Color_flag[30] = 1;
			sprintf( BoxData[0], "%10.3e ", Volt_div[8]);
			sprintf( BoxData[2], "%10.3e ", Volt_div[7]);
			sprintf( BoxData[4], "%10.3e ", Volt_div[6]);
			sprintf( BoxData[6], "%10.3e ", Volt_div[5]);
			sprintf( BoxData[8], "%10.3e ", Volt_div[4]);
			sprintf( BoxData[10], "%10.3e ", Volt_div[3]);
			sprintf( BoxData[12], "%10.3e ", Volt_div[2]);
			sprintf( BoxData[14], "%10.3e ", Volt_div[1]);
			sprintf( BoxData[16], "%10.3e ", Volt_div[0]);
		break;
		case 3:
			strncpy(BoxText, "Ele. Field Mag", 14);
			Color_flag[31] = 1;
			sprintf( BoxData[0], "%10.3e ", EMmag_div[8]);
			sprintf( BoxData[2], "%10.3e ", EMmag_div[7]);
			sprintf( BoxData[4], "%10.3e ", EMmag_div[6]);
			sprintf( BoxData[6], "%10.3e ", EMmag_div[5]);
			sprintf( BoxData[8], "%10.3e ", EMmag_div[4]);
			sprintf( BoxData[10], "%10.3e ", EMmag_div[3]);
			sprintf( BoxData[12], "%10.3e ", EMmag_div[2]);
			sprintf( BoxData[14], "%10.3e ", EMmag_div[1]);
			sprintf( BoxData[16], "%10.3e ", EMmag_div[0]);
		break;
		case 4:
			strncpy(BoxText, "Ele. Field Edge", 15);
			Color_flag[32] = 1;
			sprintf( BoxData[0], "%10.3e ", EMedge_div[8]);
			sprintf( BoxData[2], "%10.3e ", EMedge_div[7]);
			sprintf( BoxData[4], "%10.3e ", EMedge_div[6]);
			sprintf( BoxData[6], "%10.3e ", EMedge_div[5]);
			sprintf( BoxData[8], "%10.3e ", EMedge_div[4]);
			sprintf( BoxData[10], "%10.3e ", EMedge_div[3]);
			sprintf( BoxData[12], "%10.3e ", EMedge_div[2]);
			sprintf( BoxData[14], "%10.3e ", EMedge_div[1]);
			sprintf( BoxData[16], "%10.3e ", EMedge_div[0]);
		break;
		case 5:
			strncpy(BoxText, "Ele. Field x", 12);
			Color_flag[33] = 1;
			sprintf( BoxData[0], "%10.3e ", EMnodex_div[8]*coord_rescale);
			sprintf( BoxData[2], "%10.3e ", EMnodex_div[7]*coord_rescale);
			sprintf( BoxData[4], "%10.3e ", EMnodex_div[6]*coord_rescale);
			sprintf( BoxData[6], "%10.3e ", EMnodex_div[5]*coord_rescale);
			sprintf( BoxData[8], "%10.3e ", EMnodex_div[4]*coord_rescale);
			sprintf( BoxData[10], "%10.3e ", EMnodex_div[3]*coord_rescale);
			sprintf( BoxData[12], "%10.3e ", EMnodex_div[2]*coord_rescale);
			sprintf( BoxData[14], "%10.3e ", EMnodex_div[1]*coord_rescale);
			sprintf( BoxData[16], "%10.3e ", EMnodex_div[0]*coord_rescale);
		break;
		case 6:
			strncpy(BoxText, "Ele. Field y", 12);
			Color_flag[34] = 1;
			sprintf( BoxData[0], "%10.3e ", EMnodey_div[8]*coord_rescale);
			sprintf( BoxData[2], "%10.3e ", EMnodey_div[7]*coord_rescale);
			sprintf( BoxData[4], "%10.3e ", EMnodey_div[6]*coord_rescale);
			sprintf( BoxData[6], "%10.3e ", EMnodey_div[5]*coord_rescale);
			sprintf( BoxData[8], "%10.3e ", EMnodey_div[4]*coord_rescale);
			sprintf( BoxData[10], "%10.3e ", EMnodey_div[3]*coord_rescale);
			sprintf( BoxData[12], "%10.3e ", EMnodey_div[2]*coord_rescale);
			sprintf( BoxData[14], "%10.3e ", EMnodey_div[1]*coord_rescale);
			sprintf( BoxData[16], "%10.3e ", EMnodey_div[0]*coord_rescale);
		break;
#if 0
		case 7:
			strncpy(BoxText, "Ele. Field z", 12);
			Color_flag[35] = 1;
			sprintf( BoxData[0], "%10.3e ", EMnodez_div[8]*coord_rescale);
			sprintf( BoxData[2], "%10.3e ", EMnodez_div[7]*coord_rescale);
			sprintf( BoxData[4], "%10.3e ", EMnodez_div[6]*coord_rescale);
			sprintf( BoxData[6], "%10.3e ", EMnodez_div[5]*coord_rescale);
			sprintf( BoxData[8], "%10.3e ", EMnodez_div[4]*coord_rescale);
			sprintf( BoxData[10], "%10.3e ", EMnodez_div[3]*coord_rescale);
			sprintf( BoxData[12], "%10.3e ", EMnodez_div[2]*coord_rescale);
			sprintf( BoxData[14], "%10.3e ", EMnodez_div[1]*coord_rescale);
			sprintf( BoxData[16], "%10.3e ", EMnodez_div[0]*coord_rescale);
		break;
#endif
		case 30:
			strncpy(BoxText, "Material", 8);
			Color_flag[5] = 1;
			input_color_flag = 0;
			Boundary_flag = 0;
			Element_flag = 0;
			Node_flag = 0;
			Material_flag = 1;
			sprintf( BoxData[0], "%4d ", matl_choice);
			strncpy( BoxData[1], "Permit.", 7);
			sprintf( BoxData[2], "%10.3e ", matl_crtl[matl_choice].er);
			strncpy( BoxData[3], "Permea.", 7);
			sprintf( BoxData[4], "%10.3e ", matl_crtl[matl_choice].ur);
			strncpy( BoxData[5], "Wave No.", 8);
			sprintf( BoxData[6], "%10.3e ", matl_crtl[matl_choice].ko);
			sprintf( BoxData[7], " ");
			sprintf( BoxData[8], " ");
			sprintf( BoxData[10], " " );
			sprintf( BoxData[12], " " );
			sprintf( BoxData[14], " " );
			sprintf( BoxData[16], " " );
		break;
		case 31:
			strncpy(BoxText, "Node", 4);
			Color_flag[3] = 1;
			input_color_flag = 0;
			Boundary_flag = 0;
			Element_flag = 0;
			Material_flag = 0;
			Node_flag = 1;
			fpointx = *(coord + nsd*node_choice)*coord_rescale;
			fpointy = *(coord + nsd*node_choice + 1)*coord_rescale;
			fpointz = 0.0;
			if(!After_flag)
			{
				fpointx = *(coord0 + nsd*node_choice)*coord_rescale;
				fpointy = *(coord0 + nsd*node_choice + 1)*coord_rescale;
				fpointz = 0.0;
			}
			sprintf( BoxData[0], "%4d ", node_choice);
			strncpy( BoxData[2], "coord x", 7);
			sprintf( BoxData[4], "%10.3e ", fpointx);
			strncpy( BoxData[6], "coord y", 7);
			sprintf( BoxData[8], "%10.3e ", fpointy);
			strncpy( BoxData[10], "coord z", 7);
			sprintf( BoxData[12], "%10.3e ", fpointz);
			sprintf( BoxData[14], " " );
			sprintf( BoxData[16], " " );
		break;
		case 32:
			strncpy(BoxText, "Element", 7);
			Color_flag[4] = 1;
			input_color_flag = 0;
			Boundary_flag = 0;
			Material_flag = 0;
			Node_flag = 0;
			Element_flag = 1;
			sprintf( BoxData[0], "%4d ", ele_choice);
			strncpy( BoxData[2], "Material", 8);
			sprintf( BoxData[4], "%4d ", *(el_matl_color+ele_choice));
			strncpy( BoxData[6], "Connect", 7);
			dum1 = *(connecter + npel*ele_choice);
			dum2 = *(connecter + npel*ele_choice+1);
			sprintf( BoxData[8], "%4d,%4d ",dum1, dum2);
#if QUADE
			dum1 = *(connecter + npel*ele_choice+2);
			dum2 = *(connecter + npel*ele_choice+3);
			sprintf( BoxData[10], "%4d,%4d ",dum1, dum2);
#endif
#if TRIE
			dum1 = *(connecter + npel*ele_choice+2);
			sprintf( BoxData[10], "%4d ",dum1);
#endif
			sprintf( BoxData[12], " " );
			sprintf( BoxData[14], " " );
			sprintf( BoxData[16], " " );
			if(modal_flag)
			{
			    strncpy( BoxData[12], "Ed Connect", 10);
			    dum1 = *(edge_connect + epel*ele_choice);
			    dum2 = *(edge_connect + epel*ele_choice+1);
			    sprintf( BoxData[14], "%4d,%4d ",dum1, dum2);
#if QUADE
			    dum1 = *(edge_connect + epel*ele_choice+2);
			    dum2 = *(edge_connect + epel*ele_choice+3);
			    sprintf( BoxData[16], "%4d,%4d ",dum1, dum2);
#endif
#if TRIE
			    dum1 = *(edge_connect + epel*ele_choice+2);
			    sprintf( BoxData[16], "%4d ",dum1);
#endif
			}

		break;
	}

	input_color_flag = 0;
	if( EMfieldVec_flag )
	{
		if( color_choice > 29) input_color_flag = 1;
		Element_flag = 0;
		Material_flag = 0;
		Node_flag = 0;
	}

	Outline_flag = 1;
	if(color_choice == 4)
	{
		EMedgeColor_flag = 1;
		Transparent_flag = 1;
		Outline_flag = 0;
	}

	Color_flag[2] = Solid_flag;
/*
	Color_flag[3] = Perspective_flag;
	Color_flag[4] = Render_flag;
*/
	Color_flag[3] = Node_flag;
	Color_flag[4] = Element_flag;
	Color_flag[5] = Material_flag;
	Color_flag[6] = Boundary_flag;
	Color_flag[7] = EMfieldVec_flag;
	Color_flag[8] = Axes_flag;

	Color_flag[23] = Before_flag;
	Color_flag[24] = After_flag;
	Color_flag[25] = Both_flag;
	Color_flag[26] = Amplify_flag;

	if(!post_flag) After_flag = 0;
	if(!input_flag) Before_flag = 0;

	glutPostWindowRedisplay(ControlWindow);
	glutPostWindowRedisplay(MeshWindow);
}


