/*
    This program contains the mesh display routine for the FEM GUI
    for Electromagnetic elements.
  
                  Last Update 1/16/10

    SLFCEM source file
    Version:  1.1
    Copyright (C) 1999-2010  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
 */

#if WINDOWS
#include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>

#if QUADE
#include "../quad/quad_em/qdeconst.h"
#include "../quad/quad_em/qdestruct.h"
#endif
#if TRIE
#include "../tri/tri_em/treconst.h"
#include "../tri/tri_em/trestruct.h"
#endif



#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#if QUADE || QUAD2
void qdmeshdraw(void);
void qdrender(void);
#endif

#if TRIE || TRI2
void trmeshdraw(void);
void trrender(void);
#endif

void Volt_bc_node2D(BOUND , double *);
void EM_edgedraw( double *, double *, int *, int * );
void EMnode_vectors2D(BOUND , double *, XYF *);

extern int modal_flag, static_flag;
extern double *coord;
extern BOUND bc;

extern double left_right, up_down, in_out, left_right0,
	up_down0, in_out0, xAngle, yAngle, zAngle;
extern GLuint AxesList, VoltBCList, EMnodeList, EMedgeBCList;   /* Display lists */
extern XYF *EMnode_vec;
extern double *EMedge;
extern int *EMedge_color, *node_edge_connect;
extern int Render_flag, EMedgeColor_flag, EMfieldVec_flag,
    Axes_flag, Before_flag, After_flag, Boundary_flag;
extern int CrossSection_flag;

void AxesNumbers2(void);

void AxesNumbers(void);

void AxesLabel(void);

void Volt_bc_node2D(BOUND , double *);

void EMnode_vectors2D( BOUND, double *, XYF * );

void EM_bc_edge2D( BOUND , double *);

void CrossSetionPlaneDraw(void);

void EMeshDisplay(void)
{
	glClear (GL_COLOR_BUFFER_BIT| GL_DEPTH_BUFFER_BIT);

	glLoadIdentity ();  /*  clear the matrix    */

	glTranslatef (left_right, up_down, in_out);
	glRotatef (xAngle, 1, 0, 0);
	glRotatef (yAngle, 0, 1, 0);
	glRotatef (zAngle, 0, 0, 1);

	glPointSize(8);
	if(Axes_flag)
		glCallList(AxesList);
	if(Boundary_flag)
	{
	    if(static_flag)
	    {
		if(Before_flag ) glCallList(VoltBCList);
		if(After_flag ) Volt_bc_node2D(bc, coord);
	    }
	    if(modal_flag) 
	    {
		if(Before_flag ) glCallList(EMedgeBCList);
		if(After_flag ) EM_bc_edge2D( bc, coord);
	    }
	}
	if(EMedgeColor_flag)
	{
	    if(modal_flag) EM_edgedraw( coord, EMedge, EMedge_color, node_edge_connect );
	}
	if(EMfieldVec_flag)
	{
		if(Before_flag )
			glCallList(EMnodeList);
		if(After_flag )
			EMnode_vectors2D(bc, coord, EMnode_vec);
	}
	if(CrossSection_flag)
	{
		CrossSetionPlaneDraw();
	}
	glLineWidth (2.0);

#if QUADE || QUAD2
	if(Render_flag)
		qdrender();
	else
		qdmeshdraw();
#endif
#if TRIE || TRI2
	if(Render_flag)
		trrender();
	else
		trmeshdraw();
#endif

	if(Axes_flag)
	{
		AxesNumbers();
		/*AxesNumbers2();*/
	}
	glutSwapBuffers();
}

