/*
    This program contains the mesh key routine for the FEM GUI
    for triangle and quadrilateral elements.
  
                        Last Update 1/18/10

    SLFCEM source file
    Version:  1.1
    Copyright (C) 1999-2010  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
 */

#if WINDOWS
#include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>

#if QUADE
#include "../quad/quad_em/qdeconst.h"
#include "../quad/quad_em/qdestruct.h"
#include "../quad/quadegr/qdestrcgr.h"
#endif
#if TRIE
#include "../tri/tri_em/treconst.h"
#include "../tri/tri_em/trestruct.h"
#include "../tri/triegr/trestrcgr.h"
#endif


#include "control.h"

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

extern int nmat, numnp, numel, dof;
extern double *coord, *coord0;
extern double *EMedge, *EMnode, *EMmag;
extern int *connecter;
extern BOUND bc;
extern double *Volt, *charge;
extern XYF *EMnode_vec, *EMnode_vec0;
extern int *EMnode_color, *EMedge_color, *EMmag_color, *charge_color, *Volt_color;


extern GLfloat MeshColor[boxnumber+5][4];
extern double step_sizex, step_sizey, step_sizez;
extern NORM *norm;
extern double left_right, up_down, in_out, left_right0, up_down0, in_out0;
extern double cross_sec_left_right, cross_sec_up_down, cross_sec_in_out,
	cross_sec_left_right0, cross_sec_up_down0, cross_sec_in_out0;
extern double ortho_left, ortho_right, ortho_top, ortho_bottom,
	ortho_left0, ortho_right0, ortho_top0, ortho_bottom0;
extern int ortho_redraw_flag;
extern double xAngle, yAngle, zAngle;
extern int mesh_width, mesh_height;
extern int input_flag, post_flag, color_choice,
    choice, matl_choice, node_choice, ele_choice;
extern double AxisMax_x, AxisMax_y, AxisMax_z,
	AxisMin_x, AxisMin_y, AxisMin_z;
extern int input_color_flag;
extern int Solid_flag, Perspective_flag, Render_flag,
    Boundary_flag, EMedgeColor_flag, EMfieldVec_flag,
    Material_flag, Node_flag, Element_flag, Axes_flag,
    Outline_flag, Transparent_flag, CrossSection_flag;
extern int Before_flag, After_flag,
    Both_flag, Amplify_flag;
extern double amplify_factor, amplify_step, amplify_step0;

void ScreenShot( int , int );

void MeshInit(void);

void EReGetparameter(void);

int Eset( BOUND , int *, XYF *, double *, int *, double *, int *, double *,
	int *, double *, int *, double *, int *);

#if TRIE
int trGetNewMesh(void);
#endif

#if QUADE
int qdGetNewMesh(void);
#endif

void MeshReshape(int , int );

void EMeshKeys( unsigned char key, int x, int y )
{

	int i, check;
	double fpointx, fpointy, fpointz;
/* Define Hotkeys */

/* 'i' zooms in on the mesh, 'o' zooms out */

	switch (key) {
	    case 'i':
		if ( Perspective_flag )
		{
			in_out += step_sizez;
		}
		else
		{
			ortho_left *= 0.90;
			ortho_right *= 0.90;
			ortho_bottom *= 0.90;
			ortho_top *= 0.90;
			MeshReshape( glutGet(GLUT_WINDOW_WIDTH),
				glutGet(GLUT_WINDOW_HEIGHT));
		}
		break;
	    case 'o':
		if ( Perspective_flag )
		{
			in_out -= step_sizez;
		}
		else
		{
			ortho_left *= 1.10;
			ortho_right *= 1.10;
			ortho_bottom *= 1.10;
			ortho_top *= 1.10;
			MeshReshape( glutGet(GLUT_WINDOW_WIDTH),
				glutGet(GLUT_WINDOW_HEIGHT));
		}
		break;

/* These keys control the selection of viewing the Electric Field, Voltage,
   and Charge. */

	    case '1':
		Solid_flag = 1;
		color_choice = 1;
		break;
	    case '2':
		Solid_flag = 1;
		color_choice = 2;
		break;
	    case '3':
		Solid_flag = 1;
		color_choice = 3;
		break;
	    case '4':
		EMedgeColor_flag = 1 - EMedgeColor_flag;
		if(EMedgeColor_flag)
		{
		    Solid_flag = 0;
		    color_choice = 4;
		    EMedgeColor_flag = 1;
		    Transparent_flag = 1;
		    Outline_flag = 0;
		}
		break;
	    case '5':
		Solid_flag = 1;
		color_choice = 5;
		break;
	    case '6':
		Solid_flag = 1;
		color_choice = 6;
		break;

/* 'n' selects the node to be viewed.  */

	    case 'n':
		color_choice = 31;
		input_color_flag = 0;
		EMfieldVec_flag = 0;
		EMedgeColor_flag = 0;
		Element_flag = 0;
		Material_flag = 0;
		Node_flag = 1;

		printf("\n What is the desired node number?\n");
		scanf("%d", &node_choice);
		if ( node_choice > numnp - 1 )
		{
			node_choice = 0;
		}
		break;

/* 'e' selects the element to be viewed.  */

	    case 'e':
		color_choice = 32;
		input_color_flag = 0;
		EMfieldVec_flag = 0;
		EMedgeColor_flag = 0;
		Element_flag = 1;
		Material_flag = 0;
		Node_flag = 0;

		printf("\n What is the desired element number?\n");
		scanf("%d", &ele_choice);
		if ( ele_choice > numel - 1 )
		{
			ele_choice = 0;
		}
		Solid_flag = 1;
		break;

/* 'm' selects the material to be viewed.  */

	    case 'm':
		color_choice = 30;
		input_color_flag = 0;
		EMfieldVec_flag = 0;
		EMedgeColor_flag = 0;
		Element_flag = 0;
		Material_flag = 1;
		Node_flag = 0;

		printf("\n What is the desired material number?\n");
		scanf("%d", &matl_choice);
		if ( matl_choice > nmat - 1 )
		{
			matl_choice = 0;
		}
		Solid_flag = 1;
		break;

/* '>' and '<' amplify and shrink the displacements on the deformed object */

	    case '.':
		if( post_flag )
		{
			After_flag = 1;
			/*Amplify_flag = 1;*/
			amplify_step = amplify_step0;
			if( amplify_factor < 1.0 - SMALL2 )
				amplify_step = .02;
			amplify_factor += amplify_step;
			/*printf("amplify factor %16.8e %16.8e \n", amplify_factor, amplify_step);*/
/* Update Coordinates */
			for ( i = 0; i < numnp; ++i )
			{
			   *(coord + nsd*i) = *(coord0 + nsd*i) +
				*(EMnode + ndof*i)*amplify_factor;
			   *(coord + nsd*i + 1) = *(coord0 + nsd*i + 1) +
				*(EMnode + ndof*i + 1)*amplify_factor;
			}

/* Update charge graphics vectors */	
			for( i = 0; i < numnp; ++i)
			{
			    fpointx = *(coord+nsd*i);
			    fpointy = *(coord+nsd*i + 1);
			    EMnode_vec[i].x = fpointx - EMnode_vec0[i].x;
			    EMnode_vec[i].y = fpointy - EMnode_vec0[i].y;
	
			}
		}
		break;
	    case ',':
		if( post_flag )
		{
			After_flag = 1;
			/*Amplify_flag = 1;*/
			amplify_step = amplify_step0;
			if( amplify_factor < 1.0 + amplify_step0 - SMALL2 )
				amplify_step = .02;
			amplify_factor -= amplify_step;
			if ( amplify_factor < 0.0 )
				amplify_factor = 0.0;
			/*printf("amplify factor %f \n", amplify_factor);*/
/* Update Coordinates */
			for ( i = 0; i < numnp; ++i )
			{
			   *(coord + nsd*i) = *(coord0 + nsd*i) +
				*(EMnode + ndof*i)*amplify_factor;
			   *(coord + nsd*i + 1) = *(coord0 + nsd*i + 1) +
				*(EMnode + ndof*i + 1)*amplify_factor;
			}

/* Update charge graphics vectors */	
			for( i = 0; i < numnp; ++i)
			{
			    fpointx = *(coord+nsd*i);
			    fpointy = *(coord+nsd*i + 1);
			    EMnode_vec[i].x = fpointx - EMnode_vec0[i].x;
			    EMnode_vec[i].y = fpointy - EMnode_vec0[i].y;
	
			}
		}
		break;

/* The '<' and '>' keys move the Cross Section Plane on the Z
   Axes in the mesh window. */

	    case '<':
		cross_sec_in_out -= step_sizez;
		if( cross_sec_in_out < AxisMin_z )
			cross_sec_in_out = AxisMin_z;
		break;
	    case '>':
		cross_sec_in_out += step_sizez;
		if( cross_sec_in_out > AxisMax_z )
			cross_sec_in_out = AxisMax_z;
		break;

/* 'a' and 'b' turns on and off the deformed and undeformed mesh */

	    case 'a':
		After_flag = 1 - After_flag;
		break; 
	    case 'b':
		Before_flag = 1 - Before_flag;
		break; 

/* 'd' turns on and off the applied Electric Field Edge colors */

	    case 'd':
		Boundary_flag = 1 - Boundary_flag;
		break; 

/* Reset the rotation */

	    case 'c':
		xAngle = 0.0;
		yAngle = 0.0;
		zAngle = 0.0;
		break; 

/* Reset the translation */

	    case 'v':
		left_right = left_right0;
		up_down = up_down0;
		in_out = in_out0;
		if ( !Perspective_flag )
		{
			left_right = 0.0;
			up_down = 0.0;
			ortho_right = ortho_right0;
			ortho_left = ortho_left0;
			ortho_top = ortho_top0;
			ortho_bottom = ortho_bottom0;
			MeshReshape( glutGet(GLUT_WINDOW_WIDTH),
				glutGet(GLUT_WINDOW_HEIGHT));
		}
		cross_sec_left_right = cross_sec_left_right0;
		cross_sec_up_down = cross_sec_up_down0;
		cross_sec_in_out = cross_sec_in_out0;
		break; 

	    case 'f':
		EMfieldVec_flag = 1 - EMfieldVec_flag;
		break;
#if QUADE
	    case 'g':
		check = qdGetNewMesh();
		if(!check) printf( " Problems with qdGetNewMesh\n");
		break;
#endif
#if TRIE
	    case 'g':
		check = trGetNewMesh();
		if(!check) printf( " Problems with trGetNewMesh\n");
		break;
#endif
	    case 'h':
		EReGetparameter();
		check = Eset( bc, connecter, EMnode_vec0, EMnode, EMnode_color, EMmag,
			EMmag_color, EMedge, EMedge_color, charge, charge_color,
			Volt, Volt_color );
		if(!check) printf( " Problems with trset \n");
		if ( !Perspective_flag )
		{
			MeshReshape( glutGet(GLUT_WINDOW_WIDTH),
				glutGet(GLUT_WINDOW_HEIGHT));
		}
		break;
	    case 'j':
		Outline_flag = 1 - Outline_flag;
		break;
	    case 'p':
		Perspective_flag = 1 - Perspective_flag;
		MeshReshape( glutGet(GLUT_WINDOW_WIDTH),
			glutGet(GLUT_WINDOW_HEIGHT));
		left_right = left_right0;
		up_down = up_down0;
		/*in_out = in_out0;*/
		if ( !Perspective_flag )
		{
			left_right = 0.0;
			up_down = 0.0;
		}
		break;
	    case 'q':
		exit(1);
		break;
	    case 'r':
		Render_flag = 1 - Render_flag;
		EMfieldVec_flag = 0;
		EMedgeColor_flag = 0;
		MeshInit();
		break;
	    case 's':
		CrossSection_flag = 1 - CrossSection_flag;
		break;
	    case 't':
		Transparent_flag = 1 - Transparent_flag;
		break;
	    case 'w':
		Solid_flag = 1 - Solid_flag;
		if(!Solid_flag) Outline_flag = 1;
		break;
	    case 'x':
		Axes_flag = 1 - Axes_flag;
		break;
	    case 'y':
		ScreenShot( mesh_width, mesh_height );
		break;
	    case 27:
		exit(0);
		break;
	}

	input_color_flag = 0;
        if( EMfieldVec_flag )
        {
                if( color_choice > 29) input_color_flag = 1;
                Element_flag = 0;
                Material_flag = 0;
                Node_flag = 0;
        }

	if(!post_flag) After_flag = 0;
	if(!input_flag) Before_flag = 0;

	glutPostRedisplay();
}

