/*
    This program calculates and writes the parameters for
    the FEM GUI for triangle elements.
  
	                Last Update 1/14/10

    SLFCEM source file
    Version:  1.1
    Copyright (C) 1999-2010  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
 */
#include <assert.h>
#include <stddef.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#if QUADE
#include "../quad/quad_em/qdeconst.h"
#include "../quad/quad_em/qdestruct.h"
#include "../quad/quadegr/qdestrcgr.h"
#endif
#if TRIE
#include "../tri/tri_em/treconst.h"
#include "../tri/tri_em/trestruct.h"
#include "../tri/triegr/trestrcgr.h"
#endif
#include "control.h"

#define init_far0      -2.0

extern int nmat, numnp, numed, numel, dof;
extern modal_flag, static_flag;
extern double step_sizex, step_sizey, step_sizez;
extern double left, right, top, bottom, near, far, fscale, coord_rescale;
extern double cross_sec_left_right, cross_sec_up_down, cross_sec_in_out,
	cross_sec_left_right0, cross_sec_up_down0, cross_sec_in_out0;
extern int control_height, control_width, mesh_height, mesh_width;
extern double ortho_left, ortho_right, ortho_top, ortho_bottom,
	ortho_left0, ortho_right0, ortho_top0, ortho_bottom0;
extern double left_right, up_down, in_out, left_right0, up_down0, in_out0;
extern double AxisMax_x, AxisMax_y, AxisMax_z,
	AxisMin_x, AxisMin_y, AxisMin_z,
	IAxisMin_x, IAxisMin_y, IAxisMin_z;
extern double AxisLength_x, AxisLength_y, AxisLength_z,
	AxisLength_max;
extern double AxisPoint_step;
extern double amplify_step0;

extern double init_right, init_left, init_top,
	init_bottom, init_near, init_far, true_far, dim_max;
extern double max_EMnodex, min_EMnodex, del_EMnodex, max_EMnodey, min_EMnodey, del_EMnodey,
	absolute_max_EMnode, max_EMmag, min_EMmag, del_EMmag,
	max_EMedge, min_EMedge, del_EMedge, absolute_max_coord;
extern double max_Volt, min_Volt, del_Volt, max_charge, min_charge, del_charge;

int Eparameter( double *coord, double *EMnode, double *EMmag, double *EMedge,
	double *charge, double *Volt )
{
	int i, j, check;
	int node_EMnodex_max, node_EMnodex_min, node_EMnodey_max, node_EMnodey_min;
	int node_EMmag_max, node_EMmag_min, edge_EMedge_max, edge_EMedge_min;
	int node_Volt_max, node_Volt_min, node_charge_max, node_charge_min;
	FILE *view_data;
	
/*   view_data contains all the parameters and extreme values
*/
#if QUADE
	view_data = fopen( "qdeview.dat","w" );
#endif
#if TRIE
	view_data = fopen( "treview.dat","w" );
#endif

/* Initialize parameters */

	step_sizex = .1; step_sizey = .1; step_sizez = .1;
	init_right = - BIG; init_left = BIG;
	init_top = - BIG; init_bottom = BIG;
	init_near = - BIG; init_far = init_far0; true_far = BIG;
	max_charge  = - BIG; min_charge  = BIG;
	max_Volt    = - BIG; min_Volt    = BIG;
	max_EMmag   = - BIG; min_EMmag   = BIG;
	max_EMedge  = - BIG; min_EMedge = BIG;
	max_EMnodex = - BIG; min_EMnodex = BIG;
	max_EMnodey = - BIG; min_EMnodey = BIG;

	node_charge_max  = 0; node_charge_min  = 0;
	node_Volt_max    = 0; node_Volt_min    = 0;
	node_EMmag_max   = 0; node_EMmag_min   = 0;
	edge_EMedge_max  = 0; edge_EMedge_min  = 0;
	node_EMnodex_max = 0; node_EMnodex_min = 0;
	node_EMnodey_max = 0; node_EMnodey_min = 0;

/* Search for extreme values */
 
/* Search for extreme values of the Electric Field, Voltage, Charge, and
   and nodal point */

	init_near = 0.0;
	true_far = 0.0;

	for( i = 0; i < numnp; ++i )
	{

/* Search for extreme nodal coordinates for parameters when object is
   viewed orthographically */

		if( init_right < *(coord+nsd*i))
			init_right=*(coord+nsd*i);

		if( init_left > *(coord+nsd*i))
			init_left=*(coord+nsd*i);

		if( init_top < *(coord+nsd*i+1))
			init_top=*(coord+nsd*i+1);

		if( init_bottom > *(coord+nsd*i+1))
			init_bottom=*(coord+nsd*i+1);

/* Search for extreme nodal electric field components */

		if( max_EMnodex < *(EMnode+ndof*i))
		{
			max_EMnodex=*(EMnode+ndof*i);
			node_EMnodex_max = i;
		}
		if( min_EMnodex > *(EMnode+ndof*i))
		{
			min_EMnodex=*(EMnode+ndof*i);
			node_EMnodex_min = i;
		}
		if( max_EMnodey < *(EMnode+ndof*i+1))
		{
			max_EMnodey=*(EMnode+ndof*i+1);
			node_EMnodey_max = i;
		}
		if( min_EMnodey > *(EMnode+ndof*i+1))
		{
			min_EMnodey=*(EMnode+ndof*i+1);
			node_EMnodey_min = i;
		}

/* Search for extreme nodal electric field magnitudes */

		if( max_EMmag < *(EMmag+Vndof*i))
		{
			max_EMmag=*(EMmag+Vndof*i);
			node_EMmag_max = i;
		}
		if( min_EMmag > *(EMmag+Vndof*i))
		{
			min_EMmag=*(EMmag+Vndof*i);
			node_EMmag_min = i;
		}

/* Search for extreme nodal Charges */

		if( max_charge < *(charge+Vndof*i))
		{
			max_charge=*(charge+Vndof*i);
			node_charge_max = i;
		}
		if( min_charge > *(charge+Vndof*i))
		{
			min_charge=*(charge+Vndof*i);
			node_charge_min = i;
		}
/* Search for extreme nodal Voltages */

		if( max_Volt < *(Volt+Vndof*i))
		{
			max_Volt=*(Volt+Vndof*i);
			node_Volt_max = i;
		}
		if( min_Volt > *(Volt+Vndof*i))
		{
			min_Volt=*(Volt+Vndof*i);
			node_Volt_min = i;
		}
	}

	for( i = 0; i < numed; ++i )
	{

/* Search for extreme edge electric field values */

		if( max_EMedge < *(EMedge+Vndof*i))
		{
			max_EMedge=*(EMedge+Vndof*i);
			edge_EMedge_max = i;
		}
		if( min_EMedge > *(EMedge+Vndof*i))
		{
			min_EMedge=*(EMedge+Vndof*i);
			edge_EMedge_min = i;
		}
	}

/* Because Mesa has problems with Meshes that have dimensions larger than 1000
   or smaller than .1, I am rescaling everything so that things are on the order
   of 10.0.
*/

	absolute_max_coord = fabs(init_left);
	if(absolute_max_coord < fabs(init_right)) absolute_max_coord = fabs(init_right);
	if(absolute_max_coord < fabs(init_bottom)) absolute_max_coord = fabs(init_bottom);
	if(absolute_max_coord < fabs(init_top)) absolute_max_coord = fabs(init_top);

	coord_rescale = 1.0;
	if( absolute_max_coord > 10.0 )     coord_rescale = 10.0;
	if( absolute_max_coord > 100.0 )    coord_rescale = 100.0;
	if( absolute_max_coord > 1000.0 )   coord_rescale = 1000.0;
	if( absolute_max_coord > 10000.0 )  coord_rescale = 10000.0;
	if( absolute_max_coord > 100000.0 ) coord_rescale = 100000.0;

	if( absolute_max_coord < 1.0 )     coord_rescale = 0.1;
	if( absolute_max_coord < 0.1 )     coord_rescale = 0.01;
	if( absolute_max_coord < 0.01 )    coord_rescale = 0.001;
	if( absolute_max_coord < 0.001 )   coord_rescale = 0.0001;
	if( absolute_max_coord < 0.0001 )  coord_rescale = 0.00001;
	if( absolute_max_coord < 0.00001 ) coord_rescale = 0.000001;

/* Rescale coordinates and Electric Field Compononts in x and y

   This really shouldn't be done, but I keep it since I sometimes
   want to treat the electric field as a displacement field, and
   see how this vector field deforms the mesh.
 */
	if( coord_rescale > 1.01 || coord_rescale < .99 )
	{
		for( i = 0; i < numnp; ++i )
		{
			*(coord+nsd*i) /= coord_rescale;
			*(coord+nsd*i+1) /= coord_rescale;

			*(EMnode+ndof*i)   /= coord_rescale;
			*(EMnode+ndof*i+1) /= coord_rescale;
		}

		init_left /= coord_rescale;
		init_right /= coord_rescale;
		init_bottom /= coord_rescale;
		init_top /= coord_rescale;
		init_near /= coord_rescale;
		true_far /= coord_rescale;

		min_EMnodex /= coord_rescale;
		max_EMnodex /= coord_rescale;
		min_EMnodey /= coord_rescale;
		max_EMnodey /= coord_rescale;
	}

/* Search for largest absolute value U */

	absolute_max_EMnode = fabs(min_EMnodex);
	if( absolute_max_EMnode < fabs(max_EMnodex))
		absolute_max_EMnode = fabs(max_EMnodex);
	if( absolute_max_EMnode < fabs(min_EMnodey))
		absolute_max_EMnode = fabs(min_EMnodey);
	if( absolute_max_EMnode < fabs(max_EMnodey))
		absolute_max_EMnode = fabs(max_EMnodey);

	if( init_far > true_far)
		init_far=true_far;

/* Set the axes parameters */

	AxisMax_x = 1.2*init_right + 1.0;
	AxisMax_y = 1.2*init_top + 1.0;
	AxisMax_z = 1.2*init_near + 1.0;
	AxisMin_x = 1.2*init_left - 1.0;
	AxisMin_y = 1.2*init_bottom - 1.0;
	AxisMin_z = 1.2*true_far - 1.0;

	if( AxisMax_x < 0.0 )
		AxisMax_x = -.1*AxisMin_x;
	if( AxisMax_y < 0.0 )
		AxisMax_y = -.1*AxisMin_y;
	if( AxisMax_z < 0.0 )
		AxisMax_z = -.1*AxisMin_z;
	if( AxisMin_x > 0.0 )
		AxisMin_x = -.1*AxisMax_x;
	if( AxisMin_y > 0.0 )
		AxisMin_y = -.1*AxisMax_y;
	if( AxisMin_z > 0.0 )
		AxisMin_z = -.1*AxisMax_z;

	IAxisMin_x = (double)(int)AxisMin_x; 
	IAxisMin_y = (double)(int)AxisMin_y;
	IAxisMin_z = (double)(int)AxisMin_z;

	AxisLength_x = (int)(AxisMax_x - AxisMin_x);
	AxisLength_y = (int)(AxisMax_y - AxisMin_y);
	AxisLength_z = (int)(AxisMax_z - AxisMin_z);
	AxisLength_max = AxisLength_x;
	if( AxisLength_max < AxisLength_y )
		AxisLength_max = AxisLength_y;
	if( AxisLength_max < AxisLength_z )
		AxisLength_max = AxisLength_z;

	AxisPoint_step = .1;
	if( AxisLength_max > 1.0 )
		AxisPoint_step = 1.0;
	if( AxisLength_max > 10.0 )
		AxisPoint_step = 10.0;
	if( AxisLength_max > 100.0 )
		AxisPoint_step = 100.0;
	if( AxisLength_max > 1000.0 )
		AxisPoint_step = 1000.0;
	if( AxisLength_max > 10000.0 )
		AxisPoint_step = 10000.0;
	AxisLength_max *= .05;
	/*printf(" AxisPoint_step %10.5e\n",AxisPoint_step);*/

/* Determine amplification step size */

	amplify_step0 = .5*AxisLength_max/(absolute_max_EMnode+SMALL);
	/*printf(" amplify_step0 %10.5e\n",amplify_step0);*/

/* Calculate orthographic viewport parameters */

	right = init_right + (init_right - init_left) / 10.0;
	left = init_left - (init_right - init_left) / 10.0;
	top = init_top + (init_top - init_bottom) / 10.0;
	bottom = init_bottom - (init_top - init_bottom) / 10.0;
	near = init_near + (init_near - init_far) / 10.0;
	far = init_far - (init_near - init_far) / 10.0;

	dim_max = right - left;
	if( dim_max < top - bottom )
		dim_max = top - bottom;

	ortho_right0 = left + dim_max + 1.0;
	ortho_left0 = left - 1.0;
	ortho_top0 = bottom + dim_max + 1.0;
	ortho_bottom0 = bottom - 1.0;

	ortho_right = ortho_right0;
	ortho_left = ortho_left0;
	ortho_top = ortho_top0;
	ortho_bottom = ortho_bottom0;

/* Set the Viewer parameters */

	step_sizex = (right - left) / 20.0;
	step_sizey = (top - bottom) / 20.0;
	step_sizez = (near - far) / 5.0;

	left_right0 = -(left + right) / 2.0;
	up_down0 = - (top + bottom ) / 2.0;
	/*in_out  = (far + near ) / 2.0 - 5.0;*/
	in_out0 = far - 20.0*AxisLength_max;
	left_right = left_right0;
	up_down = up_down0;
	in_out = in_out0;

/* Set the Cross Section Plane parameters */

	cross_sec_left_right0 = AxisMax_x;
	cross_sec_up_down0 = AxisMax_y;
	cross_sec_in_out0 = AxisMax_z;

	cross_sec_left_right = cross_sec_left_right0;
	cross_sec_up_down = cross_sec_up_down0;
	cross_sec_in_out = cross_sec_in_out0;

	mesh_width = mesh_width0;
	mesh_height = mesh_height0;

/* Print the above data in the file "treview.dat" or "qdeview.dat" */

	fprintf( view_data, "                            node\n");
	fprintf( view_data, "                          min  max       min            max\n");
	fprintf( view_data,"Charge                %5d %5d   %14.6e %14.6e\n", node_charge_min,
		node_charge_max, min_charge, max_charge);
	fprintf( view_data,"Voltage               %5d %5d   %14.6e %14.6e\n", node_Volt_min,
		node_Volt_max, min_Volt, max_Volt);
	fprintf( view_data,"Ele. Field magnitude  %5d %5d   %14.6e %14.6e\n", node_EMmag_min,
		node_EMmag_max, min_EMmag, max_EMmag);
	fprintf( view_data,"Ele. Field in X       %5d %5d   %14.6e %14.6e\n", node_EMnodex_min,
		node_EMnodex_max, min_EMnodex*coord_rescale, max_EMnodex*coord_rescale);
	fprintf( view_data,"Ele. Field in Y       %5d %5d   %14.6e %14.6e\n", node_EMnodey_min,
		node_EMnodey_max, min_EMnodey*coord_rescale, max_EMnodey*coord_rescale);
	fprintf( view_data, "                            edge\n");
	fprintf( view_data, "                          min  max       min            max\n");
	fprintf( view_data,"Ele. Field edge       %5d %5d   %14.6e %14.6e\n", edge_EMedge_min,
		edge_EMedge_max, min_EMedge, max_EMedge);
	fprintf( view_data,"\n");
	fprintf( view_data,"Orthographic viewport parameters(right, left, top, bottom, near, far)\n ");
	fprintf( view_data,"%14.6e %14.6e %14.6e %14.6e %14.6e %14.6e\n", ortho_right, ortho_left,
		ortho_top, ortho_bottom, near, 1000.0);
	fprintf( view_data,"Perspective viewport parameters( mesh width and height)\n ");
	fprintf( view_data,"%6d %6d\n", mesh_width, mesh_height);
	fprintf( view_data,"Step sizes in x, y, z\n ");
	fprintf( view_data,"%14.6e %14.6e %14.6e\n",step_sizex, step_sizey, step_sizez);
	fprintf( view_data,"Amplification size\n ");
	fprintf( view_data,"%14.6e\n",amplify_step0);

	fclose( view_data );

	return 1;    /* ANSI C requires main to return int. */
}
