/*
    This program calculates and writes the parameters for
    the FEM GUI for triangle elements.
  
	                Last Update 1/14/10

    SLFCEM source file
    Version:  1.1
    Copyright (C) 1999-2010  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
 */
#include <assert.h>
#include <stddef.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#if QUADE
#include "../quad/quad_em/qdeconst.h"
#include "../quad/quad_em/qdestruct.h"
#include "../quad/quadegr/qdestrcgr.h"
#endif
#if TRIE
#include "../tri/tri_em/treconst.h"
#include "../tri/tri_em/trestruct.h"
#include "../tri/triegr/trestrcgr.h"
#endif
#include "control.h"

#define init_far0      -2.0

extern int nmat, numnp, numel, dof;
extern double step_sizex, step_sizey, step_sizez;
extern double left, right, top, bottom, near, far, fscale, coord_rescale;
extern int control_height, control_width, mesh_height, mesh_width;
extern double ortho_left, ortho_right, ortho_top, ortho_bottom;
extern double left_right, up_down, in_out, left_right0, up_down0, in_out0;
extern double AxisMax_x, AxisMax_y, AxisMax_z,
	AxisMin_x, AxisMin_y, AxisMin_z,
	IAxisMin_x, IAxisMin_y, IAxisMin_z;
extern double AxisLength_x, AxisLength_y, AxisLength_z,
	AxisLength_max;
extern double AxisPoint_step;
extern double amplify_step0;

extern double init_right, init_left, init_top,
	init_bottom, init_near, init_far, true_far, dim_max;
extern double max_EMnodex, min_EMnodex, del_EMnodex, max_EMnodey, min_EMnodey, del_EMnodey,
	absolute_max_EMnode, max_EMmag, min_EMmag, del_EMmag,
	max_EMedge, min_EMedge, del_EMedge, absolute_max_coord;
extern double max_Volt, min_Volt, del_Volt, max_charge, min_charge, del_charge;

void EReGetparameter(void)
{
	int i, j, check;
	int node_EMnodex_max, node_EMnodex_min, node_EMnodey_max, node_EMnodey_min;
	int node_EMmag_max, node_EMmag_min, edge_EMedge_max, edge_EMedge_min;
	int node_Volt_max, node_Volt_min, node_charge_max, node_charge_min;
	char char_dum[20], char_dum2[10], char_dum3[10], char_dum4[10], buf[ BUFSIZ ];
	double fdum;
	FILE *view_data;
	
/*   view_data contains all the parameters and extreme values
*/
#if QUADE
	view_data = fopen( "qdeview.dat","r" );
	if(view_data == NULL ) {
		printf("Can't find file qdeview.dat\n");
		exit(1);
	}
#endif
#if TRIE
	view_data = fopen( "treview.dat","r" );
	if(view_data == NULL ) {
		printf("Can't find file treview.dat\n");
		exit(1);
	}
#endif


	fgets( buf, BUFSIZ, view_data );
	fgets( buf, BUFSIZ, view_data );
	fscanf( view_data,"%20s          %d %d   %lf %lf\n", char_dum, &node_charge_min,
		&node_charge_max, &min_charge, &max_charge);
	fscanf( view_data,"%20s          %d %d   %lf %lf\n", char_dum, &node_Volt_min,
		&node_Volt_max, &min_Volt, &max_Volt);
	fscanf( view_data,"%20s %10s %10s      %d %d   %lf %lf\n", char_dum, char_dum2, char_dum3,
		&node_EMmag_min, &node_EMmag_max, &min_EMmag, &max_EMmag);
	fscanf( view_data,"%20s %10s %10s %5s  %d %d   %lf %lf\n", char_dum, char_dum2, char_dum3, char_dum4,
		&node_EMnodex_min, &node_EMnodex_max, &min_EMnodex, &max_EMnodex);
	fscanf( view_data,"%20s %10s %10s %5s  %d %d   %lf %lf\n", char_dum, char_dum2, char_dum3, char_dum4,
		&node_EMnodey_min, &node_EMnodey_max, &min_EMnodey, &max_EMnodey);
	fgets( buf, BUFSIZ, view_data );
	fgets( buf, BUFSIZ, view_data );
	fscanf( view_data,"%20s %10s %10s      %d %d   %lf %lf\n", char_dum, char_dum2, char_dum3,
		&edge_EMedge_min, &edge_EMedge_max, &min_EMedge, &max_EMedge);

/* Rescale the Electric Field Compononts in x and y data

   This really doesn't need to be done.  You can look in "eparam.c"
   for the reason why I rescale the Electric Field components. 
*/

	min_EMnodex /= coord_rescale;
	max_EMnodex /= coord_rescale;
	min_EMnodey /= coord_rescale;
	max_EMnodey /= coord_rescale;

	fgets( buf, BUFSIZ, view_data );
	fscanf( view_data,"%lf %lf %lf %lf %lf %lf\n", &ortho_right, &ortho_left,
		&ortho_top, &ortho_bottom, &near, &fdum);
	fgets( buf, BUFSIZ, view_data );
	fscanf( view_data,"%d %d\n", &mesh_width, &mesh_height);
	fgets( buf, BUFSIZ, view_data );
	fscanf( view_data,"%lf %lf %lf\n", &step_sizex, &step_sizey, &step_sizez);
	fgets( buf, BUFSIZ, view_data );
	fscanf( view_data,"%lf\n", &amplify_step0);

	fclose( view_data );

	printf( "                            node\n");
	printf( "                          min  max       min            max\n");
        printf( "Charge                %5d %5d   %14.6e %14.6e\n", node_charge_min,
                node_charge_max, min_charge, max_charge);
        printf( "Voltage               %5d %5d   %14.6e %14.6e\n", node_Volt_min,
                node_Volt_max, min_Volt, max_Volt);
        printf( "Ele. Field magnitude  %5d %5d   %14.6e %14.6e\n", node_EMmag_min,
                node_EMmag_max, min_EMmag, max_EMmag);
	printf( "Electric Field in x   %5d %5d   %14.6e %14.6e\n", node_EMnodex_min,
		node_EMnodex_max, min_EMnodex*coord_rescale, max_EMnodex*coord_rescale);
	printf( "Electric Field in y   %5d %5d   %14.6e %14.6e\n", node_EMnodey_min,
		node_EMnodey_max, min_EMnodey*coord_rescale, max_EMnodey*coord_rescale);
	printf( "                            edge\n");
	printf( "                          min  max       min            max\n");
        printf( "Ele. Field edge value %5d %5d   %14.6e %14.6e\n", edge_EMedge_min,
                edge_EMedge_max, min_EMedge, max_EMedge);
	printf("\n");
	printf("Orthographic viewport parameters(right, left, top, bottom, near, far)\n ");
	printf("%14.6e %14.6e %14.6e %14.6e %14.6e %14.6e\n", ortho_right, ortho_left,
		ortho_top, ortho_bottom, near, 1000.0);
	printf("Perspective viewport parameters( mesh width and height)\n ");
	printf("%6d %6d\n", mesh_width, mesh_height);
	printf("Step sizes in x, y, z\n ");
	printf("%14.6e %14.6e %14.6e\n",step_sizex, step_sizey, step_sizez);
	printf("Amplification size\n ");
	printf("%14.6e\n",amplify_step0);
}

