/*
    This program sets viewing and analysis values based on the parameters 
    determined in qdparameters for the FEM GUI for quad elements.
  
   			Last Update 1/22/10

    SLFCEM source file
    Version:  1.1
    Copyright (C) 1999-2010  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
 */
#include <assert.h>
#include <stddef.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#if QUADE
#include "../quad/quad_em/qdeconst.h"
#include "../quad/quad_em/qdestruct.h"
#include "../quad/quadegr/qdestrcgr.h"
#endif
#if TRIE
#include "../tri/tri_em/treconst.h"
#include "../tri/tri_em/trestruct.h"
#include "../tri/triegr/trestrcgr.h"
#endif
#include "control.h"

#define init_far0      -2.0

extern int nmat, numnp, numed, numel, dof;
extern double step_sizex, step_sizey, step_sizez;
extern double left, right, top, bottom, near, far, fscale, coord_rescale;
extern int control_height, control_width, mesh_height, mesh_width;
extern double ortho_left, ortho_right, ortho_top, ortho_bottom;
extern double left_right, up_down, in_out, left_right0, up_down0, in_out0;
extern double AxisMax_x, AxisMax_y, AxisMax_z,
	AxisMin_x, AxisMin_y, AxisMin_z,
	IAxisMin_x, IAxisMin_y, IAxisMin_z;
extern double AxisLength_x, AxisLength_y, AxisLength_z,
	AxisLength_max;
extern double AxisPoint_step;
extern double amplify_step0;

extern double EMnodex_div[boxnumber+1], EMnodey_div[boxnumber+1], EMmag_div[boxnumber+1],
	EMedge_div[boxnumber+1], charge_div[boxnumber+1], Volt_div[boxnumber+1];
extern double init_right, init_left, init_top,
	init_bottom, init_near, init_far, true_far, dim_max;
extern double max_EMnodex, min_EMnodex, del_EMnodex, max_EMnodey, min_EMnodey, del_EMnodey,
	max_EMmag, min_EMmag, del_EMmag, max_EMedge, min_EMedge, del_EMedge,
	absolute_max_EMnode;

extern double max_Volt, min_Volt, del_Volt, max_charge, min_charge, del_charge;

int Eset( BOUND bc, int *connecter, XYF *EMnode_vec0, double *EMnode, int *EMnode_color,
	double *EMmag, int *EMmag_color, double *EMedge, int *EMedge_color,
	double *charge, int *charge_color, double *Volt, int *Volt_color)
{
	int i, j, check;
	double EMnode_vec_length, fdum;
	int node;

/* Determine nodal electric field color scheme */

	del_charge  = (max_charge - min_charge + SMALL)/(double)(boxnumber);
	del_Volt    = (max_Volt - min_Volt + SMALL)/(double)(boxnumber);
	del_EMmag   = (max_EMmag - min_EMmag + SMALL)/(double)(boxnumber);
	del_EMnodex = (max_EMnodex - min_EMnodex + SMALL)/(double)(boxnumber);
	del_EMnodey = (max_EMnodey - min_EMnodey + SMALL)/(double)(boxnumber);
	charge_div[0]  = min_charge;
	Volt_div[0]    = min_Volt;
	EMmag_div[0]   = min_EMmag;
	EMnodex_div[0] = min_EMnodex;
	EMnodey_div[0] = min_EMnodey;

	for( i = 0; i < boxnumber; ++i )
	{
		charge_div[i+1]  = charge_div[i] + del_charge;
		Volt_div[i+1]    = Volt_div[i] + del_Volt;
		EMmag_div[i+1]   = EMmag_div[i] + del_EMmag;
		EMnodex_div[i+1] = EMnodex_div[i] + del_EMnodex;
		EMnodey_div[i+1] = EMnodey_div[i] + del_EMnodey;
		/*printf(" EMnode div x y %10.5e %10.5e\n",
			EMnodex_div[i]);*/
	}

/* Assign Colors for Electric Field */

	for( i = 0; i < numnp; ++i )
	{
/* Assign colors for the charge */
	       *(charge_color + Vndof*i) = 0;
	       if(  *(charge + Vndof*i) > charge_div[1] )
	       {
		  *(charge_color + Vndof*i) = 1;
		  if(  *(charge + Vndof*i) > charge_div[2] )
		  {
		     *(charge_color + Vndof*i) = 2;
		     if(  *(charge + Vndof*i) > charge_div[3] )
		     {
			*(charge_color + Vndof*i) = 3;
			if(  *(charge + Vndof*i) > charge_div[4] )
			{
			   *(charge_color + Vndof*i) = 4;
			   if(  *(charge + Vndof*i) > charge_div[5] )
			   {
			      *(charge_color + Vndof*i) = 5;
			      if(  *(charge + Vndof*i) > charge_div[6] )
			      {
				 *(charge_color + Vndof*i) = 6;
				 if(  *(charge + Vndof*i) > charge_div[7] )
				 {
				    *(charge_color + Vndof*i) = 7;
				 }
			      }
			   }
			}
		     }
		  }
	       }
	       /*printf(" charge %4d %16.8e %4d\n", i,
			*(charge +Vndof*i), *(charge_color+Vndof*i));*/
/* Assign colors for the Voltage */
	       *(Volt_color + Vndof*i) = 0;
	       if(  *(Volt + Vndof*i) > Volt_div[1] )
	       {
		  *(Volt_color + Vndof*i) = 1;
		  if(  *(Volt + Vndof*i) > Volt_div[2] )
		  {
		     *(Volt_color + Vndof*i) = 2;
		     if(  *(Volt + Vndof*i) > Volt_div[3] )
		     {
			*(Volt_color + Vndof*i) = 3;
			if(  *(Volt + Vndof*i) > Volt_div[4] )
			{
			   *(Volt_color + Vndof*i) = 4;
			   if(  *(Volt + Vndof*i) > Volt_div[5] )
			   {
			      *(Volt_color + Vndof*i) = 5;
			      if(  *(Volt + Vndof*i) > Volt_div[6] )
			      {
				 *(Volt_color + Vndof*i) = 6;
				 if(  *(Volt + Vndof*i) > Volt_div[7] )
				 {
				    *(Volt_color + Vndof*i) = 7;
				 }
			      }
			   }
			}
		     }
		  }
	       }
	       /*printf(" Volt %4d %16.8e %4d\n", i,
			*(Volt +Vndof*i), *(Volt_color+Vndof*i));*/
/* Assign colors for EMnode x */
	       *(EMnode_color + ndof*i) = 0;
	       if(  *(EMnode + ndof*i) > EMnodex_div[1] )
	       {
		  *(EMnode_color + ndof*i) = 1;
		  if(  *(EMnode + ndof*i) > EMnodex_div[2] )
		  {
		     *(EMnode_color + ndof*i) = 2;
		     if(  *(EMnode + ndof*i) > EMnodex_div[3] )
		     {
			*(EMnode_color + ndof*i) = 3;
			if(  *(EMnode + ndof*i) > EMnodex_div[4] )
			{
			   *(EMnode_color + ndof*i) = 4;
			   if(  *(EMnode + ndof*i) > EMnodex_div[5] )
			   {
			      *(EMnode_color + ndof*i) = 5;
			      if(  *(EMnode + ndof*i) > EMnodex_div[6] )
			      {
				 *(EMnode_color + ndof*i) = 6;
				 if(  *(EMnode + ndof*i) > EMnodex_div[7] )
				 {
				    *(EMnode_color + ndof*i) = 7;
				 }
			      }
			   }
			}
		     }
		  }
	       }
	       /*printf(" EMnode x %4d %16.8e %4d\n", i,
			*(EMnode +ndof*i), *(EMnode_color+ndof*i));*/
/* Assign colors for EMnode y */
	       *(EMnode_color + ndof*i + 1) = 0;
	       if(  *(EMnode + ndof*i + 1) > EMnodey_div[1] )
	       {
		  *(EMnode_color + ndof*i + 1) = 1;
		  if(  *(EMnode + ndof*i + 1) > EMnodey_div[2] )
		  {
		     *(EMnode_color + ndof*i + 1) = 2;
		     if(  *(EMnode + ndof*i + 1) > EMnodey_div[3] )
		     {
			*(EMnode_color + ndof*i + 1) = 3;
			if(  *(EMnode + ndof*i + 1) > EMnodey_div[4] )
			{
			   *(EMnode_color + ndof*i + 1) = 4;
			   if(  *(EMnode + ndof*i + 1) > EMnodey_div[5] )
			   {
			      *(EMnode_color + ndof*i + 1) = 5;
			      if(  *(EMnode + ndof*i + 1) > EMnodey_div[6] )
			      {
				 *(EMnode_color + ndof*i + 1) = 6;
				 if(  *(EMnode + ndof*i + 1) > EMnodey_div[7] )
				 {
				    *(EMnode_color + ndof*i + 1) = 7;
				 }
			      }
			   }
			}
		     }
		  }
	       }
	       /*printf(" EMnode y %4d %16.8e %4d\n", i,
			*(EMnode +ndof*i + 1), *(EMnode_color+ndof*i + 1));*/
/* Assign colors for the magnitude of EMnode, EMmag */
	       *(EMmag_color + Vndof*i) = 0;
	       if(  *(EMmag + Vndof*i) > EMmag_div[1] )
	       {
		  *(EMmag_color + Vndof*i) = 1;
		  if(  *(EMmag + Vndof*i) > EMmag_div[2] )
		  {
		     *(EMmag_color + Vndof*i) = 2;
		     if(  *(EMmag + Vndof*i) > EMmag_div[3] )
		     {
			*(EMmag_color + Vndof*i) = 3;
			if(  *(EMmag + Vndof*i) > EMmag_div[4] )
			{
			   *(EMmag_color + Vndof*i) = 4;
			   if(  *(EMmag + Vndof*i) > EMmag_div[5] )
			   {
			      *(EMmag_color + Vndof*i) = 5;
			      if(  *(EMmag + Vndof*i) > EMmag_div[6] )
			      {
				 *(EMmag_color + Vndof*i) = 6;
				 if(  *(EMmag + Vndof*i) > EMmag_div[7] )
				 {
				    *(EMmag_color + Vndof*i) = 7;
				 }
			      }
			   }
			}
		     }
		  }
	       }
	       /*printf(" EMmag %4d %16.8e %4d\n", i,
			*(EMmag +Vndof*i), *(EMmag_color+Vndof*i));*/
	}

/* Determine edge electric field color scheme */

	del_EMedge = (max_EMedge - min_EMedge + SMALL)/(double)(boxnumber);
	EMedge_div[0] = min_EMedge;
	for( i = 0; i < boxnumber; ++i ) EMedge_div[i+1] = EMedge_div[i] + del_EMedge;

	for( i = 0; i < numed; ++i )
	{
/* Assign colors for the edges of the Electric Field, EMedge */

	       *(EMedge_color + Vndof*i) = 0;
	       if(  *(EMedge + Vndof*i) > EMedge_div[1] )
	       {
		  *(EMedge_color + Vndof*i) = 1;
		  if(  *(EMedge + Vndof*i) > EMedge_div[2] )
		  {
		     *(EMedge_color + Vndof*i) = 2;
		     if(  *(EMedge + Vndof*i) > EMedge_div[3] )
		     {
			*(EMedge_color + Vndof*i) = 3;
			if(  *(EMedge + Vndof*i) > EMedge_div[4] )
			{
			   *(EMedge_color + Vndof*i) = 4;
			   if(  *(EMedge + Vndof*i) > EMedge_div[5] )
			   {
			      *(EMedge_color + Vndof*i) = 5;
			      if(  *(EMedge + Vndof*i) > EMedge_div[6] )
			      {
				 *(EMedge_color + Vndof*i) = 6;
				 if(  *(EMedge + Vndof*i) > EMedge_div[7] )
				 {
				    *(EMedge_color + Vndof*i) = 7;
				 }
			      }
			   }
			}
		     }
		  }
	       }
	       /*printf(" EMedge %4d %16.8e %4d\n", i,
			*(EMedge +Vndof*i), *(EMedge_color+Vndof*i));*/
	}
/* Calculate the graphical Electric Field vectors at a node for display */

	fdum = -1.e-10;
	for( i = 0; i < numnp; ++i )
	{
		
		EMnode_vec_length =
		   *(EMnode + ndof*i)*(*(EMnode + ndof*i)) +
		   *(EMnode + ndof*i +1)*(*(EMnode + ndof*i +1)) +
		   SMALL;
		EMnode_vec_length = sqrt(EMnode_vec_length);

		if(fdum < EMnode_vec_length) fdum = EMnode_vec_length;

		EMnode_vec0[i].x =
		   *(EMnode + ndof*i)*3*AxisLength_max/5.0;
		EMnode_vec0[i].y =
		   *(EMnode + ndof*i + 1)*3*AxisLength_max/5.0;
		/*printf(" EMnode %4d %16.8e %16.8e %16.8e %16.8e %16.8e\n", i,
		   *(EMnode + ndof*i), *(EMnode + ndof*i + 1),
		   EMnode_vec0[i].x, EMnode_vec0[i].y, fdum);*/
	}

	for( i = 0; i < numnp; ++i )
	{
		EMnode_vec0[i].x /= fdum*coord_rescale;
		EMnode_vec0[i].y /= fdum*coord_rescale;
	}


	return 1;    /* ANSI C requires main to return int. */
}
