/*
    This program performs finite element analysis by reading in
    the data, doing assembling, and then solving the linear system
    for a 4 node quadrilateral element.  It is for eletromagnetism.

        Updated 1/3/10

    SLFCEM source file
    Version:  1.1
    Copyright (C) 1999-2010  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

    The main texts used to write this code is based on:

        Jin, Jianming, The Finite Element Method in Electromagnetics, John Wiley & Sons,
            New York, 1993.

        Volakis, John L., et. al., Finite Element Method for Electromagnetics,
            IEEE Press, 1998.

    Also look at:

       http://www.antenna-theory.com/tutorial/waveguides/waveguides.php
       http://en.wikipedia.org/wiki/Waveguide_(electromagnetism)

    for lots of good information about waveguides.

    IMPORTANT!!: Here are some things facts about this code:

    1) For TE modes:

     1a) If you want to solve the problem of getting the transverse electric
         field at the cut-off frequency, then read my notes at the beginning
         of "qdekasmbl.c".   

     1b) The resulting linear system is only positive semi-definite.  So
         there are many zero modes which come out as very small values
         rather than zero.  There can be even up to 200 zero modes for a
         mesh having 231 DOFs.
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "../../common/eigen.h"
#include "qdeconst.h"
#include "qdestruct.h"

#define LAPACK    0

int Ewriter ( BOUND , int *, int *, int *, double *, int *, double *, int *,
	MATL *, char *, double *, double *, double *, double *);

int writer ( BOUND , int *, double *, int *, double *, int *, MATL *, char *,
	double *, double *);

int qdEMnode_calculate(int *, int *, int *, double *, double *, double *, double *,
	double * );

int qdEVnode_calculate(int *, double *, double *, double *, double *, double *);

int eval_data_print( EIGEN *, char *, int );

int qdELanczos(double *, BOUND , int *, double *, int *, EIGEN *, int *, int *,
	int *, double *,  double *, MATL *, int *, double *, int );

/*
int qdEConjGrad(double *, BOUND , int *, double *, int*, int *, double *, double *,
	MATL *, int *, double *);
*/

#if LAPACK
int qdLAPACK( BOUND, double *, double *, double *, double *, double *, int *, int *,
	int *, double *, double *, EIGEN *, int *, int *, int *, MATL *, char *,
	double *, double *, double *, double *, double * );
#endif

int solve(double *,double *,int *, int );

int decomp(double *,int *,int );

int qdEKassemble(double *, double *, double *, double *, double *, double *, int *,
	int *, int *, double *, int *, int *, int *, double *, double *, double *,
	int *, MATL *, double *, double *);

int qdVConjPassemble(double *, int *, double *, int *, MATL *, double *, double *);

int qdVKassemble( double *, int *, double *, int *, double *, int *, int *, double *,
	int *, MATL *, double *, double *);

int diag( int *, int *, int, int, int, int);

int formlm( int *, int *, int *, int, int, int );

int Eformid( BOUND, int *);

int edge_id( BOUND , double *, int *, int );

int qdedge( int *, int *, int *, int * );

int Ereader( BOUND , int *, int *, int *, double *, int *, double *, MATL *,
	char *, FILE *, double *, double *);

int EMemory( double **, int, int **, int, MATL **, int , XYI **, int );

int qdshl( double, double *, double * );

int qdshl_node2(double * );


int analysis_flag, dof, sdof, EMdof, Vdof, modal_flag, neqn, EMneqn, Vneqn, nmat,
	nmode, numed, numel, numnp, sof, edge_connect_read_flag;
int static_flag, AB_matrix_store, Bzz_matrix_store, eigen_print_flag;

int LU_decomp_flag, numel_EM, numel_EM_K, numel_EM_P, numel_V, numel_V_K, numel_V_P,
	numnp_LUD_max;
int iteration_max, iteration_const, iteration;
double tolerance;

double dcdx[nsdsq*num_int], shg[sosh], shg_node[sosh], shl[sosh], shg_E[sosh_E],
	shg_node_E[sosh_E], curl_E[num_int*epel], shl_node[sosh], shl_node2[sosh_node2],
	*Area0, w[num_int], sigma, low_mode;

int main(int argc, char** argv)
{
	int i, i2, j, k;
	int *id, *lm, *idiag, check, name_length, counter, MemoryCounter,
		neg_eval_counter, nmode2;
	XYI *mem_XYI;
	int *mem_int, sofmAtt_sigma, sofmAtt_sigma_K, sofmAttpBzz, sofmBzz, sofmAB,
		sofmTOTAL, sofmi, sofmf, sofmXYI, sofmSDIM, ptr_inc;
	int sofmKV;
	MATL *matl;
	double *mem_double;
	double fpointx, fpointy, fpointz;
	int *connect, *node_edge_connect, *edge_connect, *el_matl, dum;
	double *coord, *charge, *Volt, *EMedge, *EMnode, *EMmag, *Arean,
		*node_counter, *edge_counter, *vector_dum;
	double *Att, *Att_sigma, *Btt, *Btz, *Bzt, *Bzz, *TOTAL;
	double *KV, *KV_diag, *Att_diag, *Att_sigma_diag, *Bzz_diag, *ritz;
	EIGEN *eigen;
	int num_eigen;
	char name[30], buf[ BUFSIZ ];
	char name_mode[30], *ccheck;
	FILE *o1, *o2;
	BOUND bc;
	double g, fdum, fdum1, fdum2, fdum3, fdum4, length;
	int node, node1, node2;
	long timec;
	long timef;
	int  mem_case;
	double RAM_max, RAM_usable, RAM_needed, MEGS;
	double kosq, evalsq, kt;
	int numed_mem, numed2;

	sof = sizeof(double);

/* Create local shape funcions at gauss points */

	g = 2.0/sq3;
	check = qdshl( g, shl, w );
	if(!check) printf( " Problems with qdshl \n");

/* Create local shape funcions at nodal points */

	g = 2.0;
	check = qdshl( g, shl_node, w );
	if(!check) printf( " Problems with qdshl \n");

/* Create local streamlined shape funcion matrix at nodal points */

	check = qdshl_node2(shl_node2);
	if(!check) printf( " Problems with qdshl_node2 \n");

	memset(name,0,30*sizeof(char));
	
	printf("What is the name of the file containing the \n");
	printf("quad structural data? (example: diske4a)\n");
	scanf( "%30s",name);

/*   o1 contains all the structural quad data  */
/*   o2 contains input parameters */

	o1 = fopen( name,"r" );
	o2 = fopen( "qdeinput","r" );

	if(o1 == NULL ) {
		printf("Can't find file %30s\n",name);
		exit(1);
	}

	if( o2 == NULL ) {
		printf("Can't find file qdeinput\n");
		tolerance = 1.e-13;
		iteration_max = 2000;
		RAM_max = 160.0;
		eigen_print_flag = 0;
	}
	else
	{
		fgets( buf, BUFSIZ, o2 );
		fscanf( o2, "%lf\n ",&tolerance);
		fgets( buf, BUFSIZ, o2 );
		fscanf( o2, "%d\n ",&iteration_max);
		fgets( buf, BUFSIZ, o2 );
		fscanf( o2, "%lf\n ",&RAM_max);
		fgets( buf, BUFSIZ, o2 );
		fscanf( o2, "%d\n ",&dum);
		fgets( buf, BUFSIZ, o2 );
		fscanf( o2, "%d\n ",&dum);
		fgets( buf, BUFSIZ, o2 );
		fscanf( o2, "%d\n ",&dum);
		fgets( buf, BUFSIZ, o2 );
		fscanf( o2, "%d\n ",&eigen_print_flag);
	}

	fgets( buf, BUFSIZ, o1 );
	fscanf( o1, "%d %d %d %d %d %lf %lf\n",
		&numed, &numel, &numnp, &nmat, &nmode, &sigma, &low_mode);
	dof=numnp*ndof;
	sdof=numnp*nsd;
	Vdof=numnp*Vndof;

	numnp_LUD_max = 1125;
	numel_EM = numel;
	numel_V = numel;
	numel_V_K = numel;
	numel_V_P = 0;

	numed_mem = numel*epel;  /* numed_mem is for the allocation of memory.
	                            I need it because the data file may contain
	                            the edges exactly specified or they may be
	                            generated in the code.  numel*epel is the
	                            greatest numeber of edges possible.  This usually
	                            means I allocate twice the amount I really need. */
	EMdof=numed_mem*edof;    /* I set EMdof temporarily to this maximum value for
	                            memory allocation.  I set it to its true value
	                            aftor "qdedge" is potentially called.*/


/* Assuming Conjugate gradient method is used, determine how much RAM is needed.
   This determines the largest problem that can be run on this machine.
   If problem small enough that LU decomposition is used, then calculation below
   is irrelevant.

   RAM variables given in bytes

*/

#if LAPACK
/* Using LAPACK requires all the element matrices be stored and calculated.
*/
	if(nmode) if(RAM_max < 64.0) RAM_max = 64.0;
#endif

	RAM_max *= MB;
	RAM_usable = 0.5*RAM_max;

	static_flag = 1;
	modal_flag = 0;


	if(!nmode)
	{
/* FOR STATIC ANALYSIS */

	    RAM_needed = numel_V*neqlsq16*sof;

	    fdum = RAM_usable - RAM_needed;
	    if(fdum > 0.0)
	    {
/* Enough RAM to store all element Att and Bzz matrices for conjugate
   gradient
*/
		numel_V_K = numel;
		numel_V_P = 0;
	    }
	    else
	    {
/* Store numel_V_K element Att matrices while remaining numel_V_P element Att
   matrices are calculated through product [Att_el][et_el] = [P_el] */

		numel_V_P = numel_V - (((int)RAM_usable)/((double)neqlsq16*sof));
		numel_V_K = numel_V - numel_V_P;
	    }

	    LU_decomp_flag = 1;
	    if(numnp > numnp_LUD_max)
	    {
		LU_decomp_flag = 0;
	    }
	}

	if(nmode)
	{
/* FOR WAVEGUIDE ANALYSIS */

	    static_flag = 0;
	    modal_flag = 1;

	    RAM_needed = numel_EM*neqlsq16*sof;

	    fdum = RAM_usable - RAM_needed;
	    if(fdum > 0.0)
	    {
/* Enough RAM to store all element Att and Bzz matrices for conjugate
   gradient
*/
		numel_EM_K = numel;
		numel_EM_P = 0;
	    }
	    else
	    {
/* Store numel_EM_K element Att matrices while remaining numel_EM_P element Att
   matrices are calculated through product [Att_el][et_el] = [P_el] */

		numel_EM_P = numel_EM - (((int)RAM_usable)/((double)neqlsq16*sof));
		numel_EM_K = numel_EM - numel_EM_P;
	    }

	    LU_decomp_flag = 1;
	    if(numnp > numnp_LUD_max)
	    {
		LU_decomp_flag = 0;
	    }

/* The criteria for over-calculating the number of desired eigenvalues is
   taken from "The Finite Element Method" by Thomas Hughes, page 578.  It
   is actually given for subspace iteration, but I find it works well
   for the Lanczos Method as well.
*/

	    num_eigen = (int)(2.0*nmode);
	    num_eigen = MIN(nmode + 8, num_eigen);
	    num_eigen = MIN(dof, num_eigen);
	}

#if 0
	LU_decomp_flag = 0;
#endif


/*   Begin allocation of meomory */

	MemoryCounter = 0;

/* For the doubles */
	sofmf=sdof + 2*dof + Vdof + EMdof + dof + numnp + 2*numel + numnp + numed_mem +
		Vdof + 4*EMdof;
	if(modal_flag)
	{
		sofmf = sdof + 2*dof + Vdof + EMdof + dof + numnp + 2*numel + numnp +
			numed_mem + Vdof + 4*EMdof + num_eigen*EMdof;
	}
	MemoryCounter += sofmf*sizeof(double);
	printf( "\n Memory requrement for doubles is %15d bytes\n",MemoryCounter);

/* For the integers */
	sofmi = numel*npel + numed_mem*nped + numel*epel + 2*EMdof + numel*epel*edof +
		numel + numnp+1 + numed_mem+1 + nped*numed_mem+1 + 2*numnp+2 + 4;
	MemoryCounter += sofmi*sizeof(int);
	printf( "\n Memory requrement for integers is %15d bytes\n",MemoryCounter);

/* For the XYI integers */
	sofmXYI=numnp+1+1;
	MemoryCounter += sofmXYI*sizeof(XYI);
	printf( "\n Memory requrement for XYI integers is %15d bytes\n",MemoryCounter);

	check = EMemory( &mem_double, sofmf, &mem_int, sofmi, &matl, nmat,
		&mem_XYI, sofmXYI );
	if(!check) printf( " Problems with EMemory \n");

/* For the doubles */
	                                        ptr_inc=0;
	coord=(mem_double+ptr_inc);             ptr_inc += sdof;
	vector_dum=(mem_double+ptr_inc);        ptr_inc += dof;
	charge=(mem_double+ptr_inc);            ptr_inc += dof;
	Volt=(mem_double+ptr_inc);              ptr_inc += Vdof;
	EMedge=(mem_double+ptr_inc);            ptr_inc += EMdof;
	EMnode=(mem_double+ptr_inc);            ptr_inc += dof;
	EMmag=(mem_double+ptr_inc);             ptr_inc += numnp;
	Arean=(mem_double+ptr_inc);             ptr_inc += numel;
	Area0=(mem_double+ptr_inc);             ptr_inc += numel;
	node_counter=(mem_double+ptr_inc);      ptr_inc += numnp;
	edge_counter=(mem_double+ptr_inc);      ptr_inc += numed_mem;
	KV_diag=(mem_double+ptr_inc);           ptr_inc += Vdof;
	Att_diag=(mem_double+ptr_inc);          ptr_inc += EMdof;
	Att_sigma_diag=(mem_double+ptr_inc);    ptr_inc += EMdof;
	Bzz_diag=(mem_double+ptr_inc);          ptr_inc += EMdof;
	bc.EMedge=(mem_double+ptr_inc);         ptr_inc += EMdof;

/* If modal analysis is desired, allocate for ritz vectors */

	if(modal_flag)
	{
		ritz=(mem_double+ptr_inc);      ptr_inc += num_eigen*EMdof;
	}

/* For the integers */

	                                        ptr_inc = 0; 
	connect=(mem_int+ptr_inc);              ptr_inc += numel*npel; 
	node_edge_connect=(mem_int+ptr_inc);    ptr_inc += numed_mem*nped; 
	edge_connect=(mem_int+ptr_inc);         ptr_inc += numel*epel; 
	id=(mem_int+ptr_inc);                   ptr_inc += EMdof;
	idiag=(mem_int+ptr_inc);                ptr_inc += EMdof;
	lm=(mem_int+ptr_inc);                   ptr_inc += numel*epel*edof;
	el_matl=(mem_int+ptr_inc);              ptr_inc += numel;
	bc.Volt =(mem_int+ptr_inc);             ptr_inc += numnp+1;
	bc.edge =(mem_int+ptr_inc);             ptr_inc += numed_mem+1;
	bc.edge_node =(mem_int+ptr_inc);        ptr_inc += nped*numed_mem+1;
	bc.charge =(mem_int+ptr_inc);           ptr_inc += numnp+1;
	bc.EMnode =(mem_int+ptr_inc);           ptr_inc += numnp+1;
	bc.num_Volt=(mem_int+ptr_inc);          ptr_inc += 1;
	bc.num_edge=(mem_int+ptr_inc);          ptr_inc += 1;
	bc.num_charge=(mem_int+ptr_inc);        ptr_inc += 1;
	bc.num_EMnode=(mem_int+ptr_inc);        ptr_inc += 1;

/* For the XYI integers */
	                                        ptr_inc = 0; 
	bc.fix =(mem_XYI+ptr_inc);              ptr_inc += numnp+1;
	bc.num_fix=(mem_XYI+ptr_inc);           ptr_inc += 1;

/* If modal analysis is desired, allocate for the eigens */

	if(modal_flag)
	{
		eigen=(EIGEN *)calloc(num_eigen,sizeof(EIGEN));
		if(!eigen )
		{
			printf( "failed to allocate memory for eigen\n ");
			exit(1);
		}
	}

	timec = clock();
	timef = 0;

	check = Ereader( bc, connect, node_edge_connect, edge_connect, coord, el_matl,
		charge, matl, name, o1, EMedge, Volt);
	if(!check) printf( " Problems with Ereader \n");

	if(modal_flag)
	{
	    if(!edge_connect_read_flag)
	    {
		memset(node_edge_connect, -1, numed_mem*nped*sizeof(int));
		memset(edge_connect, -1, numel*epel*sizeof(int));

		check = qdedge( connect, edge_connect, node_edge_connect, &numed2 );
		if(!check) printf( " Problems with qdedge \n");

		numed2 -= 1;        /* If the edge connectivity is generated by qdedge, */
		numed = numed2;     /* set numed to numed2. */

		check = edge_id( bc, EMedge, node_edge_connect, numed2);
		if(!check) printf( " Problems with edge_id\n");

	    }
	}

/* FOR STATIC ANALYSIS */

	if(static_flag)
	{
	    check = Vformid( bc, id );
	    if(!check) printf( " Problems with Vformid \n");
/*
	    printf( "\n This is the id matrix \n");
	    for( i = 0; i < numel; ++i )
	    {
		printf("\n node(%4d)",i);
		for( j = 0; j < Vndof; ++j )
		{
			printf(" %4d  ",*(id+Vndof*i+j));
		}
	    }
*/

	    check = formlm( connect, id, lm, Vndof, npel, numel );
	    if(!check) printf( " Problems with formlm \n");
/*
	    printf( "\n\n This is the lm matrix \n");
	    for( i = 0; i < numel; ++i )
	    {
		printf("\n element(%4d)",i);
		for( j = 0; j < Vneqel4; ++j )
		{
		    printf( "%5d   ",*(lm+Vneqel4*i+j));
		}
	    }
	    printf( "\n");
*/

	    check = diag( idiag, lm, Vndof, Vneqn, npel, numel);
	    if(!check) printf( " Problems with diag \n");
/*
	    printf( "\n\n This is the idiag matrix \n");
	    for( j = 0; j < Vneqn; ++j )
	    {
		printf( "\n Vdof %5d   %5d",j,*(idiag+j));
	    }
	    printf( "\n");
*/

/* allocate meomory for A, the global stiffness.  There are 2 possibilities:

   1) Use standard Linear Algebra with LU decomposition and skyline storage
      of global stiffness matrix.

   2) Use the Conjugate Gradient method with storage of numel_V_K element
      stiffness matrices.
*/
	    sofmKV = numel_V_K*Vneqlsq16;                /* case 2 */
	    mem_case = 2;

	    if(LU_decomp_flag)
	    {
		    sofmKV = *(idiag+Vneqn-1)+1;       /* case 1 */
		    mem_case = 1;
	    }

	    if( sofmKV*sof > (int)RAM_usable )
	    {

/* Even if the linear algebra flag is on because there are only a few nodes, there
   is a possibility that there is not enough memory because of poor node numbering.
   If this is the case, then we have to use the conjugate gradient method.
*/

		    sofmKV = numel_V_K*Vneqlsq16;
		    LU_decomp_flag = 0;
		    mem_case = 2;
	    }

	    printf( "\n We are in case %3d\n\n", mem_case);
	    switch (mem_case) {
		    case 1:
			    printf( " Linear algebra\n\n");
		    break;
		    case 2:
			    printf( " Conjugate gradient \n\n");
		    break;
	    }

	    MemoryCounter += sofmKV*sizeof(double);
	    printf( "\n Memory requrement for voltage potential calc. with %15d doubles is %15d bytes\n",
		sofmKV, MemoryCounter);
	    MEGS = ((double)(MemoryCounter))/MB;
	    printf( "\n Which is %16.4e MB\n", MEGS);
	    printf( "\n This is numel, numel_V_K, numel_V_P %5d %5d %5d\n", numel, numel_V_K, numel_V_P);

	    if(sofmKV < 1) sofmKV = 2;

	    KV=(double *)calloc(sofmKV,sizeof(double));
	    if(!KV )
	    {
		printf( "failed to allocate memory for A double\n ");
		exit(1);
	    }

	    analysis_flag = 1;
	    check = qdVKassemble( KV, connect, coord, el_matl, charge, id, idiag, KV_diag,
		lm, matl, node_counter, Volt);
	    if(!check) printf( " Problems with qdVKassemble \n");

	    if(LU_decomp_flag)
	    {

/* Perform LU Crout decompostion on the system */

		check = decomp(KV,idiag,Vneqn);
		if(!check) printf( " Problems with decomp \n");

/* Using LU decomposition to solve the system */

		check = solve(KV,charge,idiag,Vneqn);
		if(!check) printf( " Problems with solve \n");

		/* printf( "\n This is the solution to the problem \n");*/
		for( i = 0; i < Vdof; ++i )
		{
		    if( *(id + i) > -1 )
		    {
			*(Volt + i) = *(charge + *(id + i));
		    }
		}
	    }
	    if(!LU_decomp_flag)
	    {
		check = qdVConjGrad(KV, bc, connect, coord, el_matl, charge, KV_diag,
		    matl, Volt);
		if(!check) printf( " Problems with qdVConjGrad\n");
	    }

/* Calculate the reaction charges */

	    analysis_flag = 2;
	    memset(charge,0,Vdof*sof);
	    check = qdVKassemble( KV, connect, coord, el_matl, charge, id, idiag, KV_diag,
		lm, matl, node_counter, Volt);
	    if(!check) printf( " Problems with qdVKassemble \n");

	    check = qdEVnode_calculate(connect, coord, Volt, EMnode, EMmag, node_counter);
	    if(!check) printf( " Problems with qdEVnode_calculate\n");

	    check = Ewriter( bc, connect, node_edge_connect, edge_connect, coord,
		    el_matl, charge, id, matl, name, EMnode, EMedge, EMmag, Volt);
	    if(!check) printf( " Problems with Ewriter \n");

/*
	    check = writer ( bc, connect, coord, el_matl, charge, id, matl, name,
		    EMnode, EMmag);
	    if(!check) printf( " Problems with writer \n");
*/

	}

	if(modal_flag)
	{
/* FOR WAVEGUIDE ANALYSIS */

	    EMdof=numed*edof;

	    printf(" \n\n");

	    check = Eformid( bc, id );
	    if(!check) printf( " Problems with Eformid \n");
/*
	    printf( "\n This is the id matrix \n");
	    for( i = 0; i < numed; ++i )
	    {
		printf("\n edge(%4d)",i);
		for( j = 0; j < edof; ++j )
		{
			printf(" %4d  ",*(id+edof*i+j));
		}
	    }
*/

	    check = formlm( edge_connect, id, lm, edof, epel, numel );
	    if(!check) printf( " Problems with formlm \n");
/*
	    printf( "\n\n This is the lm matrix \n");
	    for( i = 0; i < numel; ++i )
	    {
		printf("\n element(%4d)",i);
		for( j = 0; j < neqel4; ++j )
		{
		    printf( "%5d   ",*(lm+neqel4*i+j));
		}
	    }
	    printf( "\n");
*/
	    check = diag( idiag, lm, edof, EMneqn, epel, numel);
	    if(!check) printf( " Problems with diag \n");
/*
	    printf( "\n\n This is the idiag matrix \n");
	    for( j = 0; j < EMneqn; ++j )
	    {
		printf( "\n EMdof %5d   %5d",j,*(idiag+j));
	    }
	    printf( "\n");
*/

/*   allocate meomory for Att_sigma, the global Att_sigma matrix.  There
     are 2 possibilities:

     1) Use standard Linear Algebra with LU decomposition and skyline storage
	for Att.

     2) Use the Conjugate Gradient method with storage of numel_EM_K element
	Att matrices.
*/
	    sofmAtt_sigma = numel_EM_K*neqlsq16;   /* 2 */
	    mem_case = 2;

	    if(LU_decomp_flag)
	    {
		sofmAtt_sigma = *(idiag+EMneqn-1)+1;  /* case 1 */
	    }

	    if( sofmAtt_sigma*sof > (int)RAM_usable )
	    {

/* Even if the LU decomposition flag is on because there are only a few nodes, there
   is a possibility that there is not enough memory because of poor node numbering.
   If this is the case, then we have to use the conjugate gradient method.
 */

		sofmAtt_sigma = numel_EM_K*neqlsq16;
		LU_decomp_flag = 0;
		mem_case = 2;

	    }

	    printf( "\n We are in case %3d\n\n", mem_case);
	    switch (mem_case) {
		case 1:
			printf( " LU decomposition\n\n");
		break;
		case 2:
			printf( " Conjugate gradient \n\n");
		break;
	    }

/* If modal analysis is desired, determine how much memory is needed and available
   for Att, Btt, Btz, Bzt, Bzz matrices */

	    sofmAtt_sigma_K = sofmAtt_sigma;              /* mass case 2 */
	    sofmAB = 0;
	    AB_matrix_store = 0;
	    mem_case = 2;

/* determine how much memory is needed and available for the Btt, Btz, Bzt matrices. 
   We want enough memory to store all element matrices of Btt, Btz, Bzt
*/

	    sofmTOTAL = sofmAtt_sigma_K;
	    fdum = RAM_usable - (double)(sof*(sofmAtt_sigma + 5*numel*neqlsq16));
	    if(fdum > 0.0)
	    {
/* Enough RAM to store all element Att, Btt, Btz, Bzt, Bzz matrices for modal analysis */

		sofmAB = 5*numel_EM*neqlsq16;      /* B case 1 */
		sofmTOTAL += sofmAB;
		AB_matrix_store = 1;
		mem_case = 1;

	    }

	    printf( "\n We are in B case %3d\n\n", mem_case);
	    switch (mem_case) {
		case 1:
		    printf( "\n all element Att, Btt, Btz, Bzt, Bzz matrices stored \n\n");
		break;
		case 2:
		    printf( "\n no storage of element Att, Btt, Btz, Bzt, Bzz matrices \n\n");
		break;
	    }

	    MemoryCounter += sofmTOTAL*sizeof(double);
	    printf( "\n Memory requrement for electric field calc. with %15d doubles is %15d bytes\n",
		sofmAtt_sigma, MemoryCounter);
	    MEGS = ((double)(MemoryCounter))/MB;
	    printf( "\n Which is %16.4e MB\n", MEGS);
	    printf( "\n This is numel, numel_EM_K, numel_EM_P %5d %5d %5d\n", numel, numel_EM_K, numel_EM_P);

#if 0
	    AB_matrix_store = 0;
#endif

	    if(sofmTOTAL < 1)
	    {
		sofmTOTAL = 6;
		sofmAtt_sigma = 1;
		sofmAB = 5;
	    }

	    TOTAL = (double *)calloc(sofmTOTAL,sizeof(double));
	    if(!TOTAL )
	    {
		printf( "failed to allocate memory for TOTAL double\n ");
		exit(1);
	    }

	    if (AB_matrix_store)
	    {
/* Allocate memory for Att_sigma, Att, Btt, Btz, Bzt, Bzz matrices */

	                                          ptr_inc = 0;
		Att_sigma = (TOTAL + ptr_inc);    ptr_inc += sofmAtt_sigma;

/* Note that I do not have to store Att.  I am keeping it in case it is
   needed in future use.
*/
		Att = (TOTAL + ptr_inc);          ptr_inc += numel_EM*neqlsq16;
		Btt = (TOTAL + ptr_inc);          ptr_inc += numel_EM*neqlsq16;
		Btz = (TOTAL + ptr_inc);          ptr_inc += numel_EM*neqlsq16;
		Bzt = (TOTAL + ptr_inc);          ptr_inc += numel_EM*neqlsq16;
		Bzz = (TOTAL + ptr_inc);          ptr_inc += numel_EM*neqlsq16;
	    }
	    else
	    {
	                                          ptr_inc = 0;
		Att_sigma = (TOTAL + ptr_inc);    ptr_inc += 1;
		Att = (TOTAL + ptr_inc);          ptr_inc += 1;
		Btt = (TOTAL + ptr_inc);          ptr_inc += 1;
		Btz = (TOTAL + ptr_inc);          ptr_inc += 1;
		Bzt = (TOTAL + ptr_inc);          ptr_inc += 1;
		Bzz = (TOTAL + ptr_inc);          ptr_inc += 1;
	    }

	    printf("eval \n");

#if LAPACK
	    LU_decomp_flag = 0;
	    AB_matrix_store = 1;
#endif

	    analysis_flag = 1;
	    memset(TOTAL,0,sofmTOTAL*sof);
	    check = qdEKassemble(Att, Att_sigma, Btt, Btz, Bzt, Bzz, connect, node_edge_connect,
		edge_connect, coord, el_matl, id, idiag, Att_diag, Att_sigma_diag,
		Bzz_diag, lm, matl, node_counter, edge_counter);
	    if(!check) printf( " Problems with qdEKassemble \n");

/*
	    for( j = 0; j < numel*neqlsq16; ++j )
	    {
		printf( "\n EMdof %5d   %14.5f %14.5f %14.5f",j,*(Btt+j),*(Bzt+j),*(Btz+j));
	    }
*/


/*
	    printf( "\n\n This is the charge matrix \n");
	    for( j = 0; j < neqn; ++j )
	    {
		printf( "\n EMdof %5d   %14.5f",j,*(charge+j));
	    }
	    printf(" \n");
*/
	    if(LU_decomp_flag)
	    {

/* Perform LU Crout decompostion on the system */

		check = decomp(Att_sigma,idiag,EMneqn);
		if(!check) printf( " Problems with decomp \n");

		/*check = decomp(Bzz,idiag,EMneqn);
		if(!check) printf( " Problems with decomp \n");*/

/*
		for( i = 0; i < *(idiag+EMneqn-1)+1; ++i )
		{
		    printf( "\n Att[%3i] = %12.5e  Btt[%3i] = %12.5e",i,*(Att+i),i,*(Bzz+i));
		}
*/
		printf(" \n");
	    }

#if LAPACK

/* To test the code, I assembled the full global Att, Bzz, Btt, Btz, Bzt matrices and solved them
   using LAPACK(actually the C version CLAPACK).
*/

	    check = qdLAPACK( bc, Att, Btt, Btz, Bzt, Bzz, connect, node_edge_connect,
		edge_connect, charge, coord, eigen, el_matl, id, lm, matl, name, node_counter,
		EMedge, EMnode, EMmag, Volt);
	    if(!check) printf( " Problems with qdLAPACK\n");

#endif

#if !LAPACK
/* Calculate the eigenmodes using the solvers that are part of SLFCEM.  */

	    counter = 0;

	    name_length = strlen(name);
	    if( name_length > 20) name_length = 20;

	    memset(name_mode,0,30*sizeof(char));

/* name_mode is the name of the output data file for the mode shapes.
   It changes based on the number of the eigenmode.
*/
	    ccheck = strncpy(name_mode, name, name_length);
	    if(!ccheck) printf( " Problems with strncpy \n");

/* Number of calculated eigenvalues cannot exceed EMneqn */
	    num_eigen = MIN(EMneqn, num_eigen);
	    printf("\n The number of eigenvalues is: %5d \n", num_eigen);

/* nmode cannot exceed num_eigen */
	    nmode = MIN(num_eigen, nmode);

	    if(num_eigen < 2)
	    {
		printf("\n There is only one DOF \n");
		printf("\n so no modal analysis is performed.\n");
		exit(1);
	    }

/* Use Lanczos method for determining eigenvalues */

	    /*printf("\n hhhhh\n");*/

	    check = qdELanczos(Att_sigma, bc, connect, coord, edge_connect, eigen,
		el_matl, id, idiag, Att_sigma_diag, Btt, matl, node_edge_connect,
		ritz, num_eigen);
	    if(!check) printf( " Problems with qdELanczos \n");

/* Apply the sigma shift to the eigenvalues based on equation 10.6.5 of Hughes on page 583.
   Since the eigenvalues are not positive definite for EM systems(The system seems positive
   semi-definite), I check to see if the resulting eigenvalues after the shift are positive.
   These are the only non-trivial solutions.
*/
	    neg_eval_counter = 0;
	    for( j = 0; j < num_eigen; ++j )
	    {
		eigen[j].val += sigma;
		if( eigen[j].val < low_mode ) ++neg_eval_counter; 
	    }

	    nmode2 = nmode;
	    if( nmode + neg_eval_counter > num_eigen) nmode2 = num_eigen - neg_eval_counter;

/* Write out the eigenvalues to a file */

	    check = eval_data_print( eigen + neg_eval_counter, name, nmode);
	    if(!check) printf( " Problems with eval_data_print \n");

/* Write out the eigenmodes to a (name).mod-x.oqd file */

	    for( j = 0; j < nmode2; ++j )
	    {
		/*printf( "\n mode %4d\n", j+1 );*/
		/*printf( "\n eigen index = %4d", eigen[j].index);*/
		for( i = 0; i < EMdof; ++i )
		{
		    if( *(id + i) > -1 )
		    {
			/* *(EMedge + i) = *(ritz + num_eigen*(*(id + i))+4); */
			*(EMedge + i) = *(ritz + num_eigen*(*(id + i)) + j + neg_eval_counter);
			/*printf( "\n %4d EMedge[%4d] = %16.8e", *(id + i), i, *(EMedge+i));*/
		    }
		    /*printf( "EMedge %4d = %16.8e", i, *(EMedge + i));
		    printf( "    nodes  %4d %4d %4d\n", i,
			*(node_edge_connect+nped*i), *(node_edge_connect+nped*i+1));*/
		}

		printf( "\n Eigenvalue %4d = %16.8e",j+1,eigen[j + neg_eval_counter].val);

/* write the number of the eigenmode onto the name of the output file, name_mode */

		sprintf((name_mode+name_length+5), "%d",j+1);
		if(j + 1 > 9 )
			sprintf((name_mode+name_length+5), "%2d",j+1);
		if(j + 1 > 99 )
			sprintf((name_mode+name_length+5), "%3d",j+1);

		ccheck = strncpy(name_mode+name_length, ".mod-", 5);
		if(!ccheck) printf( " Problems with strncpy \n");

		check = qdEMnode_calculate(connect, edge_connect, node_edge_connect, coord,
		    EMedge, EMnode, EMmag, node_counter);
		if(!check) printf( " Problems with qdEMnode_calculate \n");

		check = Ewriter( bc, connect, node_edge_connect, edge_connect, coord,
		    el_matl, charge, id, matl, name_mode, EMnode, EMedge, EMmag, Volt);
		if(!check) printf( " Problems with Ewriter \n");

/*
		check = writer ( bc, connect, coord, el_matl, charge, id, matl, name_mode,
		    EMnode, EMmag);
		if(!check) printf( " Problems with writer \n");
*/

		++counter;

	    }
#endif

	}

	timec = clock();
	printf("\n\n elapsed CPU = %lf\n\n",( (double)timec)/800.);

	free(matl);
	free(mem_double);
	free(mem_int);
	free(mem_XYI);
	if(modal_flag)
	{
		free(eigen);
		free(TOTAL);
	}
}

