/*
    This program generates the mesh for a full hollow disk(Also called annulus)
    with an inner and outer radius. 

		Updated 11/12/09

    SLFFEA source file
    Version:  1.6
    Copyright (C) 1999-2009  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define pi              3.141592654

int main(int argc, char** argv)
{
	int dof, nmat, numel, numed, numnp, plane_stress_flag, edge_conncet_write_flag;
	int i,j,dum,dum2,dum3,dum4;
	double fdum1, fdum2, fdum3, ux[9], uy[9];
	double angle, side_length, ray_angle, el_length, R1, R2, sigma, low_mode;
	int side_el_num, angle_div, angleh_div, el_type;
	char name[20];
	char buf[ BUFSIZ ];
	FILE *o1;
	char text;

	printf( "\n What is the inner Radius?\n ");
	scanf( "%lf",&R1);
	printf( "\n What is the outer Radius?\n ");
	scanf( "%lf",&R2);
	if( R2 <= R1 )
	{
		printf( "\n R2 must be larger than R1.\n ");
		exit(1);
	}
	printf( "\n How many elements on one side?\n ");
	scanf( "%d",&side_el_num);
	printf( "\n What angle(in degrees) between rays?\n");
	printf( "Angle should be < 360 and a factor of 360.\n ");
	scanf( "%lf",&ray_angle);
	printf("What type of element is this for?\n");
	printf("1) quad \n");
	printf("2) quad for electro-magnetics\n");
	scanf( "%d",&el_type);
	if(el_type > 1)
	{
		printf("Do you want the edge connection data?\n");
		printf("0 = no, 1 = yes.\n");
		scanf( "%d",&edge_conncet_write_flag);
	}


	if(el_type < 2 ) o1 = fopen( "disk","w" );
	else	     o1 = fopen( "diske","w" );

	angle_div = (int)(360.0/ray_angle);
	angleh_div = (int)(((double)angle_div)/2.0);
	ray_angle *= pi/180.0;

	nmat = 1; 
	plane_stress_flag = 1;
	sigma = 1.0;
	low_mode = 0.10;
	numel = side_el_num*(angle_div);
	numed = (side_el_num+1)*(angle_div) + (side_el_num)*(angle_div);
	numnp = (side_el_num+1)*(angle_div);
	el_length = (R2-R1)/((double)side_el_num);
	printf( "\n %16.8e\n", el_length);

	if(el_type < 2 )
	{
	    fprintf( o1, "   numel numnp nmat nmode plane_stress_flag  This is for the disk \n ");
	    fprintf( o1, "    %4d %4d %4d %4d %4d \n ", numel,numnp,nmat,0,plane_stress_flag);
	}

	if(el_type > 1 )
	{
	    fprintf( o1, "   numed numel numnp nmat nmode sigma low_mode");
	    fprintf( o1, "  This is for the disk waveguide problem \n ");
	    fprintf( o1, "  %4d %4d %4d %4d %4d %lf %lf\n",
		numed, numel, numnp, nmat, 5, sigma, low_mode);
	}

	printf( "    %4d %4d %4d %4d %4d %4d \n ", numel,numnp,nmat,numed, 0,plane_stress_flag);

	if(el_type < 2 )
	{
	    fprintf( o1, "matl no., E modulus, Poisson Ratio, density \n");
	    for( i = 0; i < nmat; ++i )
	    {
		fprintf( o1, "%3d ",0);
		fprintf( o1, " %9.4f %9.4f     %9.4f \n ",10000000.0, 0.3, 2.77e3);
	    }
	}


	if(el_type > 1 )
	{
	    fprintf( o1, "matl no., permeability, permittivity, operating fq\n");
	    for( i = 0; i < nmat; ++i )
	    {
		fprintf( o1, "%3d ",0);
		fprintf( o1, " %9.4f %9.4f %9.4f\n", 1.0, 1.0, 40.0307);
	    }
	}

	if(el_type > 1 )
	{
	    if(edge_conncet_write_flag)
	    {
		dum = 0;
		fprintf( o1, "edge no., node connectivity\n");
		for( i = 0; i < angle_div-1; ++i )
		{
		    for( j = 0; j < side_el_num; ++j )
		    {
			fprintf( o1, "%4d    %4d %4d",dum, i*(side_el_num+1) + j,
				i*(side_el_num+1)+j+1 ); 
			fprintf( o1, "\n");
			++dum;
		    }

		    for( j = 0; j < side_el_num + 1; ++j )
		    {
			fprintf( o1, "%4d    %4d %4d",dum,i*(side_el_num+1) + j,
				(i+1)*(side_el_num+1)+j);
			fprintf( o1, "\n");
			++dum;
		    }
		}

/* This is the last set of edges which have to be coded separately because
   their connectivity involve the first set of nodes.
*/

		i = angle_div-1;
		for( j = 0; j < side_el_num; ++j )
		{
		    fprintf( o1, "%4d    %4d %4d",dum, i*(side_el_num+1) + j,
			    i*(side_el_num+1)+j+1 ); 
		    fprintf( o1, "\n");
		    ++dum;
		}
		for( j = 0; j < side_el_num + 1; ++j )
		{
		    fprintf( o1, "%4d    %4d %4d",dum,i*(side_el_num+1) + j,
			    j);
		    fprintf( o1, "\n");
		    ++dum;
		}

	    }
	}


	dum = 0;
	if(el_type < 2 )
		fprintf( o1, "el no., connectivity, matl no. \n");
	if(el_type > 1 )
		fprintf( o1, "el no., node connectivity, edge connectivity, matl no.\n");
	for( i = 0; i < angle_div-1; ++i )
	{
		for( j = 0; j < side_el_num; ++j )
		{
			fprintf( o1, "%4d ",dum);
			fprintf( o1, " %4d %4d %4d %4d",i*(side_el_num+1) + j,
				i*(side_el_num+1)+j+1, (i+1)*(side_el_num+1)+j+1,
				(i+1)*(side_el_num+1)+j);
			if(el_type > 1)
			{
			    if(edge_conncet_write_flag)
			    {
				fprintf( o1, "    %4d %4d %4d %4d", i*(2*side_el_num+1) + j,
				    (i+1)*(2*side_el_num+1)+j,
				    i*(2*side_el_num+1)+side_el_num+j,
				    i*(2*side_el_num+1)+side_el_num+j+1);
			    }
			}
			fprintf( o1, "%4d ",0);
			fprintf( o1, "\n");
			++dum;
		}
	}


/* This is the last set of elements which have to be coded separately because
   their connectivity involve the first set of nodes.
*/

	i = angle_div-1;
	for( j = 0; j < side_el_num; ++j )
	{
		fprintf( o1, "%4d ",dum);
		fprintf( o1, " %4d %4d %4d %4d", i*(side_el_num+1) + j,
			i*(side_el_num+1)+j+1, j+1, j);
		if(el_type > 1)
		{
		    if(edge_conncet_write_flag)
		    {
			fprintf( o1, "    %4d %4d %4d %4d", i*(2*side_el_num+1) + j,
			    j,
			    i*(2*side_el_num+1)+side_el_num+j,
			    i*(2*side_el_num+1)+side_el_num+j+1);
		    }
		}
		fprintf( o1, "%4d ",0);
		fprintf( o1, "\n");
		++dum;
	}


	dum = 0;
	fdum3 = 0.0;
	fprintf( o1, "node no., coordinates \n");
	for( i = 0; i < angle_div; ++i )
	{
		angle = ray_angle*fdum3;
		for( j = 0; j < side_el_num + 1; ++j )
		{
			fdum1 = (R1 + el_length*((double)j))*cos(angle);
			fdum2 = (R1 + el_length*((double)j))*sin(angle);
			fprintf( o1, "%d ",dum);
			fprintf( o1, "%9.4f %9.4f ",fdum1,fdum2);
			fprintf( o1, "\n");
			++dum;
		}
		fdum3 += 1.0;
	}

	if(el_type < 2)
	{
	    dum= 0;
	    fprintf( o1, "prescribed displacement x: node  disp value\n");
	    for( i = 0; i < angle_div; ++i )
	    {
		fprintf( o1, "%4d %14.6e\n", i*(side_el_num+1), 0.0);
		++dum;
	    }
	    fprintf( o1, "%4d\n ",-10);

	    fprintf( o1, "prescribed displacement y: node  disp value\n");
	    for( i = 0; i < angle_div; ++i )
	    {
		fprintf( o1, "%4d %14.6e\n", i*(side_el_num+1), 0.0);
		++dum;
	    }
	    fprintf( o1, "%4d\n ",-10);

	    fprintf( o1, "node with point load and load vector in x,y\n");

	    j = side_el_num;
	    dum = j;
	    fdum3 = 0.0;
	    for( i = 0; i < angle_div; ++i )
	    {
		angle = ray_angle*fdum3;
		fdum1 = (R1 + el_length*((double)j))*cos(angle);
		fdum2 = (R1 + el_length*((double)j))*sin(angle);
		fprintf( o1, "%d ",dum);
		fprintf( o1, "%9.4f %9.4f ",fdum1,fdum2);
		fprintf( o1, "\n");
		dum += j+1;
		fdum3 += 1.0;
	    }
#if 0
/* This code puts the charges on the nodes of the inner radius, R1. */

	    dum = 0;
	    fdum3 = 0.0;
	    for( i = 0; i < angle_div; ++i )
	    {
		angle = ray_angle*fdum3;
		j = side_el_num;
		fdum1 = (R1 + el_length*((double)j))*cos(angle);
		fdum2 = (R1 + el_length*((double)j))*sin(angle);
		fprintf( o1, "%d ",dum);
		fprintf( o1, "%9.4f %9.4f ",fdum1,fdum2);
		fprintf( o1, "\n");
		dum += j+1;
		fdum3 += 1.0;
	    }
#endif

	    fprintf( o1, "%4d\n ",-10);
	    fprintf( o1, "element and gauss pt. with stress and stress vector in xx,yy,xy\n");
	    fprintf( o1, "%4d ",-10);
	}

	if(el_type > 1 )
	{
	    if(edge_conncet_write_flag)
	    {
		fprintf( o1, "prescribed edge: edge value\n");
		for( i = 0; i < angle_div; ++i )
		{
			j = 0;
			fprintf( o1, "    %4d %9.4f\n", i*(2*side_el_num+1)+side_el_num+j, 0.0);
			j = side_el_num-1;
			fprintf( o1, "    %4d %9.4f\n", i*(2*side_el_num+1)+side_el_num+j+1, 0.0);
		}
	    }
	    else
	    {
		fprintf( o1, "prescribed edge: node1, node2 edge value\n");
		for( i = 0; i < angle_div-1; ++i )
		{
			j = 0;
			fprintf( o1, " %4d %4d %9.4f\n",i*(side_el_num+1) + j, (i+1)*(side_el_num+1)+j, 0.0);
			j = side_el_num;
			fprintf( o1, " %4d %4d %9.4f\n",i*(side_el_num+1) + j, (i+1)*(side_el_num+1)+j, 0.0);
		}

		i = angle_div-1;
		j = 0;
		fprintf( o1, " %4d %4d %9.4f\n",i*(side_el_num+1) + j, j, 0.0);
		j = side_el_num;
		fprintf( o1, " %4d %4d %9.4f\n",i*(side_el_num+1) + j, j, 0.0);
	    }

	    fprintf( o1, "%4d\n ",-10);


	    fprintf( o1, "node with point charge\n");
	    for( i = 0; i < side_el_num+1; ++i )
	    {
		dum = (side_el_num+1)*i + side_el_num;
		fprintf( o1, "%4d %14.6e\n",dum,100000.0);
	    }
	    fprintf( o1, "%4d\n ",-10);
	    fprintf( o1, "Electric Field: node, x and y component, magnitude\n");
	    fprintf( o1, "%4d ",-10);
	}

	fprintf( o1, "\n\n\n\n");
	fprintf( o1, "\n What is the inner Radius?\n ");
	fprintf( o1, "%16.8e",R1);
	fprintf( o1, "\n What is the outer Radius?\n ");
	fprintf( o1, "%16.8e",R2);
	fprintf( o1, "\n How many elements on one side?\n ");
	fprintf( o1, "%4d",side_el_num);
	fprintf( o1, "\n What angle(in degrees) between rays?\n");
	fprintf( o1, "Angle should be < 360 and a factor of 360.\n ");
	fprintf( o1, "%16.8e",ray_angle);
	fprintf( o1,"What type of element is this for?\n");
	fprintf( o1,"1) quad \n");
	fprintf( o1,"2) quad for electro-magnetics\n");
	fprintf( o1, "%4d",el_type);
	if(el_type > 1)
	{
		fprintf( o1,"Do you want the edge connection data?\n");
		fprintf( o1,"0 = no, 1 = yes.\n");
		fprintf( o1, "%4d",edge_conncet_write_flag);
	}

	return 1;
}

