/*
    This program generates the mesh for a solid rectangle rather than disk
    or square.  It was originally for the point charge electrostatics problem,
    but it will also generate a wavegude mesh for a solid disk.

		Updated 1/21/10

    SLFCEM source file
    Version:  1.1
    Copyright (C) 1999-2010  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define pi              3.141592654
#define SMALL           1.0e-20

int main(int argc, char** argv)
{
	int dof, nmat, numel, numed, numnp, nmode,
		plane_stress_flag, edge_conncet_write_flag, modal_flag;
	int adjust_flag;
	int i,j,dum,dum2,dum3,dum4;
	double fdum1, fdum2, fdum3, fdum4, fdum5, fdum6, ux[9], uy[9];
	double angle, side_length, ray_angle, el_length, R1, R2, R22, sigma, low_mode;
	double length, width;
	int side_el_num, angle_div, angleh_div, el_type;
	char name[20];
	char buf[ BUFSIZ ];
	FILE *o1;
	char text;

	printf( "\n What is the length?\n ");
	scanf( "%lf",&length);
	printf( "\n What is the width?\n ");
	scanf( "%lf",&width);
	printf( "\n How many elements on one side?\n ");
	scanf( "%d",&side_el_num);
	printf( "\n What angle(in degrees) between rays?\n");
	printf( "Angle should be < 360 and a factor of 360.\n ");
	scanf( "%lf",&ray_angle);
	printf("What type of element is this for?\n");
	printf("1) quad \n");
	printf("2) quad for electro-magnetics\n");
	scanf( "%d",&el_type);
	nmode = 0;
	if(el_type > 1)
	{
		printf("Electrostatics or Waveguide?\n");
		printf("0 = Electrostatics, 1 = Waveguide\n");
		scanf( "%d",&nmode);
		if(nmode) modal_flag = 1;
		edge_conncet_write_flag = 0;
		if(modal_flag)
		{
		    printf("Do you want the edge connection data?\n");
		    printf("0 = no, 1 = yes.\n");
		    scanf( "%d",&edge_conncet_write_flag);
		}
	}

	R2 = length;
	if( R2 > width) R2 = width;
	R1 = R2/(4.0*(double)(side_el_num));
	/*R1 = R2/(100.0*(double)(side_el_num));*/

	if(el_type < 2 ) o1 = fopen( "mon","w" );
	else	     o1 = fopen( "mono","w" );

	angle_div = (int)(360.0/ray_angle);
	angleh_div = (int)(((double)angle_div)/2.0);
	ray_angle *= pi/180.0;

	nmat = 1; 
	plane_stress_flag = 1;
	sigma = 1.0;
	low_mode = 0.90;
	numel = side_el_num*(angle_div) + angle_div/2;
	numed = (side_el_num+1)*(angle_div) + (side_el_num)*(angle_div) + angle_div/2;
	numnp = (side_el_num+1)*(angle_div) + 1;
	el_length = (R2-R1)/((double)side_el_num);
	printf( "\n %16.8e\n", el_length);

	if(el_type < 2 )
	{
	    fprintf( o1, "   numel numnp nmat nmode plane_stress_flag  This is for the disk \n ");
	    fprintf( o1, "    %4d %4d %4d %4d %4d \n ", numel,numnp,nmat,0,plane_stress_flag);
	}

	if(el_type > 1 )
	{
	    fprintf( o1, "   numed numel numnp nmat nmode sigma low_mode");
	    fprintf( o1, "  This is for the disk waveguide problem \n ");
	    fprintf( o1, "  %4d %4d %4d %4d %4d %lf %lf\n",
		numed, numel, numnp, nmat, nmode, sigma, low_mode);
	}

	printf( "    %4d %4d %4d %4d %4d %4d \n ", numel,numnp,nmat,numed, 0,plane_stress_flag);

	if(el_type < 2 )
	{
	    fprintf( o1, "matl no., E modulus, Poisson Ratio, density \n");
	    for( i = 0; i < nmat; ++i )
	    {
		fprintf( o1, "%3d ",0);
		fprintf( o1, " %9.4f %9.4f     %9.4f \n ",10000000.0, 0.3, 2.77e3);
	    }
	}


	if(el_type > 1 )
	{
	    fprintf( o1, "matl no., permeability, permittivity, operating fq\n");
	    for( i = 0; i < nmat; ++i )
	    {
		fprintf( o1, "%3d ",0);
		fprintf( o1, " %9.4f %9.4f %9.4f\n", 1.0, 1.0, 40.0307);
	    }
	}

	if(modal_flag)
	{
	  if(el_type > 1 )
	  {
	    if(edge_conncet_write_flag)
	    {
		dum = 0;
		fprintf( o1, "edge no., node connectivity\n");
		for( i = 0; i < angle_div-1; ++i )
		{
		    for( j = 0; j < side_el_num; ++j )
		    {
			fprintf( o1, "%4d    %4d %4d",dum, i*(side_el_num+1) + j,
				i*(side_el_num+1)+j+1 ); 
			fprintf( o1, "\n");
			++dum;
		    }

		    for( j = 0; j < side_el_num + 1; ++j )
		    {
			fprintf( o1, "%4d    %4d %4d",dum,i*(side_el_num+1) + j,
				(i+1)*(side_el_num+1)+j);
			fprintf( o1, "\n");
			++dum;
		    }
		}

/* This is the last set of edges which have to be coded separately because
   their connectivity involve the first set of nodes.
*/
	
		i = angle_div-1;
		for( j = 0; j < side_el_num; ++j )
		{
		    fprintf( o1, "%4d    %4d %4d",dum, i*(side_el_num+1) + j,
			    i*(side_el_num+1)+j+1 ); 
		    fprintf( o1, "\n");
		    ++dum;
		}
		for( j = 0; j < side_el_num + 1; ++j )
		{
		    fprintf( o1, "%4d    %4d %4d",dum,i*(side_el_num+1) + j,
			    j);
		    fprintf( o1, "\n");
		    ++dum;
		}

/* These are for the edges that make up the center disk. */

		for( i = 0; i < angle_div-1; i+=2 )
		{
		    j = 0; 
		    fprintf( o1, "%4d    %4d %4d",dum, (side_el_num+1)*(angle_div),
			i*(side_el_num+1) + j,
			i*(side_el_num+1)+j+1 ); 
		    fprintf( o1, "\n");
		    ++dum;
		}
	    }
	  }
	}


	dum = 0;
	if(el_type < 2 )
		fprintf( o1, "el no., connectivity, matl no. \n");
	if(el_type > 1 )
		fprintf( o1, "el no., node connectivity, edge connectivity, matl no.\n");
	for( i = 0; i < angle_div-1; ++i )
	{
		for( j = 0; j < side_el_num; ++j )
		{
			fprintf( o1, "%4d ",dum);
			fprintf( o1, " %4d %4d %4d %4d",i*(side_el_num+1) + j,
				i*(side_el_num+1)+j+1, (i+1)*(side_el_num+1)+j+1,
				(i+1)*(side_el_num+1)+j);
			if(el_type > 1)
			{
			  if(modal_flag)
			  {
			    if(edge_conncet_write_flag)
			    {
				fprintf( o1, "    %4d %4d %4d %4d", i*(2*side_el_num+1) + j,
				    (i+1)*(2*side_el_num+1)+j,
				    i*(2*side_el_num+1)+side_el_num+j,
				    i*(2*side_el_num+1)+side_el_num+j+1);
			    }
			  }
			}
			fprintf( o1, "%4d ",0);
			fprintf( o1, "\n");
			++dum;
		}
	}

/* This is the last set of elements which have to be coded separately because
   their connectivity involve the first set of nodes.
*/
	
	i = angle_div-1;
	for( j = 0; j < side_el_num; ++j )
	{
		fprintf( o1, "%4d ",dum);
		fprintf( o1, " %4d %4d %4d %4d", i*(side_el_num+1) + j,
			i*(side_el_num+1)+j+1, j+1, j);
		if(el_type > 1)
		{
		  if(modal_flag)
		  {
		    if(edge_conncet_write_flag)
		    {
			fprintf( o1, "    %4d %4d %4d %4d", i*(2*side_el_num+1) + j,
			    j,
			    i*(2*side_el_num+1)+side_el_num+j,
			    i*(2*side_el_num+1)+side_el_num+j+1);
		    }
		  }
		}
		fprintf( o1, "%4d ",0);
		fprintf( o1, "\n");
		++dum;
	}


/* These are for the elements that make up the center disk. */

	dum2 = 0;
	for( i = 0; i < angle_div - 2; i+=2 )
	{
		j = 0;
		fprintf( o1, "%4d ",dum);
		fprintf( o1, " %4d %4d %4d %4d", (side_el_num+1)*(angle_div),
			i*(side_el_num+1) + j, (i+1)*(side_el_num+1)+j,
			(i+2)*(side_el_num+1) + j);
		if(el_type > 1)
		{
		  if(modal_flag)
		  {
		    if(edge_conncet_write_flag)
		    {
			fprintf( o1, "    %4d %4d %4d %4d", 
			    (side_el_num+1)*(angle_div) + (side_el_num)*(angle_div) + dum2,
			    (i+1)*(2*side_el_num+1)+side_el_num+j,
			    (side_el_num+1)*(angle_div) + (side_el_num)*(angle_div) + dum2+1,
			    i*(2*side_el_num+1)+side_el_num+j);
		    }
		    ++dum2;
		  }
		}
		fprintf( o1, "%4d ",0);
		fprintf( o1, "\n");
		++dum;
	}

/* This is the last element which has to be coded separately because
   their connectivity involve the first set of nodes.
*/
	i = angle_div - 2;
	j = 0;
	fprintf( o1, "%4d ",dum);
	fprintf( o1, " %4d %4d %4d %4d", (side_el_num+1)*(angle_div),
		i*(side_el_num+1) + j, (i+1)*(side_el_num+1)+j, 0);
	if(el_type > 1)
	{
	  if(modal_flag)
	  {
	    if(edge_conncet_write_flag)
	    {
		fprintf( o1, "    %4d %4d %4d %4d", 
		    (side_el_num+1)*(angle_div) + (side_el_num)*(angle_div) + dum2,
		    (i+1)*(2*side_el_num+1)+side_el_num+j,
		    (side_el_num+1)*(angle_div) + (side_el_num)*(angle_div),
		    i*(2*side_el_num+1)+side_el_num+j);
	    }
	    ++dum2;
	  }
	}
	fprintf( o1, "%4d ",0);
	fprintf( o1, "\n");
	++dum;


	dum = 0;
	fdum3 = 0.0;
	adjust_flag = 1;
	fprintf( o1, "node no., coordinates \n");
	for( i = 0; i < angle_div; ++i )
	{
		angle = ray_angle*fdum3;
		for( j = 0; j < side_el_num + 1; ++j )
		{

			R22 = length/fabs((cos(angle) + SMALL));
			fdum2 = fabs(R22*sin(angle));
			if(fdum2 >= width ) R22 = width/fabs((sin(angle) + SMALL));

			fdum4 = ((double)(j)/(double)(side_el_num));
			fdum5 = fdum4*fdum4;
			/*fdum5 = fdum4*fdum4*fdum4;*/
			fdum6 = (R22 - R1)*fdum5;
			fdum1 = (R1 + fdum6)*cos(angle);
			fdum2 = (R1 + fdum6)*sin(angle);
			fprintf( o1, "%d ",dum);
			if(j == 0)
			{
				if(adjust_flag)
				{
					fdum1 *= .5;
					fdum2 *= .5;
				}
			}
			fprintf( o1, "%9.4f %9.4f ",fdum1,fdum2);
			fprintf( o1, "\n");
			++dum;
		}
		fdum3 += 1.0;
		adjust_flag = 1 - adjust_flag;
	}

	fprintf( o1, "%d ",dum);
	fprintf( o1, "%9.4f %9.4f ", 0.0, 0.0);
	fprintf( o1, "\n");
	++dum;

	if(el_type < 2)
	{
	    dum= 0;
	    fprintf( o1, "prescribed displacement x: node  disp value\n");
	    for( i = 0; i < angle_div; ++i )
	    {
		fprintf( o1, "%4d %14.6e\n", i*(side_el_num+1), 0.0);
		++dum;
	    }
	    fprintf( o1, "%4d\n ",-10);

	    fprintf( o1, "prescribed displacement y: node  disp value\n");
	    for( i = 0; i < angle_div; ++i )
	    {
		fprintf( o1, "%4d %14.6e\n", i*(side_el_num+1), 0.0);
		++dum;
	    }
	    fprintf( o1, "%4d\n ",-10);

	    fprintf( o1, "node with point load and load vector in x,y\n");

	    j = side_el_num;
	    dum = j;
	    fdum3 = 0.0;
	    for( i = 0; i < angle_div; ++i )
	    {
		angle = ray_angle*fdum3;
		fdum1 = (R1 + el_length*((double)j))*cos(angle);
		fdum2 = (R1 + el_length*((double)j))*sin(angle);
		fprintf( o1, "%d ",dum);
		fprintf( o1, "%9.4f %9.4f ",fdum1,fdum2);
		fprintf( o1, "\n");
		dum += j+1;
		fdum3 += 1.0;
	    }
#if 0
/* This code puts the charges on the nodes of the inner radius, R1. */

	    dum = 0;
	    fdum3 = 0.0;
	    for( i = 0; i < angle_div; ++i )
	    {
		angle = ray_angle*fdum3;
		j = side_el_num;
		fdum1 = (R1 + el_length*((double)j))*cos(angle);
		fdum2 = (R1 + el_length*((double)j))*sin(angle);
		fprintf( o1, "%d ",dum);
		fprintf( o1, "%9.4f %9.4f ",fdum1,fdum2);
		fprintf( o1, "\n");
		dum += j+1;
		fdum3 += 1.0;
	    }
#endif

	    fprintf( o1, "%4d\n ",-10);
	    fprintf( o1, "element and gauss pt. with stress and stress vector in xx,yy,xy\n");
	    fprintf( o1, "%4d ",-10);
	}

	if(el_type > 1 )
	{
	  if(modal_flag)
	  {
	    if(edge_conncet_write_flag)
	    {
		fprintf( o1, "prescribed edge: edge value\n");
		for( i = 0; i < angle_div; ++i )
		{
			j = side_el_num-1;
			fprintf( o1, "    %4d %9.4f\n", i*(2*side_el_num+1)+side_el_num+j+1, 0.0);
		}
	    }
	    else
	    {
		fprintf( o1, "prescribed edge: node1, node2 edge value\n");
		for( i = 0; i < angle_div-1; ++i )
		{
			j = side_el_num;
			fprintf( o1, " %4d %4d %9.4f\n",i*(side_el_num+1) + j, (i+1)*(side_el_num+1)+j, 0.0);
		}

		i = angle_div-1;
		j = 0;
		fprintf( o1, " %4d %4d %9.4f\n",i*(side_el_num+1) + j, j, 0.0);
		j = side_el_num;
		fprintf( o1, " %4d %4d %9.4f\n",i*(side_el_num+1) + j, j, 0.0);
	    }
	  }
	  if(!modal_flag)
	  {
		fprintf( o1, "prescribed voltage: node, voltage\n");
		for( i = 0; i < angle_div; ++i )
		{
			j = side_el_num;
			fprintf( o1, " %4d %9.4f\n",i*(side_el_num+1) + j, 0.0);
		}
	  }

	  fprintf( o1, "%4d\n ",-10);


	  fprintf( o1, "node with point charge\n");
	  fprintf( o1, "%4d %14.6e\n", (side_el_num+1)*(angle_div), 1.0);
	  fprintf( o1, "%4d\n ",-10);
	  fprintf( o1, "Electric Field: node, x and y component, magnitude\n");
	  fprintf( o1, "%4d ",-10);
	}

	fprintf( o1, "\n\n\n\n");
/*
	fprintf( o1, "\n What is the inner Radius?\n ");
	fprintf( o1, "%16.8e",R1);
*/
	fprintf( o1, "\n What is the length?\n ");
	fprintf( o1, "%16.8e\n", length);
	fprintf( o1, "\n What is the width?\n ");
	fprintf( o1, "%16.8e\n", width);
	fprintf( o1, "How many elements on one side?\n");
	fprintf( o1, "%4d\n",side_el_num);
	fprintf( o1, "What angle(in degrees) between rays?\n");
	fprintf( o1, "Angle should be < 360 and a factor of 360.\n ");
	fprintf( o1, "%12.4f\n",ray_angle*180.0/pi);
	fprintf( o1, "What type of element is this for?\n");
	fprintf( o1, "1) quad \n");
	fprintf( o1, "2) quad for electro-magnetics\n");
	fprintf( o1, "%4d\n",el_type);
	if(el_type > 1)
	{
		fprintf( o1,"Waveguide or electrostatics?\n");
		fprintf( o1,"0 = no, 1 = yes.\n");
		fprintf( o1,"%d\n",nmode);
		if(nmode) modal_flag = 1;
		edge_conncet_write_flag = 0;
		if(modal_flag)
		{
		    fprintf( o1,"Do you want the edge connection data?\n");
		    fprintf( o1,"0 = no, 1 = yes.\n");
		    fprintf( o1,"%d\n",edge_conncet_write_flag);
		}
	}

	return 1;
}

