/*
    This utility function takes the product of a vector with the
    either the Btt, Btz, or Bzt matrix.  This is for a finite element program
    which does analysis on a quad.  It is for modal analysis.

		Updated 12/16/09

    SLFCEM source file
    Version:  1.1
    Copyright (C) 1999-2009  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "qdeconst.h"
#include "qdestruct.h"

extern int numed, numel, numel_EM, numnp, dof, EMdof, sof;
extern double dcdx[nsdsq*num_int], shg_E[sosh_E], curl_E[num_int*epel], w[num_int];
extern double dcdx[nsdsq*num_int], shg[sosh], shg_node[sosh], shl[sosh],
	shg_E[sosh_E], curl_E[num_int*epel], w[num_int];
extern int AB_matrix_store;

int matXT(double *, double *, double *, int, int, int);

int matX(double *, double *, double *, int, int, int);

int quadB_mass(double *,double *);

int qdshg_E( double *, double *, int, double *, double *, double *, double *,
	double *, double *, double *);

int qdBPassemble(int *connect, double *coord, int *edge_connect, int *el_matl, double *Btt,
	MATL *matl, int *node_edge_connect, double *P_global, double *EMedge) 
{
        int i, i1, i2, i3, j, k, dof_el[neqel], EMdof_el[neqel4], sdof_el[npel*nsd];
	int check, counter, node, node0, node1, node2, node3, edge;
	int matl_num;
	double permit, permea, kosq, G;
        double B[npel], gradB[soB8], Bedge[soB8], curlBedge[epel],
		Bedge2[soB8], curlBedge2[epel];
	double Btt_temp[neqlsq16], Btt_el[neqlsq16];
	double EMedge_el[neqel4];
	double coord_el_trans[npel*nsd], length[epel], lengthsq[epel];
        double det[num_int];
	double P_el[neqel4];
	double fdum, fdum2;
	double shg_sign[4];

	memset(P_global,0,EMdof*sof);

	if(AB_matrix_store)
	{

/* Assemble P matrix using stored element Btt matrices */

	    for( k = 0; k < numel_EM; ++k )
	    {

		for( j = 0; j < epel; ++j )
		{
			edge = *(edge_connect+epel*k+j);

			*(EMdof_el+edof*j) = edof*edge;
		}

/* Assembly of the global P matrix */

		for( j = 0; j < neqel4; ++j )
		{
			*(EMedge_el + j) = *(EMedge + *(EMdof_el+j));
		}

		check = matX(P_el, (Btt + k*neqlsq16), EMedge_el, neqel4, 1, neqel4);
		if(!check) printf( "Problems with matX \n");

		for( j = 0; j < neqel4; ++j )
		{
			*(P_global+*(EMdof_el+j)) += *(P_el+j);
		}
	    }
	}
	else
	{

/* Assemble P matrix by re-deriving element B_matrix matrices */

            for( k = 0; k < numel_EM; ++k )
            {
		matl_num = *(el_matl+k);
		permit = matl[matl_num].er;
		permea = matl[matl_num].ur;
		kosq = matl[matl_num].ko*matl[matl_num].ko;

		/*printf("lamda, mu, permit, permea  %f %f %f %f \n", lamda, mu, Emod, Pois);*/

/* Create the coord_el transpose vector for one element */

		for( j = 0; j < npel; ++j )
		{
			node = *(connect+npel*k+j);

			*(sdof_el+nsd*j) = nsd*node;
			*(sdof_el+nsd*j+1) = nsd*node+1;

			*(coord_el_trans+j) = *(coord+*(sdof_el+nsd*j));
			*(coord_el_trans+npel*1+j) = *(coord+*(sdof_el+nsd*j+1));

			*(dof_el+ndof*j) = ndof*node;
			*(dof_el+ndof*j+1) = ndof*node+1;

		}

/* Create edge_el for one element */

		for( j = 0; j < epel; ++j )
		{
			edge = *(edge_connect+epel*k+j);
			node1 = *(node_edge_connect+edge*nped);
			node2 = *(node_edge_connect+edge*nped + 1);

			*(EMdof_el+edof*j) = edof*edge;

/* Calculate the edge lengths */

			fdum = *(coord+node2*nsd) - *(coord+node1*nsd);
			fdum2 = *(coord+node2*nsd+1) - *(coord+node1*nsd+1);
			*(lengthsq + j) = fdum*fdum + fdum2*fdum2;
			*(length + j) = sqrt(*(lengthsq + j));
		}

/* The variables below are needed to address the issue explained in:

   ~/slfcem-1.1/EDGE-ELEMENTS

   Read in the code "qdekasmbl.c" to get more discussion of how I set
   "shg_sign".
*/

		node0 = *(connect+npel*k);
		node1 = *(connect+npel*k+1);
		node2 = *(connect+npel*k+2);
		node3 = *(connect+npel*k+3);

		*(shg_sign)   = 1.0; *(shg_sign+1) = 1.0;
		*(shg_sign+2) = 1.0; *(shg_sign+3) = 1.0;

		if(node0 > node1) *(shg_sign)   = -1.0;
		if(node3 > node2) *(shg_sign+1) = -1.0;
		if(node0 > node3) *(shg_sign+2) = -1.0;
		if(node1 > node2) *(shg_sign+3) = -1.0;

/* Assembly of the dcdx and shg matrix for each integration point */

		check=qdshg_E(dcdx, det, k, shl, shg, shg_E, curl_E,
			coord_el_trans, length, shg_sign);
		if(!check) printf( "Problems with qdshg_E\n");

/* The loop over j below calculates the 4 points of the gaussian integration 
   for several quantities */

		memset(Btt_el,0,neqlsq16*sof);

		for( j = 0; j < num_int; ++j )
		{

		    memset(B,0,npel*sof);
		    memset(gradB,0,soB8*sof);
		    memset(Bedge,0,soB8*sof);
		    memset(curlBedge,0,epel*sof);
		    memset(Btt_temp,0,neqlsq16*sof);

/* Assembly of the component matrices */

		    check = quadB_E((dcdx+nsdsq*j), (shl+npel*(nsd+1)*j),
			(shg+npel*(nsd+1)*j), (shg_E+epel*nsd2*j), (curl_E+epel*j),
			B, gradB, Bedge, curlBedge, length);
		    if(!check) printf( "Problems with quadB_E\n");

		    memcpy(Bedge2, Bedge, soB8*sizeof(double));

		    check=matXT(Btt_temp, Bedge, Bedge2, neqel4, neqel4, nsd);
		    if(!check) printf( "Problems with matXT  \n");

		    fdum = *(w+j)*(*(det+j));

		    for( i2 = 0; i2 < neqlsq16; ++i2 )
		    {
			  *(Btt_el+i2) += *(Btt_temp+i2)*fdum*permit;
		    }

		}


/* Assembly of the global P matrix */

		memset(EMedge_el,0,neqel4*sof);
		for( j = 0; j < neqel4; ++j )
		{
			*(EMedge_el + j) = *(EMedge + *(EMdof_el+j));
		}

		check = matX(P_el, Btt_el, EMedge_el, neqel4, 1, neqel4);
		if(!check) printf( "Problems with matX \n");

		for( j = 0; j < neqel4; ++j )
		{
			*(P_global+*(EMdof_el+j)) += *(P_el+j);
		}

	    }
	}

        return 1;
}
