/*
    This program generates a benchmark data file for a rectangular
    waveguide based on the analytical solution given on:

       http://www.antenna-theory.com/tutorial/waveguides/waveguides3.php

    I have also confirmed the correctness of this solution with:

      Volakis, John L., et. al., Finite Element Method for Electromagnetics,
        IEEE Press, 1998.

    See page 112, figure 4.9.

		Updated 1/24/10

    SLFCEM source file
    Version:  1.1
    Copyright (C) 2010  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define pi              3.141592654
#define cc              3.0e8

int main(int argc, char** argv)
{
	int dof, nmat, numel, numed, numnp, plane_stress_flag;
        int i,j,dum,dum2,dum3,dum4;
	double fdum1, fdum2, fdum3, ux[9], uy[9];
	double side_length, side_width, el_length, el_width;
	double Ex, Ey, Emag, mm, nn, freq, omega;
	int length_el_num, width_el_num, el_type;
        char name[20];
	char buf[ BUFSIZ ];
        FILE *o1;
	char text;

        printf( "\n What is the value of the length?\n ");
	scanf( "%lf",&side_length);
        printf( "\n How many elements on the length?\n ");
	scanf( "%d",&length_el_num);
        printf( "\n What is the value of the width?\n ");
	scanf( "%lf",&side_width);
        printf( "\n How many elements on the width?\n ");
	scanf( "%d",&width_el_num);
        printf( "\n What is the m in TEmn?\n ");
	scanf( "%lf",&mm);
        printf( "\n What is the n in TEmn?\n ");
	scanf( "%lf",&nn);
        el_type = 1;

       	o1 = fopen( "qdebench.oqd","w" );

	nmat = 1; 
	plane_stress_flag = 1;
	numel = length_el_num*width_el_num;
	numed = length_el_num*(width_el_num+1) + (length_el_num+1)*width_el_num;
	numnp = (length_el_num+1)*(width_el_num+1);
	el_length = side_length/((double)length_el_num);
	el_width = side_width/((double)width_el_num);

        fprintf( o1, "   numed numel numnp nmat nmode sigma low_mode");
        fprintf( o1, "  This is the rectangular waveguide problem \n ");
        fprintf( o1, "    %4d %4d %4d %4d %4d %16.8e %16.8e\n ",
		numed, numel, numnp, nmat, 0, 6.0, 1.0);

        fprintf( o1, "matl no., permeability, permittivity, operating fq\n");
        for( i = 0; i < nmat; ++i )
        {
        	fprintf( o1, "%3d ",0);
        	fprintf( o1, " %9.4f %9.4f     %9.4f \n ",1.0, 1.0, 40.0307);
        }


	dum = 0;
	fprintf( o1, "el no., connectivity, matl no. \n");

	for( i = 0; i < width_el_num; ++i )
	{
		for( j = 0; j < length_el_num; ++j )
		{
			fprintf( o1, "%4d ",dum);
			fprintf( o1, " %4d %4d %4d %4d",i*(length_el_num+1) + j,
				i*(length_el_num+1)+j+1, (i+1)*(length_el_num+1)+j+1,
				(i+1)*(length_el_num+1)+j);
			fprintf( o1, "%4d ",0);
			fprintf( o1, "\n");
			++dum;
		}
	}

	dum = 0;
        fprintf( o1, "node no., coordinates \n");
	for( i = 0; i < width_el_num + 1; ++i )
	{
		for( j = 0; j < length_el_num + 1; ++j )
		{
			fdum1 = el_length*((double)j);
			fdum2 = el_width*((double)i);
           		fprintf( o1, "%d ",dum);
           		fprintf( o1, "%9.4f %9.4f ",fdum1,fdum2);
           		fprintf( o1, "\n");
			++dum;
         	}
        }

        fprintf( o1, "prescribed edge: node1, node2 edge value\n");
        fprintf( o1, "%4d\n ",-10);
        fprintf( o1, "node with point charged, point charge\n");
        fprintf( o1, "%4d\n ",-10);

/* Look at equation [1] and [2] on the page:

   http://www.antenna-theory.com/tutorial/waveguides/waveguides3.php

   for Ex and Ey.
*/
        fprintf( o1, "Electric Field: node, x and y component, magnitude\n");
	dum = 0;
	for( i = 0; i < width_el_num + 1; ++i )
	{
		for( j = 0; j < length_el_num + 1; ++j )
		{
			fdum1 = nn*pi*el_length*((double)j)/side_length;
			fdum2 = mm*pi*el_width*((double)i)/side_width;

			Ex = cos(fdum1)*sin(fdum2);
			Ey = -sin(fdum1)*cos(fdum2);
			Emag = sqrt(Ex*Ex + Ey*Ey);

           		fprintf( o1, "%4d ",dum);
           		fprintf( o1, "%16.8e %16.8e %16.8e\n", Ex, Ey, Emag);
			++dum;
         	}
        }
        fprintf( o1, "%4d\n ",-10);

        printf( "mode number     frequency         omega(relative um and eta)\n");
	mm = 1.0; nn = 0.0;
	fdum1 = mm*mm/side_length/side_length; fdum2 = nn*nn/side_width/side_width;
	freq = cc/2.0*sqrt( fdum1 + fdum2 ); omega = pi*sqrt( fdum1 + fdum2 );
        printf( "    10      %16.8e         %16.8e\n", freq, omega);
	mm = 2.0; nn = 0.0;
	fdum1 = mm*mm/side_length/side_length; fdum2 = nn*nn/side_width/side_width;
	freq = cc/2.0*sqrt( fdum1 + fdum2 ); omega = pi*sqrt( fdum1 + fdum2 );
        printf( "    20      %16.8e         %16.8e\n", freq, omega);
	mm = 0.0; nn = 1.0;
	fdum1 = mm*mm/side_length/side_length; fdum2 = nn*nn/side_width/side_width;
	freq = cc/2.0*sqrt( fdum1 + fdum2 ); omega = pi*sqrt( fdum1 + fdum2 );
        printf( "    01      %16.8e         %16.8e\n", freq, omega);
	mm = 1.0; nn = 1.0;
	fdum1 = mm*mm/side_length/side_length; fdum2 = nn*nn/side_width/side_width;
	freq = cc/2.0*sqrt( fdum1 + fdum2 ); omega = pi*sqrt( fdum1 + fdum2 );
        printf( "    11      %16.8e         %16.8e\n", freq, omega);
	mm = 3.0; nn = 0.0;
	fdum1 = mm*mm/side_length/side_length; fdum2 = nn*nn/side_width/side_width;
	freq = cc/2.0*sqrt( fdum1 + fdum2 ); omega = pi*sqrt( fdum1 + fdum2 );
        printf( "    30      %16.8e         %16.8e\n", freq, omega);
	mm = 1.0; nn = 1.0;
	fdum1 = mm*mm/side_length/side_length; fdum2 = nn*nn/side_width/side_width;
	freq = cc/2.0*sqrt( fdum1 + fdum2 ); omega = pi*sqrt( fdum1 + fdum2 );
        printf( "    21      %16.8e         %16.8e\n", freq, omega);
	mm = 0.0; nn = 2.0;
	fdum1 = mm*mm/side_length/side_length; fdum2 = nn*nn/side_width/side_width;
	freq = cc/2.0*sqrt( fdum1 + fdum2 ); omega = pi*sqrt( fdum1 + fdum2 );
        printf( "    02      %16.8e         %16.8e\n", freq, omega);

        return 1;
}

