/*
    This is the include file "qdeconst.h" for the finite element progam 
    which uses quadrialteral elements to calculate electro-magnetic fields.

                Updated 12/25/09

    SLFCEM source file
    Version:  1.1
    Copyright (C) 1999-2009  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

    Look at:

       http://www.antenna-theory.com/definitions/main.php

    for definitions of waveguide variables and terms.
 
*/


#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define pt5            .5 
#define pt25           .25
#define pt1667         .166666666667
#define sq3           1.732050808 
#define pi            3.141592654
#define speed_light   3.0e+08                /* Speed of Light m/s. */


#define vac_permit 8.854e-12                 /* Permittivity in vacuum */
#define vac_permea 4*pi*e-7                  /* Permeability in vacuum */

#define nsd           2                      /* spatial dimensions per node */
#define nsd2          2                      /* spatial dimensions per node */
#define ndof          2                      /* degrees of freedom per node */
#define npel          4                      /* nodes per element */
#define neqel         npel*ndof              /* degrees of freedom per element */
#define num_int       4                      /* number of integration points */
#define nsdsq         nsd*nsd                /* nsd squared */
#define neqlsq        neqel*neqel            /* neqel squared */
#define sdim          3                      /* stress dimensions per element */
#define soB           sdim*neqel             /* size of B matrix */
#define MsoB          nsd*neqel              /* size of B_mass matrix */
#define sosh          (nsd +1)*npel*num_int  /* size of shl and shg matrix */
#define sosh_node2    nsd*2*num_int          /* size of shl_node2 */
#define sosh_E        epel*num_int*nsd2      /* size of shg_E matrix */
#define KB            1024.0                 /* number of bytes in kilobyte */
#define MB            1.0486e+06             /* number of bytes in megabyte */

#define MIN(x,y) (((x)<(y))?(x):(y))
#define MAX(x,y) (((x)>(y))?(x):(y))

/*    Below is the additional list of constants for the quad element eqd
      to calculate waveguides.
*/

#define edof          1                      /* degrees of freedom per edge */
#define epel          4                      /* edges per element */
#define nped          2                      /* nodes per edge */
#define neqel4        epel*edof              /* degrees of freedom per element for waveguide calculation */
#define neqlsq16      neqel4*neqel4          /* neqel4 squared */
#define soB8          nsd*npel               /* size of B matrix for electromagnetics */

/*    Below is the additional list of constants for the quad element eqd
      to calculate voltages from charge distributions.
*/

#define Vndof         1                      /* Voltage degrees of freedom per node */
#define Vneqel4       npel*Vndof             /* degrees of freedom per element for voltage calculation */
#define Vneqlsq16     Vneqel4*Vneqel4        /* Vneqel4 squared */

/*
  title               problem title
  numed               number of edges
  numel               number of elements
  numnp               number of nodal points 
  nmat                number of materials
  dof                 total number of degrees of freedom 
  Vdof                total number of degrees of freedom for Voltage calculation
  EMdof               total number of degrees of freedom for waveguide calculation 
  coord               *(coord + 0) x coordinate of node
                      *(coord + 1) y coordinate of node
  connect             (0-3) connectivity array 
  matl                material structure
  EMod                young's modulus
  nu                  poisson's ratio
  charge              *(charge + 0) x component of applied load
                      *(charge + 1) y component of applied load
  analysis_flag       1 calculate unknown displacemnts
                      2 calculate reaction charges
  LU_decomp_flag      0 if numel <= 1125 elements, use LU Decomposition for
                        displacements
                      1 if numel > 1125 elements, use conjugate
                        gradient method for displacements
*/
