/*
    This function is for testing the code by using the LAPACK linear
    algebra library(actually the C version CLAPACK) to solve the
    eigensystems arising from the waveguide problem.

    I have to assemble the full global Att, Bzz, Btt, Btz, Bzt matrices
    to use LAPACK. 

    The motivation for using LAPACK was that their libraries can solve
    very general linear systems.  Unlike in structural mechanics, the
    matrices of electro-magnetism are positive semi-definite.  Fortunately,
    the linear solvers I developed for SLFFEA are capable of solving such
    systems. 

        Updated 1/23/10

    SLFCEM source file
    Version:  1.1
    Copyright (C) 2009  San Le 
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "../../common/eigen.h"
#include "qdeconst.h"
#include "qdestruct.h"

#include "/home/fem/CLAPACK/SRC/f2c.h"
#include "/home/fem/CLAPACK/SRC/blaswrap.h"

int Ewriter ( BOUND , int *, int *, int *, double *, int *, double *, int *,
	MATL *, char *, double *, double *, double *, double *);

int writer ( BOUND , int *, double *, int *, double *, int *, MATL *, char *,
	double *, double *);

int qdEMnode_calculate(int *, int *, int *, double *, double *, double *, double *,
	double * );


/* The functions below are used to solve the system using LAPACK will fully
   stored matrices.
*/

int dsyevr_(char *, char *, char *, integer *, doublereal *, integer *, doublereal *,
	doublereal *, integer *, integer *, doublereal *, integer *, doublereal *,
	doublereal *, integer *, integer *, doublereal *, integer *, integer *,
	integer *, integer *);

int dsygvx_(integer *, char *, char *, char *, integer *, doublereal *, integer *,
	doublereal *, integer *, doublereal *, doublereal *, integer *, integer *,
	doublereal *, integer *, doublereal *, doublereal *, integer *, doublereal *,
	integer *, integer *, integer *, integer *);

int zhegvx_(integer *, char *, char *, char *, integer *, doublecomplex *, integer *,
	doublecomplex *, integer *, doublereal *, doublereal *, integer *, integer *,
	doublereal *, integer *, doublereal *, doublecomplex *, integer *,
	doublecomplex *, integer *, doublereal *, integer *, integer *, integer *);

int dsytri_(char *, integer *, doublereal *, integer *, integer *, doublereal *,
	integer *);

int dsytrf_(char *, integer *, doublereal *, integer *, integer *, doublereal *,
	integer *, integer *);

int qdEKglobalassemble( double *, double *, double *, double *, double *,
	double *, double *, double *, double *, double *, int *, int *, int *);


int dof, sdof, EMdof, neqn, EMneqn, nmode, numed, numel, numnp, sof;

double dcdx[nsdsq*num_int], shg[sosh], shg_node[sosh], shl[sosh], shg_E[sosh_E],
	shg_node_E[sosh_E], curl_E[num_int*epel], shl_node[sosh], shl_node2[sosh_node2],
	w[num_int], sigma, low_mode;

int qdLAPACK( BOUND bc, double *Att, double *Btt, double *Btz, double *Bzt, double *Bzz,
	int *connect, int *node_edge_connect, int *edge_connect, double *charge, double *coord,
	EIGEN *eigen, int *el_matl, int *id, int *lm, MATL *matl, char *name,
	double *node_counter, double *EMedge, double *EMnode, double *EMmag, double *Volt )
{
	int i, i2, j, k;
	int dum, ptr_inc;
	int check, name_length, counter, MemoryCounter, neg_eval_counter, nmode2;
	int num_eigen;
	char buf[ BUFSIZ ];
	char name_mode[30], *ccheck;
	FILE *o1, *o2;
	double fdum, fdum2;
	int node, node1, node2;
	double kosq, evalsq, kt;

/* The variables below include the full matrices as well as the additional
   parameters needed LAPACK.
*/
	FILE *o_eigen;
	double *mem_doubleG, *AttG, *BttG, *BtzG, *BztG, *BzzG, *BttpG, *B_temp, *mat_temp;
	char jobz[1], range[1], uplo[1];
	integer itype, nn, lda, ldb, il, iu, m, ldz, lwork, *iwork, *ifail, info,
		*ipiv, *isuppz, liwork;
	doublereal *work, *rwork, vl, vu, abstol;
	doublereal *evec, *eval;

	o_eigen = fopen( "eigen.dat","w" );

	dum = 8*EMdof*EMdof;
	mem_doubleG = (double *)calloc(dum,sizeof(double));
	if(!mem_doubleG )
	{
		printf( "failed to allocate memory for doubleG\n");
		exit(1);
	}

/* For the global doubles */
	                                        ptr_inc=0;
	AttG=(mem_doubleG+ptr_inc);             ptr_inc += EMdof*EMdof;
	BttG=(mem_doubleG+ptr_inc);             ptr_inc += EMdof*EMdof;
	BtzG=(mem_doubleG+ptr_inc);             ptr_inc += EMdof*EMdof;
	BztG=(mem_doubleG+ptr_inc);             ptr_inc += EMdof*EMdof;
	BzzG=(mem_doubleG+ptr_inc);             ptr_inc += EMdof*EMdof;
	B_temp=(mem_doubleG+ptr_inc);           ptr_inc += EMdof*EMdof;
	BttpG=(mem_doubleG+ptr_inc);            ptr_inc += EMdof*EMdof;
	mat_temp=(mem_doubleG+ptr_inc);         ptr_inc += EMdof*EMdof;


	check = qdEKglobalassemble(AttG, BttG, BtzG, BztG, BzzG, Att, Btt, Btz, Bzt, Bzz,
	    node_edge_connect, edge_connect, lm);
	if(!check) printf( " Problems with qdEKglobalassembler \n");

/* For "zhegvx.c" line 42

    JOBZ    (input) CHARACTER*1
            = 'N':  Compute eigenvalues only;
            = 'V':  Compute eigenvalues and eigenvectors.

    RANGE   (input) CHARACTER*1
            = 'A': all eigenvalues will be found.
            = 'V': all eigenvalues in the half-open interval (VL,VU]
                   will be found.
            = 'I': the IL-th through IU-th eigenvalues will be found.
*/

	ccheck = strncpy(jobz, "V", 1);
	ccheck = strncpy(range, "V", 1);
	ccheck = strncpy(uplo, "U", 1);

	check = strncmp(range, "A", 1);
	if( !check ) ccheck = strncpy(jobz, "N", 1);

	lwork = 26*EMneqn;
	liwork = EMneqn*EMneqn;
	rwork = (doublereal *)calloc( 7*EMneqn,sizeof(doublereal));
	iwork = (integer *)calloc( 5*EMneqn,sizeof(int));
	ifail = (integer *)calloc( EMneqn,sizeof(int));
	eval = (doublereal *)calloc( EMneqn,sizeof(doublereal));
	ipiv = (integer *)calloc( EMneqn,sizeof(integer));
	isuppz = (integer *)calloc( 2*EMneqn,sizeof(integer));

	itype = 1;
	nn = EMneqn;
	lda = EMneqn;
	ldb = EMneqn;
	il = 1;   iu = 10;
	il = 97;  iu = 107;
	il = 145; iu = 158;  /* For diske4 */
	il = 170; iu = 180;  /* For rect3.save2 */

	if( iu > EMneqn )
	{
	    iu = EMneqn - 1;
	    il = iu - 20;
	}
	printf("iiii %4d %4d\n", iu, il);
	m = EMneqn;
	ldz = EMneqn;
	lda = EMneqn;
	abstol = 1.0e-8;

	vl =  low_mode; vu = 4.0; /* For diske4 */
	vl =  low_mode; vu = 73.0; /* For rect3.save2 */

	dum = EMneqn;
	if(dum < 8) dum = 8;
	work=(doublereal *)calloc(dum*EMneqn,sizeof(doublereal));
	evec=(doublereal *)calloc(2*iu*EMneqn,sizeof(doublereal));

	for( i = 0; i < EMneqn*EMneqn; ++i )
	{
		*(B_temp + i) = *(BzzG + i);
	}

/* Calculate the inverse of [BzzG] */

	check = dsytrf_( uplo, &nn, BzzG, &lda, ipiv, work, &lwork, &info);
	printf("%4d \n", check);

	check = dsytri_( uplo, &nn, BzzG, &lda, ipiv, work, &info);
	printf("%4d \n", check);

/* The lower triangular part of BzzG stores the inverse elements.  So I have to set
   the upper part BzzG[ij] = BzzG[ji].
*/

	for( i = 0; i < EMneqn; ++i )
	{
	    for( j = i+1; j < EMneqn; ++j )
	    {
		*(BzzG + EMneqn*i + j ) = *( BzzG + EMneqn*j + i);
	    }
	}


	for( i = 0; i < EMneqn*EMneqn; ++i )
	{
		*(BttpG + i) = *(BttG + i);
	}


/* hhh */

	if(0)
	{

/* I multiply Att by the inverse of BttpG to get the eigensystem:

                  [A] [x] = lam [x]             [Bttp inverse][Att] = [A]

and solve for that system instead.  I needed to do this because the general
eigen solver, "dsbgvx_", cannot solve systems with negative eigenvalues.  When
I had bad numbers for the material properties, the eigenvalues were negative
so I needed to use "dsytrf_" which can solve for negative eigenvalues.  When
I got material values and the system became positive definite, both functions
gave the same results.
*/

	    check = dsytrf_( uplo, &nn, BttpG, &lda, ipiv, work, &lwork, &info);
	    printf("%4d \n", check);

	    check = dsytri_( uplo, &nn, BttpG, &lda, ipiv, work, &info);
	    printf("%4d \n", check);

/* The lower triangular part of BttpG stores the inverse elements.  So I have to set
   the upper part BttpG[ij] = BttpG[ji].
*/

	    for( i = 0; i < EMneqn; ++i )
	    {
		for( j = i+1; j < EMneqn; ++j )
		{
		    *(BttpG + EMneqn*i + j ) = *( BttpG + EMneqn*j + i);
		}
	    }

	    check = matX( B_temp, BttpG, AttG, EMneqn, EMneqn, EMneqn);
	    if(!check) printf( "Problems with matX \n");

	    check=dsyevr_(jobz, range, uplo, &nn, B_temp, &lda, &vl, &vu, &il, &iu, &abstol,
		    &m, eval, evec, &ldz, isuppz, work, &lwork, iwork, &liwork, &info);
	    if(check) printf( "Problems with dsyevr_ \n");
	}
	else
	{
/* Solve the original eigensystem:

                  [AttG] [x] = lam [BttpG][x]

*/

	    lwork = EMneqn*EMneqn;

	    /*check=zhegvx_(&itype, jobz, range, uplo, &nn, AttG, &lda, BttpG, &ldb, &vl, &vu, &il,
		    &iu, &abstol, &m, eval, evec, &ldz, work, &lwork, rwork, iwork, ifail, &info);
	    if(check) printf( "Problems with zhegvx_\n");*/

	    check=dsygvx_( &itype, jobz, range, uplo, &nn, AttG, &lda, BttpG, &ldb, &vl, &vu,
		    &il, &iu, &abstol, &m, eval, evec, &ldz, work, &lwork, iwork, ifail, &info);
	    if(check) printf( "Problems with dsbgvx_\n");
	}

/*
	for( i = 0; i < EMneqn*EMneqn; ++i )
	{
		*(BttpG + i) = -*(BttpG + i);
		*(AttG + i)  = -*(AttG + i);
	}

	    check=dsyevr_(jobz, range, uplo, &nn, AttG, &lda, &vl, &vu, &il, &iu, &abstol,
		    &m, eval, evec, &ldz, isuppz, work, &lwork, iwork, &liwork, &info);
	    if(!check) printf( "Problems with dsyevr_ \n");
*/

	printf("info %4d\n", info);


/* Count the number of times a particular node is part of an element */

	printf("eval \n");

	check = strncmp(range, "A", 1);
	if( !check )
	{
	    for( i = 0; i < EMneqn; ++i )
	    {
		printf("%4d %16.8e\n", i, *(eval + i ));
	    }
	}

	check = strncmp(range, "I", 1);
	if( !check )
	{
	    for( i = il-1; i < iu; ++i )
	    {
		printf("%4d %16.8e\n", i, *(eval + i ));
	    }
	    for( i = il-1; i < iu; ++i )
	    {
		kosq = matl[0].ko*matl[0].ko;
		evalsq = *(eval + i )*(*(eval + i ));
		kt = sqrt(kosq - evalsq);
		printf("%4d kz = %16.8e ko = %16.8\n", i, *(eval + i ), matl[0].ko, kt);
	    }
	    for( i = il-1; i < iu; ++i )
	    {
		if( *(eval + i ) > 1.0e-2)
		printf("%4d sqrt eval = %16.8e\n", i, sqrt(*(eval + i )));
	    }
	}

	check = strncmp(range, "V", 1);
	if( !check )
	{
	    for( i = 0; i < 9; ++i )
	    {
		printf("%4d %16.8e\n", i, *(eval + i ));
		fprintf( o_eigen, "%4d %16.8e\n", i, *(eval + i ));
	    }
	    for( i = 0; i < 9; ++i )
	    {
		kosq = matl[0].ko*matl[0].ko;
		evalsq = *(eval + i )*(*(eval + i ));
		kt = sqrt(kosq - evalsq);
		printf("%4d kz = %16.8e ko = \n", i, *(eval + i ), matl[0].ko, kt);
		fprintf( o_eigen, "%4d kz = %16.8e ko = \n", i, *(eval + i ), matl[0].ko, kt);
	    }
	}

	/* Write out the eigenvalues to a file */

	dum = 9;
	if( dum > num_eigen ) dum = num_eigen;
	for( i = 0; i < dum; ++i )
	{
		eigen[i].val = *(eval + i );
	}

	check = eval_data_print( eigen, name, dum);
	if(!check) printf( " Problems with eval_data_print \n");

/* kkkk */

	/*exit(1);*/

	name_length = strlen(name);
	if( name_length > 20) name_length = 20;

	memset(name_mode,0,30*sizeof(char));

/* name_mode is the name of the output data file for the mode shapes.
   It changes based on the number of the eigenmode.
*/
	ccheck = strncpy(name_mode, name, name_length);
	if(!ccheck) printf( " Problems with strncpy \n");

	for( j = 0; j < 9; ++j )
	{

/* write the number of the eigenmode onto the name of the output file, name_mode */

	    sprintf((name_mode+name_length+5), "%d",j+1);
	    if(j + 1 > 9 )
		    sprintf((name_mode+name_length+5), "%2d",j+1);
	    if(j + 1 > 99 )
		    sprintf((name_mode+name_length+5), "%3d",j+1);

	    ccheck = strncpy(name_mode+name_length, ".mod-", 5);
	    if(!ccheck) printf( " Problems with strncpy \n");


	    for( i = 0; i < EMdof; ++i )
	    {

/* The eigectors are stored by rows by dsygvx_, unlike how it happens in SLFFEA. */

		if( *(id + i) > -1 )
		{
		    /* *(EMedge + i) = *(ritz + num_eigen*(*(id + i))+4); */
		    /**(EMedge + i) = *(ritz + num_eigen*(*(id + i))+eigen[j].index);*/
		    *(EMedge + i) = *(evec + EMneqn*j + *(id + i));
		    /*printf( "\n %4d EMedge[%4d] = %16.8e", *(id + i), i, *(EMedge+i));*/
		}
		/*printf( "%4d EMedge[%4d] = %16.8e\n", *(id + i), i, *(EMedge+i));*/
	    }

	    check = qdEMnode_calculate(connect, edge_connect, node_edge_connect, coord,
		EMedge, EMnode, EMmag, node_counter);
	    if(!check) printf( " Problems with qdEMnode_calculate \n");

/*
	    check = writer ( bc, connect, coord, el_matl, charge, id, matl, name_mode,
		EMnode, EMmag);
	    if(!check) printf( " Problems with writer \n");
*/

	    check = Ewriter( bc, connect, node_edge_connect, edge_connect, coord,
		el_matl, charge, id, matl, name_mode, EMnode, EMedge, EMmag, Volt);
	    if(!check) printf( " Problems with Ewriter \n");

	}

	free(mem_doubleG);
	exit(1);

}

