/*
    This utility code uses the conjugate gradient method
    to solve the linear system [K][U] = [f] for a finite
    element program which does analysis on a quad.  The first function
    assembles the P matrix.  It is called by the second function
    which allocates the memory and goes through the steps of the algorithm.
    These go with the calculation of displacement.

		Updated 12/16/09

    SLFCEM source file
    Version:  1.1
    Copyright (C) 1999-2009  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "qdeconst.h"
#include "qdestruct.h"

#define SMALL      1.e-20

extern int analysis_flag, dof, Vdof, numed, numel, numnp, sof;
extern int LU_decomp_flag, numel_V, numel_V_P, numel_V_K;
extern double shg[sosh], shl[sosh], w[num_int], *Area0, sigma;
extern int  iteration_max, iteration_const, iteration;
extern double tolerance;

int matX(double *,double *,double *, int ,int ,int );

int matXT(double *, double *, double *, int, int, int);

int quadB(double *,double *);

int qdshg_E( double *, double *, int, double *, double *, double *, double *,
        double *, double *, double *);

int dotX(double *, double *, double *, int);

int qdBoundary( double *, BOUND );


int qdVConjPassemble(double *A, int *connect, double *coord, int *el_matl,
	MATL *matl, double *P_global_CG, double *Volt)
{
/* This function assembles the P_global_CG matrix for the displacement calculation by
   taking the product [K_el]*[U_el].  Some of the [K_el] is stored in [A].

			Updated 12/2/09
*/
	int i, i1, i2, j, k, dof_el[neqel], Vdof_el[Vneqel4], sdof_el[npel*nsd];
	int check, counter, node, node0, node1, node2, node3, edge;
	int matl_num;
	double permit, permea, kosq, G;
	double fdum, fdum2;
	double lamda, mu;
	double B[npel], gradB[soB8], gradB2[soB8];
	double K_temp[Vneqlsq16], K_el[neqlsq16], Volt_el[neqel4];
	double coord_el_trans[npel*nsd], length[epel], lengthsq[epel];;
	double det[num_int];
	double P_el[neqel4];
	double shg_sign[4];


	memset(P_global_CG,0,Vdof*sof);

	for( k = 0; k < numel_V_K; ++k )
	{

		for( j = 0; j < npel; ++j )
		{
			node = *(connect+npel*k+j);

			*(Vdof_el+Vndof*j) = Vndof*node;

		}


/* Assembly of the global P matrix */

		for( j = 0; j < Vneqel4; ++j )
		{
			*(Volt_el + j) = *(Volt + *(Vdof_el+j));
		}

		check = matX(P_el, (A+k*Vneqlsq16), Volt_el, Vneqel4, 1, Vneqel4);
		if(!check) printf( "Problems with matX \n");

		for( j = 0; j < Vneqel4; ++j )
		{
			*(P_global_CG+*(Vdof_el+j)) += *(P_el+j);
		}
	}
	for( k = numel_V_K; k < numel_V; ++k )
	{
		matl_num = *(el_matl+k);
		permit = matl[matl_num].er;
		permea = matl[matl_num].ur;
		kosq = matl[matl_num].ko*matl[matl_num].ko;

		/*printf("lamda, mu, permit, permea  %f %f %f %f \n", lamda, mu, Emod, Pois);*/

/* Create the coord_el transpose vector for one element */

		for( j = 0; j < npel; ++j )
		{
			node = *(connect+npel*k+j);

			*(sdof_el+nsd*j) = nsd*node;
			*(sdof_el+nsd*j+1) = nsd*node+1;

			*(coord_el_trans+j) = *(coord+*(sdof_el+nsd*j));
			*(coord_el_trans+npel*1+j) = *(coord+*(sdof_el+nsd*j+1));

			*(dof_el+ndof*j) = ndof*node;
			*(dof_el+ndof*j+1) = ndof*node+1;

			*(Vdof_el+Vndof*j) = Vndof*node;
		}

/* Assembly of the shg matrix for each integration point */

		check=qdshg( det, k, shl, shg, coord_el_trans);
		if(!check) printf( "Problems with qdshg\n");

/* The loop over j below calculates the 4 points of the gaussian integration 
   for several quantities */

		memset(K_el,0, Vneqlsq16*sof);

		for( j = 0; j < num_int; ++j )
		{

		    memset(B,0,npel*sof);
		    memset(K_temp,0,Vneqlsq16*sof);

/* Assembly of the component matrices */

		    check = quadB_V((shl+npel*(nsd2+1)*j), (shg+npel*(nsd2+1)*j),
			B, gradB);
		    if(!check) printf( "Problems with quadB_V\n");

		    memcpy(gradB2, gradB, nsd2*npel*sizeof(double));

		    fdum = *(w+j)*(*(det+j));

		    check=matXT(K_temp, gradB, gradB2, Vneqel4, Vneqel4, nsd2);
		    if(!check) printf( "Problems with matXT  \n");

		    for( i2 = 0; i2 < Vneqlsq16; ++i2 )
		    {
			*(K_el + i2) += *(K_temp + i2)*fdum*permit;
		    }
		}

/* Assembly of the global P matrix */

		for( j = 0; j < Vneqel4; ++j )
		{
			*(Volt_el + j) = *(Volt + *(Vdof_el+j));
		}

                check = matX(P_el, K_el, Volt_el, Vneqel4, 1, Vneqel4);
                if(!check) printf( "Problems with matX \n");

                for( j = 0; j < Vneqel4; ++j )
                {
                	*(P_global_CG+*(Vdof_el+j)) += *(P_el+j);
		}
	}

        return 1;
}


int qdVConjGrad(double *A, BOUND bc, int *connect, double *coord, int *el_matl,
	double *charge, double *K_diag, MATL *matl, double *Volt)
{
/* This function does memory allocation and uses the conjugate gradient
   method to solve the linear system arising from the calculation of
   displacements.  It also makes the call to qdVConjPassemble to get the
   product of [A]*[p].

			Updated 12/22/09

   It is taken from the algorithm 10.3.1 given in "Matrix Computations",
   by Golub, page 534.
*/
	int i, j, sofmf, ptr_inc;
	int check, counter;
	double *mem_double;
	double *p, *P_global_CG, *r, *z;
	double alpha, alpha2, beta;
	double rXz_old, rXz;

/* For the doubles */
	sofmf = 4*Vdof;
	mem_double=(double *)calloc(sofmf,sizeof(double));

	if(!mem_double )
	{
		printf( "failed to allocate memory for doubles\n ");
		exit(1);
	}

/* For the Conjugate Gradient Method doubles */

	                                        ptr_inc = 0;
	p=(mem_double+ptr_inc);                 ptr_inc += Vdof;
	P_global_CG=(mem_double+ptr_inc);       ptr_inc += Vdof;
	r=(mem_double+ptr_inc);                 ptr_inc += Vdof;
	z=(mem_double+ptr_inc);                 ptr_inc += Vdof;

/* Using Conjugate gradient method to find displacements */

	memset(P_global_CG,0,Vdof*sof);
	memset(p,0,Vdof*sof);
	memset(r,0,Vdof*sof);
	memset(z,0,Vdof*sof);

        for( j = 0; j < Vdof; ++j )
	{
		*(K_diag + j) += SMALL;
		*(r+j) = *(charge+j);
		*(z + j) = *(r + j)/(*(K_diag + j));
		*(p+j) = *(z+j);
	}
	check = VBoundary (r, bc);
	if(!check) printf( " Problems with VBoundary \n");

	check = VBoundary (p, bc);
	if(!check) printf( " Problems with VBoundary \n");

	alpha = 0.0;
	alpha2 = 0.0;
	beta = 0.0;
	rXz = 1000.0;
	counter = 0;
	check = dotX(&rXz_old, r, z, Vdof);

	printf("\n iteration %3d iteration max %3d \n", iteration, iteration_max);
        /*for( iteration = 0; iteration < iteration_max; ++iteration )*/
	while(rXz > tolerance && counter < iteration_max )
        {

		printf( "\n %3d %16.8e\n",counter, rXz);
		check = qdVConjPassemble( A, connect, coord, el_matl, matl, P_global_CG, p);
		if(!check) printf( " Problems with qdVConjPassemble \n");
		check = VBoundary (P_global_CG, bc);
		if(!check) printf( " Problems with VBoundary \n");
		check = dotX(&alpha2, p, P_global_CG, Vdof);	
		alpha = rXz_old/(SMALL + alpha2);

        	for( j = 0; j < Vdof; ++j )
		{
            	    /*printf( "%4d %14.5e  %14.5e  %14.5e  %14.5e  %14.5e %14.5e\n",j,alpha,
			beta,*(EMedge+j),*(r+j),*(P_global_CG+j),*(p+j));*/
    		    *(Volt+j) += alpha*(*(p+j));
		    *(r+j) -=  alpha*(*(P_global_CG+j));
		    *(z + j) = *(r + j)/(*(K_diag + j));
		}

		check = dotX(&rXz, r, z, Vdof);
		beta = rXz/(SMALL + rXz_old);
		rXz_old = rXz;
	
        	for( j = 0; j < Vdof; ++j )
        	{
       		    /*printf("\n  %3d %12.7f  %14.5f ",j,*(Volt+j),*(P_global_CG+j));*/
            	    /*printf( "%4d %14.5f  %14.5f  %14.5f  %14.5f %14.5f\n",j,alpha,
			*(Volt+j),*(r+j),*(P_global_CG+j),*(charge+j));
            	    printf( "%4d %14.8f  %14.8f  %14.8f  %14.8f %14.8f\n",j,
			*(Volt+j)*beta,*(r+j)*beta,*(P_global_CG+j)*alpha,
			*(charge+j)*alpha);*/
            	    *(p+j) = *(z+j)+beta*(*(p+j));

		}
		check = VBoundary (p, bc);
		if(!check) printf( " Problems with VBoundary \n");

		++counter;
	}

	if(counter > iteration_max - 1 )
	{
		printf( "\nMaximum iterations %4d reached.  Residual is: %16.8e\n",counter,rXz);
		printf( "Problem may not have converged during Conj. Grad.\n");
	}
/*
	for( j = 0; j < Vdof; ++j )
	{
		printf( "%4d %14.5f  %14.5f %14.5f  %14.5f  %14.5f %14.5f\n",j,alpha,beta,
			*(Volt+j),*(r+j),*(P_global_CG+j),*(charge+j));
	}
*/

	check = qdVConjPassemble( A, connect, coord, el_matl, matl, P_global_CG, Volt);
	if(!check) printf( " Problems with qdVConjPassemble \n");

	free(mem_double);

	return 1;
}


