/*
    This utility function assembles the matrices for a finite 
    element program which does analysis on a quad.  It is for
    calculating the voltage potetial.

    Unlike the waveguide matrices calculated in "qdekasmbl.c",
    the calculation of the potential uses node based interpolation.
    You can still use the reference:

        Jin, Jianming, The Finite Element Method in Electromagnetics,
            John Wiley & Sons, New York, 1993.

		Updated 12/18/09

    SLFCEM source file
    Version:  1.1
    Copyright (C) 1999-2009  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "qdeconst.h"
#include "qdestruct.h"

extern int analysis_flag, Vdof, numel, numnp, sof;
extern int AB_matrix_store, Bzz_matrix_store, gauss_stress_flag;
extern int LU_decomp_flag, Bzz_LU_decomp_flag, numel_V, numel_V_K, numel_V_P;
extern double dcdx[nsdsq*num_int], shg[sosh], shg_node[sosh], shl[sosh],
	shg_E[sosh_E], shg_node_E[sosh_E], curl_E[num_int*epel], shl_node[sosh],
	shl_node2[sosh_node2], w[num_int], *Area0, sigma;

int globalConjKassemble(double *, int *, int , double *,
	double *, int , int , int );

int globalKassemble(double *, int *, double *, int *, int );

int matX( double *,double *,double *, int ,int ,int );

int matXT( double *, double *, double *, int, int, int);

int quadB_E( double *, double *, double *, double *, double *, double *, double *,
	double *, double *, double *);

int qdshg( double *, int, double *, double *, double *);

int qdVKassemble( double *KV, int *connect, double *coord, int *el_matl, double *charge,
	int *id, int *idiag, double *KV_diag, int *lm, MATL *matl, double *node_counter,
	double *Volt)
{
	int i, i1, i2, j, k, dof_el[neqel], Vdof_el[Vneqel4], sdof_el[npel*nsd];
	int check, counter, node, node0, node1, node2, node3, edge;
	int matl_num;
	double permit, permea, kosq, G;
	double B[npel], gradB[soB8], gradB2[soB8];
	double K_temp[Vneqlsq16], K_el[Vneqlsq16];
	double charge_el[Vneqel4], Volt_el[Vneqel4];
	double coord_el_trans[npel*nsd], length[epel], lengthsq[epel];
	double stress_el[sdim], strain_el[sdim];
	double det[num_int];
	double fdum, fdum2;

	for( k = 0; k < numel; ++k )
	{

		matl_num = *(el_matl+k);
		permit = matl[matl_num].er;
		permea = matl[matl_num].ur;
		kosq = matl[matl_num].ko*matl[matl_num].ko;

		/*printf("lamda, mu, permit, permea  %f %f %f %f \n", lamda, mu, Emod, Pois);*/

/* Create the coord_el transpose vector for one element */

		for( j = 0; j < npel; ++j )
		{
			node = *(connect+npel*k+j);

			*(sdof_el+nsd*j) = nsd*node;
			*(sdof_el+nsd*j+1) = nsd*node+1;

			*(coord_el_trans+j) = *(coord+*(sdof_el+nsd*j));
			*(coord_el_trans+npel*1+j) = *(coord+*(sdof_el+nsd*j+1));

			*(dof_el+ndof*j) = ndof*node;
			*(dof_el+ndof*j+1) = ndof*node+1;

			*(Vdof_el+Vndof*j) = Vndof*node;

/* Count the number of times a particular node is part of an element */

			if(analysis_flag == 1)
				*(node_counter + node) += 1.0;
		}

/* Assembly of the dcdx and shg matrix for each integration point */

		check=qdshg( det, k, shl, shg, coord_el_trans);
		if(!check) printf( "Problems with qdshg\n");

/* The loop over j below calculates the 4 points of the gaussian integration 
   for several quantities */

		memset(K_el,0, Vneqlsq16*sof);

		for( j = 0; j < num_int; ++j )
		{

		    memset(B,0,npel*sof);
		    memset(K_temp,0,Vneqlsq16*sof);

/* Assembly of the component matrices */

		    check = quadB_V((shl+npel*(nsd2+1)*j), (shg+npel*(nsd2+1)*j),
			B, gradB);
		    if(!check) printf( "Problems with quadB_V\n");

		    memcpy(gradB2, gradB, soB8*sizeof(double));

		    fdum = *(w+j)*(*(det+j));

		    check=matXT(K_temp, gradB, gradB2, Vneqel4, Vneqel4, nsd2);
		    if(!check) printf( "Problems with matXT  \n");

		    for( i2 = 0; i2 < Vneqlsq16; ++i2 )
		    {
			*(K_el + i2) += *(K_temp + i2)*fdum*permit;
		    }
		}

/*
		printf("\n K_el %4d\n", k);
		for( i2 = 0; i2 < Vneqel4; ++i2 )
		{
			for( i1 = 0; i1 < Vneqel4; ++i1 )
			{
				printf("%12.8f ",*(K_el + Vneqel4*i2 + i1));
			}
			printf("\n");
		}
		printf("\n");
*/


		for( j = 0; j < Vneqel4; ++j )
		{
			*(Volt_el + j) = *(Volt + *(Vdof_el+j));
		}

		check = matX(charge_el, K_el, Volt_el, Vneqel4, 1, Vneqel4);
		if(!check) printf( "Problems with matX \n");

		if(analysis_flag == 1)
		{

/* Compute the equivalant nodal charges based on prescribed displacements */

			for( j = 0; j < Vneqel4; ++j )
			{
				*(charge + *(Vdof_el+j)) -= *(charge_el + j);
				/*printf("%4d %12.8f\n", *(Vdof_el+j), *(charge + *(Vdof_el+j)));*/
			}

#if 1
/* Assembly of either the global skylined K matrix or numel_V of the
   element K matrices if the Conjugate Gradient method is used */

			if(LU_decomp_flag)
			{
			    check = globalKassemble(KV, idiag, K_el, (lm + k*Vneqel4), Vneqel4);
			    if(!check) printf( "Problems with globalKassemble \n");
			}
			else
			{
			    check = globalConjKassemble(KV, Vdof_el, k, KV_diag, K_el,
				Vneqel4, Vneqlsq16, numel_V_K);
			    if(!check) printf( "Problems with globalConjKassemble \n");
			}
#endif

		}
		else
		{

/* Compute the reaction charges */

			for( j = 0; j < Vneqel4; ++j )
			{
				*(charge + *(Vdof_el+j)) += *(charge_el + j);
				/*printf("%4d %12.8f\n", *(Vdof_el+j), *(charge + *(Vdof_el+j)));*/
			}
		}
	}

	if(analysis_flag == 1)
	{

/* Contract the global force matrix using the id array only if LU decomposition
   is used. */

	  if(LU_decomp_flag)
	  {
	     counter = 0;
	     for( i = 0; i < Vdof ; ++i )
	     {
		if( *(id + i ) > -1 )
		{
			*(charge + counter ) = *(charge + i );
			/*printf("%4d %4d %12.8f\n", counter, i,
				*(charge + counter));*/
			++counter;
		}
	     }
	  }
	}

	return 1;
}

