/*
     This library function assembles the [B] matrix in:

            [Btrans][B]

     the [Bedge] matrix in:

            [Bedge trans][Bedge]:

     the [curlBedge] matrix in:

            [curlBedge trans][curlBedge]:

     and the [gradB] matrix in:

            [gradB trans][Bedge], [Bedge trans][gradB], [gradB trans][gradB]

     for quad elements.  It is for eletromagnetism.  All these 
     matrices are in 8.41 to 8.45 on page 246 of "The Finite Element
     Method in Electromagnetics" by Jianming Jin.

     It is a C translation of the subroutine QDCB from the book
     "The Finite Element Method" by Thomas Hughes, page 780.

     SLFCEM source file
     Version:  1.1
     Copyright (C) 1999-2009  San Le

     The source code contained in this file is released under the
     terms of the GNU Library General Public License.
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "qdeconst.h"

int quadB_E(double *dcdx, double *shl, double *shg, double *shg_E,
	double *curl_E, double *B, double *gradB, double *Bedge,
	double *curlBedge, double *length)
{
/*
 ....  SET UP THE SHAPE FUNCTION MATRICES FOR
       TWO-DIMENSIONAL CONTINUUM ELEMENTS

		Updated 12/19/09
*/
	int i,i2,i2m1;
	for( i = 0; i < npel; ++i )
	{
		*(B+i)              = *(shg+npel*2+i);

		*(gradB+i)          = *(shg+i);
		*(gradB+neqel4*1+i) = *(shg+npel*1+i); 
	}

#if 1
        *(Bedge)            = *(shg_E); 
        *(Bedge + 1)        = *(shg_E + 1);
        *(Bedge + 2)        = *(shg_E + 2);
        *(Bedge + 3)        = *(shg_E + 3);
        *(Bedge+neqel4*1)   = *(shg_E + epel*1);
        *(Bedge+neqel4*1+1) = *(shg_E + epel*1 + 1);
        *(Bedge+neqel4*1+2) = *(shg_E + epel*1 + 2);
        *(Bedge+neqel4*1+3) = *(shg_E + epel*1 + 3);

	*(curlBedge)     = *(curl_E);
	*(curlBedge + 1) = *(curl_E + 1);
	*(curlBedge + 2) = *(curl_E + 2);
	*(curlBedge + 3) = *(curl_E + 3);
#endif

/* Below is correct for Bedge, but I decided to store shg_E. */

#if 0
	*(Bedge)            = *(dcdx)*(*(shl + 1))*(*(length));
	*(Bedge + 1)        = *(dcdx)*(*(shl + 2))*(*(length + 1));
	*(Bedge + 2)        = *(dcdx+2)*(*(shl + npel*1 + 3))*(*(length + 2));
	*(Bedge + 3)        = *(dcdx+2)*(*(shl + npel*1 + 2))*(*(length + 3));
	*(Bedge+neqel4*1)   = *(dcdx+1)*(*(shl + 1))*(*(length));
	*(Bedge+neqel4*1+1) = *(dcdx+1)*(*(shl + 2))*(*(length + 1));
	*(Bedge+neqel4*1+2) = *(dcdx+3)*(*(shl + npel*1 + 3))*(*(length + 2));
	*(Bedge+neqel4*1+3) = *(dcdx+3)*(*(shl + npel*1 + 2))*(*(length + 3));

/* Below is correct for Bedge, but I decided to store curl_E. */

        *(curlBedge)     =
                *(length)*(*(dcdx)*(*(dcdx+3))-*(dcdx+1)*(*(dcdx+2)))/4.0;
        *(curlBedge + 1) =
                *(length + 1)*(*(dcdx+2)*(*(dcdx+1))-*(dcdx+3)*(*(dcdx)))/4.0;
        *(curlBedge + 2) =
                *(length + 2)*(*(dcdx+2)*(*(dcdx+1))-*(dcdx+3)*(*(dcdx)))/4.0;
        *(curlBedge + 3) =
                *(length + 3)*(*(dcdx)*(*(dcdx+3))-*(dcdx+1)*(*(dcdx+2)))/4.0;

#endif

	return 1;
}

int quadB_V( double *shl, double *shg, double *B, double *gradB)
{
/*
 ....  SET UP THE SHAPE FUNCTION MATRICES FOR
       TWO-DIMENSIONAL CONTINUUM ELEMENTS

		Updated 12/29/09
*/
	int i,i2,i2m1;
	for( i = 0; i < npel; ++i )
	{
		*(B+i)              = *(shg+npel*2+i);

		*(gradB+i)          = *(shg+i);
		*(gradB+Vneqel4*1+i) = *(shg+npel*1+i); 
	}

	return 1;
}

