/*
    This utility function takes the product of a vector with the
    either the Btt, Btz, or Bzt matrix.  This is for a finite element program
    which does analysis on a triangle.  It is for modal analysis.

		Updated 12/6/09

    SLFCEM source file
    Version:  1.1
    Copyright (C) 1999-2009  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "treconst.h"
#include "trestruct.h"

#define SMALL          1.0e-12

extern int numed, numel, numel_EM, numnp, dof, EMdof, sof;
extern double shg[sosh], shg_node[sosh], shl[sosh], shg_E[sosh_E],
	curl_E[epel], w[num_int];
extern int AB_matrix_store;

int matXT(double *, double *, double *, int, int, int);

int matXT(double *, double *, double *, int, int, int);

int matX(double *, double *, double *, int, int, int);

int triangleB_mass(double *,double *);

int triangleB_E( double *, double *, double *, double *, double *, double *,
	double *, double *, double *);

int trshg_E( double *, int , int , double *, double *, double *, double *, double *,
	double *, double *);

int trBPassemble(int *connect, double *coord, int *edge_connect, int *el_matl, double *Btt,
	MATL *matl, int *node_edge_connect, double *P_global, double *EMedge)
{
	int i, i1, i2, i3, j, k, dof_el[neqel], EMdof_el[neqel], sdof_el[npel*nsd];
	int check, counter, node, node0, node1, node2, edge;
	int matl_num;
	double permit, permea, kosq, G;
	double B[npel], gradB[soB6], Bedge[soB6], curlBedge[epel],
		Bedge2[soB6], curlBedge2[epel];
	double Btt_temp[neqlsq9], Btt_el[neqlsq9];
	double EMedge_el[neqel3];
	double coord_el[npel*nsd], coord_el_trans[npel*nsd],
		coord_el_local[npel*nsd2], coord_el_local_trans[npel*nsd2];
	double length[epel], lengthsq[epel];
	double X1, X2, X3, Y1, Y2, Y3, Z1, Z2, Z3, X12, X13, Y12, Y13, Z12, Z13;
	double Xp, Xr, Xq, Yp, Yr, Yq, Zp, Zr, Zq, Xpr, Xpq, Ypr, Ypq, Zpr, Zpq;
	double Lpq, Ltr, Mpq, Mtr, Npq, Ntr, Dpq, Dpt, Dtr;
	double det[num_int], area_el, wXdet;
	double P_el[neqel];
	double fdum, fdum2;
	double shg_sign[3];

	memset(P_global, 0, EMdof*sof);

	memcpy(shg,shl,sosh*sizeof(double));

	if(AB_matrix_store)
	{

/* Assemble P matrix using stored element Btt matrices */

	    for( k = 0; k < numel_EM; ++k )
	    {

		for( j = 0; j < epel; ++j )
		{
			edge = *(edge_connect+epel*k+j);

			*(EMdof_el+edof*j) = edof*edge;
		}

/* Assembly of the global P matrix */

		for( j = 0; j < neqel3; ++j )
		{
			*(EMedge_el + j) = *(EMedge + *(EMdof_el+j));
		}

		check = matX(P_el, (Btt + k*neqlsq9), EMedge_el, neqel3, 1, neqel3);
		if(!check) printf( "Problems with matX \n");

		for( j = 0; j < neqel3; ++j )
		{
			*(P_global+*(EMdof_el+j)) += *(P_el+j);
		}
	    }
	}
	else
	{

/* Assemble P matrix by re-deriving element B_matrix matrices */

	    for( k = 0; k < numel_EM; ++k )
	    {
		matl_num = *(el_matl+k);
		permit = matl[matl_num].er;
		permea = matl[matl_num].ur;
		kosq = matl[matl_num].ko*matl[matl_num].ko;

		/*printf("lamda, mu, Emod, Pois  %f %f %f %f \n", lamda, mu, Emod, Pois);*/

/* Create the coord_el vector for one element */

		for( j = 0; j < npel; ++j )
		{
			node = *(connect+npel*k+j);

			*(sdof_el+nsd*j) = nsd*node;
			*(sdof_el+nsd*j+1) = nsd*node+1;

			*(coord_el_trans+j)=*(coord+*(sdof_el+nsd*j));
			*(coord_el_trans+npel*1+j)=*(coord+*(sdof_el+nsd*j+1));

			*(dof_el+ndof*j) = ndof*node;
			*(dof_el+ndof*j+1) = ndof*node+1;

		}

/* Create edge_el for one element */

		for( j = 0; j < epel; ++j )
		{
			edge = *(edge_connect+epel*k+j);
			node1 = *(node_edge_connect+edge*nped);
			node2 = *(node_edge_connect+edge*nped + 1);

			*(EMdof_el+edof*j) = edof*edge;

/* Calculate the edge lengths */

			fdum = *(coord+node2*nsd) - *(coord+node1*nsd);
			fdum2 = *(coord+node2*nsd+1) - *(coord+node1*nsd+1);
			*(lengthsq + j) = fdum*fdum + fdum2*fdum2;
			*(length + j) = sqrt(*(lengthsq + j));

		}

		X1 = *(coord_el_trans);        Y1 = *(coord_el_trans + npel*1);
		X2 = *(coord_el_trans + 1);    Y2 = *(coord_el_trans + npel*1 + 1);
		X3 = *(coord_el_trans + 2);    Y3 = *(coord_el_trans + npel*1 + 2);

/* Area is simply the the cross product of the sides connecting node 1 to node 0
   and node 2 to node 0, divided by 2.
*/

		fdum = (X2 - X1)*(Y3 - Y1) - (X3 - X1)*(Y2 - Y1);

		if( fdum <= 0.0 )
		{
			printf("the element (%d) is inverted; 2*Area:%f\n", k, fdum);
			return 0;
		}

/* The variables below are needed to address the issue explained in:

   ~/slfcem-1.1/EDGE-ELEMENTS

*/
		node0 = *(connect+npel*k);
		node1 = *(connect+npel*k+1);
		node2 = *(connect+npel*k+2);

		shg_sign[0] = 1.0; shg_sign[1] = 1.0; shg_sign[2] = 1.0;

		if(node0 > node1) shg_sign[0] = -1.0;
		if(node1 > node2) shg_sign[1] = -1.0;
		if(node2 > node0) shg_sign[2] = -1.0;

		check=trshg_E( det, k, 1, shl, shg, shg_E, curl_E,
			coord_el_trans, length, shg_sign);
		if(!check) printf( "Problems with trshg_E \n");

                memset(Btt_el,0,neqlsq9*sof);

		for( j = 0; j < num_int; ++j )
		{

		    memset(B,0,npel*sof);
		    memset(gradB,0,soB6*sof);
		    memset(Bedge,0,soB6*sof);
		    memset(curlBedge,0,epel*sof);
		    memset(Btt_temp,0,neqlsq9*sof);

/* Assembly of the component matrices */

		    check = triangleB_E((shl+npel*(nsd+1)*j), (shg+npel*(nsd+1)*j),
			    (shg_E+epel*nsd2*j), (curl_E), B, gradB, Bedge,
			    curlBedge, length);
		    if(!check) printf( "Problems with triangleB_E\n");

		    memcpy(Bedge2, Bedge, soB6*sizeof(double));

		    check=matXT(Btt_temp, Bedge, Bedge2, neqel3, neqel3, nsd);
		    if(!check) printf( "Problems with matXT  \n");

		    fdum = *(w+j)*(*(det));

		    for( i2 = 0; i2 < neqlsq9; ++i2 )
		    {
			  *(Btt_el+i2) += *(Btt_temp+i2)*fdum*permit;
		    }

		}

/* Assembly of the global P matrix */

		memset(EMedge_el,0,neqel3*sof);
		for( j = 0; j < neqel3; ++j )
		{
			*(EMedge_el + j) = *(EMedge + *(EMdof_el+j));
		}

		check = matX(P_el, Btt_el, EMedge_el, neqel3, 1, neqel3);
		if(!check) printf( "Problems with matX \n");

		for( j = 0; j < neqel3; ++j )
		{
			*(P_global+*(EMdof_el+j)) += *(P_el+j);
		}

	    }
	}

	return 1;
}
