/*
    This utility code uses the conjugate gradient method
    to solve the linear system [K][U] = [f] for a finite
    element program which does analysis on a triangle.  The first function
    assembles the P matrix.  It is called by the second function
    which allocates the memory and goes through the steps of the algorithm.
    These go with the calculation of displacement.

	        Updated 12/16/09

    SLFCEM source file
    Version:  1.1
    Copyright (C) 1999-2009  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "treconst.h"
#include "trestruct.h"

#define SMALL      1.e-20

extern int analysis_flag, dof, EMdof, numel, numnp, plane_stress_flag, sof;
extern int LU_decomp_flag, numel_EM, numel_EM_K, numel_EM_P;
extern double shg[sosh], shg_node[sosh], shl[sosh], shg_E[sosh_E],
	shg_node_E[sosh_E], curl_E[epel], shl_node[sosh],
	shl_node2[sosh_node2], w[num_int], *Area0, sigma;
extern int  iteration_max, iteration_const, iteration;
extern double tolerance;

int matX(double *,double *,double *, int ,int ,int );

int matXT(double *, double *, double *, int, int, int);

int triangleB(double *,double *);

int trshg_E( double *, int , int , double *, double *, double *, double *, double *,
	double *, double *);

int dotX(double *, double *, double *, int);

int EBoundary( double *, BOUND );


int trEConjPassemble(double *A, int *connect, double *coord, int *edge_connect,
        int *el_matl, MATL *matl, int *node_edge_connect, double *P_global_CG,
	double *EMedge)
{
/* This function assembles the P_global_CG matrix for the displacement calculation by
   taking the product [K_el]*[U_el].  Some of the [K_el] is stored in [A].

                        Updated 11/25/09
*/

        int i, i1, i2, j, k, dof_el[neqel], EMdof_el[neqel3], sdof_el[npel*nsd];
        int check, counter, node, node0, node1, node2, edge;
        int matl_num;
        double permit, permea, kosq, G;
        double fdum, fdum2;
        double lamda, mu;
        double B[npel], gradB[soB6], BXB[neqlsq9], Bedge[soB6], curlBedge[epel],
		Bedge2[soB6], curlBedge2[epel];
        double Att_temp[neqlsq9], Btt_temp[neqlsq9];
        double Att_el[neqlsq9], Att_sigma_el[neqlsq9], Btt_el[neqlsq9];
        double K_temp[neqlsq], K_el[neqlsq], EMedge_el[neqel3];
        double coord_el_trans[npel*nsd], length[epel], lengthsq[epel];
	double X1, X2, X3, Y1, Y2, Y3;
        double det[num_int];
        double P_el[neqel3];
	double shg_sign[3];

        memset(P_global_CG,0,EMdof*sof);

	for( k = 0; k < numel_EM_K; ++k )
	{

		for( j = 0; j < epel; ++j )
		{
			edge = *(edge_connect+epel*k+j);

			*(EMdof_el+edof*j) = edof*edge;

		}


/* Assembly of the global P matrix */

		for( j = 0; j < neqel3; ++j )
		{
			*(EMedge_el + j) = *(EMedge + *(EMdof_el+j));
		}

		check = matX(P_el, (A+k*neqlsq9), EMedge_el, neqel3, 1, neqel3);
		if(!check) printf( "Problems with matX \n");

		for( j = 0; j < neqel3; ++j )
		{
			*(P_global_CG+*(EMdof_el+j)) += *(P_el+j);
		}
	}
	for( k = numel_EM_K; k < numel_EM; ++k )
	{
		matl_num = *(el_matl+k);
		permit = matl[matl_num].er;
		permea = matl[matl_num].ur;
		kosq = matl[matl_num].ko*matl[matl_num].ko;

		/*printf("lamda, mu, Emod, Pois  %f %f %f %f \n", lamda, mu, Emod, Pois);*/

/* Create the coord_el vector for one element */

		for( j = 0; j < npel; ++j )
		{
			node = *(connect+npel*k+j);

			*(sdof_el+nsd*j) = nsd*node;
			*(sdof_el+nsd*j+1) = nsd*node+1;

			*(coord_el_trans+j)=*(coord+*(sdof_el+nsd*j));
			*(coord_el_trans+npel*1+j)=*(coord+*(sdof_el+nsd*j+1));

			*(dof_el+ndof*j) = ndof*node;
			*(dof_el+ndof*j+1) = ndof*node+1;
		}

/* Create edge_el for one element */

		for( j = 0; j < epel; ++j )
		{
			edge = *(edge_connect+epel*k+j);
			node1 = *(node_edge_connect+edge*nped);
			node2 = *(node_edge_connect+edge*nped + 1);

			*(EMdof_el+edof*j) = edof*edge;

/* Calculate the edge lengths */

			fdum = *(coord+node2*nsd) - *(coord+node1*nsd);
			fdum2 = *(coord+node2*nsd+1) - *(coord+node1*nsd+1);
			*(lengthsq + j) = fdum*fdum + fdum2*fdum2;
			*(length + j) = sqrt(*(lengthsq + j));
		}

		X1 = *(coord_el_trans);        Y1 = *(coord_el_trans + npel*1);
		X2 = *(coord_el_trans + 1);    Y2 = *(coord_el_trans + npel*1 + 1);
		X3 = *(coord_el_trans + 2);    Y3 = *(coord_el_trans + npel*1 + 2);

/* Area is simply the the cross product of the sides connecting node 1 to node 0
   and node 2 to node 0, divided by 2.
*/
		fdum = (X2 - X1)*(Y3 - Y1) - (X3 - X1)*(Y2 - Y1);

		if( fdum <= 0.0 )
		{
			printf("the element (%d) is inverted; 2*Area:%f\n", k, fdum);
			return 0;
		}

/* The variables below are needed to address the issue explained in:

   ~/slfcem-1.1/EDGE-ELEMENTS

*/
		node0 = *(connect+npel*k);
		node1 = *(connect+npel*k+1);
		node2 = *(connect+npel*k+2);

		shg_sign[0] = 1.0; shg_sign[1] = 1.0; shg_sign[2] = 1.0;

		if(node0 > node1) shg_sign[0] = -1.0;
		if(node1 > node2) shg_sign[1] = -1.0;
		if(node2 > node0) shg_sign[2] = -1.0;

		check=trshg_E( det, k, 1, shl, shg, shg_E, curl_E,
			coord_el_trans, length, shg_sign);
		if(!check) printf( "Problems with trshg_E \n");

                memset(EMedge_el,0,neqel3*sof);
                memset(K_el,0,neqlsq9*sof);
                memset(Att_el,0,neqlsq9*sof);
                memset(Att_sigma_el,0,neqlsq9*sof);
                memset(Btt_el,0,neqlsq9*sof);

		for( j = 0; j < num_int; ++j )
		{

		    memset(B,0,npel*sof);
		    memset(gradB,0,soB6*sof);
		    memset(Bedge,0,soB6*sof);
		    memset(curlBedge,0,epel*sof);
		    memset(Att_temp,0,neqlsq9*sof);
		    memset(Btt_temp,0,neqlsq9*sof);
		    memset(BXB,0,neqlsq9*sof);
		    memset(K_temp,0,neqlsq9*sof);

/* Assembly of the component matrices */

		    check = triangleB_E((shl+npel*(nsd+1)*j), (shg+npel*(nsd+1)*j),
			    (shg_E+epel*nsd2*j), (curl_E), B, gradB, Bedge,
			    curlBedge, length);
		    if(!check) printf( "Problems with triangleB_E\n");

		    memcpy(curlBedge2, curlBedge, epel*sizeof(double));
		    memcpy(Bedge2, Bedge, soB6*sizeof(double));

		    check=dyadicX(Att_temp, curlBedge, curlBedge2, neqel3);
		    if(!check) printf( "Problems with dyadicX \n");

		    check=matXT(Btt_temp, Bedge, Bedge2, neqel3, neqel3, nsd);
		    if(!check) printf( "Problems with matXT  \n");

		    fdum = *(w+j)*(*(det));

		    for( i2 = 0; i2 < neqlsq9; ++i2 )
		    {
			  *(Btt_el+i2) += *(Btt_temp+i2)*fdum*permit;
			  *(Att_el+i2) += *(Att_temp+i2)*fdum/permea;
		    }

		}

/* Att_sigma is the modified Att matrix which makes the eigenvalue calculation
   find the eigenvalue nearest the value of "sigma".  See page 583-584 of:

      Hughes, Thomas J. R., The Finite Element Method, Prentice-Hall Inc.,
          Englewood Cliffs,  1987.

*/
		for( i2 = 0; i2 < neqlsq9; ++i2 )
		{
		    *(Att_sigma_el + i2) = *(Att_el + i2) - sigma*(*(Btt_el + i2));
		}

/* Assembly of the global P matrix */

		for( j = 0; j < neqel3; ++j )
		{
			*(EMedge_el + j) = *(EMedge + *(EMdof_el+j));
		}

		check = matX(P_el, Att_sigma_el, EMedge_el, neqel3, 1, neqel3);
		if(!check) printf( "Problems with matX \n");

		for( j = 0; j < neqel3; ++j )
		{
			*(P_global_CG+*(EMdof_el+j)) += *(P_el+j);
		}
	}

	return 1;
}


int trEConjGrad(double *A, BOUND bc, int *connect, double *coord, int *edge_connect,
	int *el_matl, double *charge, double *K_diag, MATL *matl, int *node_edge_connect,
	double *EMedge)
{
/* This function does memory allocation and uses the conjugate gradient
   method to solve the linear system arising from the calculation of
   displacements.  It also makes the call to trConjPassemble to get the
   product of [A]*[p].

                        Updated 8/30/09

   It is taken from the algorithm 10.3.1 given in "Matrix Computations",
   by Golub, page 534.
*/
	int i, j, sofmf, ptr_inc;
	int check, counter;
	double *mem_double;
	double *p, *P_global_CG, *r, *z;
	double alpha, alpha2, beta;
	double rXz_old, rXz;

/* For the doubles */
	sofmf = 4*EMdof;
	mem_double=(double *)calloc(sofmf,sizeof(double));

	if(!mem_double )
	{
		printf( "failed to allocate memory for doubles\n ");
		exit(1);
	}

/* For the Conjugate Gradient Method doubles */

	                                        ptr_inc = 0;
	p=(mem_double+ptr_inc);                 ptr_inc += EMdof;
	P_global_CG=(mem_double+ptr_inc);       ptr_inc += EMdof;
	r=(mem_double+ptr_inc);                 ptr_inc += EMdof;
	z=(mem_double+ptr_inc);                 ptr_inc += EMdof;

/* Using Conjugate gradient method to find displacements */

	memset(P_global_CG,0,EMdof*sof);
	memset(p,0,EMdof*sof);
	memset(r,0,EMdof*sof);
	memset(z,0,EMdof*sof);

	for( j = 0; j < EMdof; ++j )
	{
		*(K_diag + j) += SMALL;
		*(r+j) = *(charge+j);
		*(z + j) = *(r + j)/(*(K_diag + j));
		*(p+j) = *(z+j);
	}
	check = EBoundary (r, bc);
	if(!check) printf( " Problems with Boundary \n");

	check = EBoundary (p, bc);
	if(!check) printf( " Problems with Boundary \n");

	alpha = 0.0;
	alpha2 = 0.0;
	beta = 0.0;
	rXz = 1000.0;
	counter = 0;
	check = dotX(&rXz_old, r, z, EMdof);

	printf("\n iteration %3d iteration max %3d \n", iteration, iteration_max);
	/*for( iteration = 0; iteration < iteration_max; ++iteration )*/
	while(rXz > tolerance && counter < iteration_max )
	{

		printf( "\n %3d %16.8e\n",counter, rXz);
		check = trEConjPassemble( A, connect, coord, edge_connect, el_matl, matl,
			node_edge_connect, P_global_CG, p);
		if(!check) printf( " Problems with trConjPassemble \n");
		check = EBoundary (P_global_CG, bc);
		if(!check) printf( " Problems with EBoundary \n");
		check = dotX(&alpha2, p, P_global_CG, EMdof);	
		alpha = rXz_old/(SMALL + alpha2);

		for( j = 0; j < EMdof; ++j )
		{
		    /*printf( "%4d %14.5e  %14.5e  %14.5e  %14.5e  %14.5e %14.5e\n",j,alpha,
			beta,*(EMedge+j),*(r+j),*(P_global_CG+j),*(p+j));*/
		    *(EMedge+j) += alpha*(*(p+j));
		    *(r+j) -=  alpha*(*(P_global_CG+j));
		    *(z + j) = *(r + j)/(*(K_diag + j));
		}

		check = dotX(&rXz, r, z, EMdof);
		beta = rXz/(SMALL + rXz_old);
		rXz_old = rXz;
		
		for( j = 0; j < EMdof; ++j )
		{
		    /*printf("\n  %3d %12.7f  %14.5f ",j,*(EMedge+j),*(P_global_CG+j));*/
		    /*printf( "%4d %14.5f  %14.5f  %14.5f  %14.5f %14.5f\n",j,alpha,
			*(EMedge+j),*(r+j),*(P_global_CG+j),*(charge+j));
		    printf( "%4d %14.8f  %14.8f  %14.8f  %14.8f %14.8f\n",j,
			*(EMedge+j)*beta,*(r+j)*beta,*(P_global_CG+j)*alpha,
			*(charge+j)*alpha);*/
		    *(p+j) = *(z+j)+beta*(*(p+j));
		}
		check = EBoundary (p, bc);
		if(!check) printf( " Problems with EBoundary \n");

		++counter;
	}

	if(counter > iteration_max - 1 )
	{
		printf( "\nMaximum iterations %4d reached.  Residual is: %16.8e\n",counter,rXz);
		printf( "Problem may not have converged during Conj. Grad.\n");
	}
/*
The lines below are for testing the quality of the calculation:

1) r should be 0.0
2) P_global_CG( = A*EMedge ) - charge should be 0.0
*/

/*
	check = trEConjPassemble( A, connect, coord, edge_connect, el_matl, matl,
		node_edge_connect, P_global_CG, EMedge);
	if(!check) printf( " Problems with trEConjPassemble \n");

	for( j = 0; j < EMdof; ++j )
	{
		printf( "%4d %14.5f  %14.5f %14.5f  %14.5f  %14.5f %14.5f\n",j,alpha,beta,
			*(EMedge+j),*(r+j),*(P_global_CG+j),*(charge+j));
	}
*/

	free(mem_double);

	return 1;
}


