/*
    This is the include file "trconst.h" for the finite element progam 
    which uses triangle elements to calculate electro-magnetic fields.

                Updated 12/23/09

    SLFCEM source file
    Version:  1.1
    Copyright (C) 1999-2009  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
*/


#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define pt5      .5 
#define pt25     .25
#define pt1667  1.0/6.0
#define pt6667  2.0/3.0
#define pt3333  1.0/3.0
#define sq3     1.732050808 
#define pi            3.141592654
#define speed_light   3.0e+08                /* Speed of Light m/s. */


#define vac_permit 8.854e-12                 /* Permittivity in vacuum */
#define vac_permea 4*pi*e-7                  /* Permeability in vacuum */


#define nsd           2                      /* spatial dimensions per node */
#define nsd2          2                      /* spatial dimensions per node in local coordinates*/
#define nsdsq         4                      /* nsd squared */
#define nsdsq9        9                      /* nsd+1 squared */
#define ndof          2                      /* degrees of freedom per node */
#define ndof2         2                      /* degrees of freedom per node in local coordinates*/
#define npel          3                      /* nodes per element */
#define npel3         3                      /* nodes per element */
#define neqel         npel*ndof              /* degrees of freedom per element */
#define neqel6        npel*ndof2             /* degrees of freedom per element in local coordinates*/
#define num_int       3                      /* number of integration points */
#define neqlsq        neqel*neqel            /* neqel squared */
#define neqlsq36      neqel6*neqel6          /* neqel6 squared = 36*/
#define sdim          3                      /* stress dimensions per element */
#define soB           sdim*neqel6            /* size of B matrix */
#define MsoB          nsd2*neqel6            /* size of B_mass matrix */
#define sosh          (nsd2+1)*npel*num_int  /* size of shl and shg matrix */
#define sosh_node2    nsd2*2*num_int         /* size of shl_node2 */
#define sosh_E        epel*num_int*nsd2      /* size of shg_E matrix */
#define KB            1024.0                 /* number of bytes in kilobyte */
#define MB            1.0486e+06             /* number of bytes in megabyte */

#define MIN(x,y) (((x)<(y))?(x):(y))
#define MAX(x,y) (((x)>(y))?(x):(y))

/*    Below is the additional list of constants for the quad element eqd
      to calculate waveguides.
*/

#define edof          1                      /* degrees of freedom per edge */
#define epel          3                      /* edges per element */
#define nped          2                      /* nodes per edge */
#define neqel3        epel*edof              /* degrees of freedom per element for electromagnetics */
#define neqlsq9       neqel3*neqel3          /* EMeqel squared */
#define soB6          nsd2*npel              /* size of B matrix for electromagnetics */

/*    Below is the additional list of constants for the quad element eqd
      to calculate voltages from charge distributions.
*/

#define Vndof         1                      /* Voltage degrees of freedom per node */
#define Vneqel3       npel*Vndof             /* degrees of freedom per element for voltage calculation */
#define Vneqlsq9      Vneqel3*Vneqel3        /* Vneqel3 squared */

/*
  title               problem title
  numel               number of elements
  numnp               number of nodal points 
  nmat                number of materials
  dof                 total number of degrees of freedom 
  Vdof                total number of degrees of freedom for Voltage calculation
  EMdof               total number of degrees of freedom for waveguide calculation
  coord               *(coord + 0) x coordinate of node
                      *(coord + 1) y coordinate of node
  connect             (0-3) connectivity array 
  matl                material structure
  Emod                young's modulus
  nu                  poisson's ratio
  force               *(force + 0) x component of applied load
                      *(force + 1) y component of applied load
  plane_stress_flag   0 flag indicating whether plane stress
                      1 or strain theory is used 
  analysis_flag       1 calculate unknown displacemnts
                      2 calculate reaction forces
  LU_decomp_flag      0 if numel <= 1125 elements, use LU Decomposition for
                        displacements
                      1 if numel > 1125 elements, use conjugate
                        gradient method for displacements
*/
