/*
    This utility function assembles the matrices for a finite
    element program which does analysis on a quad.  It is for
    eletromagnetism.

    This code solves for the more limited problem of finding
    the cut-off frequencies and corresponding eigenmodes.
    Unlike qdEKassemble which assembles all the matrices to solve
    for the system given by Equation 8.49 on page 247 of

        Jin, Jianming, The Finite Element Method in Electromagnetics,
            John Wiley & Sons, New York, 1993.

    I only assemble for the matrices given by 8.41 and solve for ko.
    This means that Att is given by the first term in the integral
    of 8.49 and Btt is the second.

                Updated 12/6/09

    SLFCEM source file
    Version:  1.1
    Copyright (C) 1999-2009  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "treconst.h"
#include "trestruct.h"

#define  DEBUG         0
#define SMALL          1.0e-12

extern int analysis_flag, numel, numnp, sof;
extern int AB_matrix_store, Bzz_matrix_store;
extern int LU_decomp_flag, numel_EM_K, numel_EM_P;
extern double shg[sosh], shg_node[sosh], shl[sosh], shg_E[sosh_E], shg_node_E[sosh_E],
	curl_E[epel], shl_node[sosh], shl_node2[sosh_node2], w[num_int], *Area0, sigma;

int globalConjKassemble(double *, int *, int , double *,
	double *, int , int , int );

int globalKassemble(double *, int *, double *, int *, int );

int matX( double *,double *,double *, int ,int ,int );

int dyadicX( double *,double *,double *, int);

int matXT( double *, double *, double *, int, int, int);

int triangleB_E( double *, double *, double *, double *, double *, double *,
	double *, double *, double *);

int trshg_E( double *, int , int , double *, double *, double *, double *, double *,
	double *, double *);

int trEKassemble(double *Att, double *Att_sigma, double *Btt, int *connect,
	int *node_edge_connect, int *edge_connect, double *coord, int *el_matl,
	int *id, int *idiag, double *Att_diag, double *Att_sigma_diag,
	double *Bzz_diag, int *lm, MATL *matl, double *node_counter,
	double *edge_counter, double *Arean)
{
	int i, i1, i2, j, k, dof_el[neqel], EMdof_el[neqel3], sdof_el[npel*nsd];
	int check, counter, node, node0, node1, node2, edge;
	int matl_num;
	double permit, permea, kosq, G;
	double D11,D12,D21,D22;
	double lamda, mu;
	double B[soB], DB[soB], gradB[soB6], BXB[neqlsq9], Bedge[soB6], curlBedge[epel],
		Bedge2[soB6], curlBedge2[epel];
	double Att_temp[neqlsq9], Btt_temp[neqlsq9];
	double Att_el[neqlsq9], Att_sigma_el[neqlsq9], Btt_el[neqlsq9];
	double K_temp[neqlsq], K_el[neqlsq], K_local[neqlsq36], rotate[nsd2*nsd];
	double coord_el[npel*nsd], coord_el_trans[npel*nsd],
		coord_el_local[npel*nsd2], coord_el_local_trans[npel*nsd2];
	double length[epel], lengthsq[epel];
	double X1, X2, X3, Y1, Y2, Y3;
	double det[1], wXdet;
	double fdum, fdum2;
	double a1, a2, a3, b1, b2, b3, c1, c2, c3;
	double shg_sign[3];

	for( k = 0; k < numel; ++k )
	{

		matl_num = *(el_matl+k);
		permit = matl[matl_num].er;
		permea = matl[matl_num].ur;
		kosq = matl[matl_num].ko*matl[matl_num].ko;

		/*printf("lamda, mu, Emod, Pois  %f %f %f %f \n", lamda, mu, Emod, Pois);*/

/* Create the coord_el vector for one element */

		for( j = 0; j < npel; ++j )
		{
			node = *(connect+npel*k+j);

			*(sdof_el+nsd*j) = nsd*node;
			*(sdof_el+nsd*j+1) = nsd*node+1;

			*(coord_el+nsd*j)=*(coord+*(sdof_el+nsd*j));
			*(coord_el+nsd*j+1)=*(coord+*(sdof_el+nsd*j+1));

			*(coord_el_trans+j)=*(coord+*(sdof_el+nsd*j));
			*(coord_el_trans+npel*1+j)=*(coord+*(sdof_el+nsd*j+1));

			*(dof_el+ndof*j) = ndof*node;
			*(dof_el+ndof*j+1) = ndof*node+1;

/* Count the number of times a particular node is part of an element */

			if(analysis_flag == 1)
				*(node_counter + node) += 1.0;
		}

/* Create edge_el for one element */

		for( j = 0; j < epel; ++j )
		{
			edge = *(edge_connect+epel*k+j);
			node1 = *(node_edge_connect+edge*nped);
			node2 = *(node_edge_connect+edge*nped + 1);

			*(EMdof_el+edof*j) = edof*edge;

/* Calculate the edge lengths */

			fdum = *(coord+node2*nsd) - *(coord+node1*nsd);
			fdum2 = *(coord+node2*nsd+1) - *(coord+node1*nsd+1);
			*(lengthsq + j) = fdum*fdum + fdum2*fdum2;
			*(length + j) = sqrt(*(lengthsq + j));

/* Count the number of times a particular edge is part of an element */

			if(analysis_flag == 1)
				*(edge_counter + edge) += 1.0;
		}

		X1 = *(coord_el_trans);        Y1 = *(coord_el_trans + npel*1);
		X2 = *(coord_el_trans + 1);    Y2 = *(coord_el_trans + npel*1 + 1);
		X3 = *(coord_el_trans + 2);    Y3 = *(coord_el_trans + npel*1 + 2);

		*(coord_el_local_trans) = X1;     *(coord_el_local_trans + 3) = Y1;
		*(coord_el_local_trans + 1) = X2; *(coord_el_local_trans + 4) = Y2;
		*(coord_el_local_trans + 2) = X3; *(coord_el_local_trans + 5) = Y3;

/* Area is simply the the cross product of the sides connecting node 1 to node 0
   and node 2 to node 0, divided by 2.
*/

		fdum = (X2 - X1)*(Y3 - Y1) - (X3 - X1)*(Y2 - Y1);

		if( fdum <= 0.0 )
		{
			printf("the element (%d) is inverted; 2*Area:%f\n", k, fdum);
			return 0;
		}

/* The variables below are needed to address the issue explained in:

   ~/slfcem-1.1/EDGE-ELEMENTS

*/
		node0 = *(connect+npel*k);
		node1 = *(connect+npel*k+1);
		node2 = *(connect+npel*k+2);

		*(shg_sign) = 1.0; *(shg_sign+1) = 1.0; *(shg_sign+2) = 1.0;

		if(node0 > node1) *(shg_sign)   = -1.0;
		if(node1 > node2) *(shg_sign+1) = -1.0;
		if(node2 > node0) *(shg_sign+2) = -1.0;

/* A factor of 0.5 is needed to do the integration.  See Eq. 3.I.34 in
   "The Finite Element Method" by Thomas Hughes, page 174
*/

		*(Arean + k) = .5*fdum;

		check=trshg_E( det, k, 1, shl, shg, shg_E, curl_E,
			coord_el_trans, length, shg_sign);
		if(!check) printf( "Problems with trshg_E \n");

                memset(K_el,0,neqlsq9*sof);
                memset(Att_el,0,neqlsq9*sof);
                memset(Att_sigma_el,0,neqlsq9*sof);
                memset(Btt_el,0,neqlsq9*sof);

		for( j = 0; j < num_int; ++j )
		{

		    memset(B,0,npel*sof);
		    memset(gradB,0,soB6*sof);
		    memset(Bedge,0,soB6*sof);
		    memset(curlBedge,0,epel*sof);
		    memset(Att_temp,0,neqlsq9*sof);
		    memset(Btt_temp,0,neqlsq9*sof);
		    memset(BXB,0,neqlsq9*sof);
		    memset(K_temp,0,neqlsq9*sof);

/* Assembly of the component matrices */

		    check = triangleB_E((shl+npel*(nsd+1)*j), (shg+npel*(nsd+1)*j),
			    (shg_E+epel*nsd2*j), (curl_E), B, gradB, Bedge,
			    curlBedge, length);
		    if(!check) printf( "Problems with triangleB_E\n");

		    memcpy(curlBedge2, curlBedge, epel*sizeof(double));
		    memcpy(Bedge2, Bedge, soB6*sizeof(double));

		    check=dyadicX(Att_temp, curlBedge, curlBedge2, neqel3);
		    if(!check) printf( "Problems with dyadicX \n");

		    check=matXT(Btt_temp, Bedge, Bedge2, neqel3, neqel3, nsd);
		    if(!check) printf( "Problems with matXT  \n");

		    fdum = *(w+j)*(*(det));

		    for( i2 = 0; i2 < neqlsq9; ++i2 )
		    {
			  *(Btt_el+i2) += *(Btt_temp+i2)*fdum*permit;
			  *(Att_el+i2) += *(Att_temp+i2)*fdum/permea;
		    }

		}

/* Att_sigma is the modified Att matrix which makes the eigenvalue calculation
   find the eigenvalue nearest the value of "sigma".  See page 583-584 of:

      Hughes, Thomas J. R., The Finite Element Method, Prentice-Hall Inc.,
          Englewood Cliffs,  1987.

*/
		for( i2 = 0; i2 < neqlsq9; ++i2 )
		{
		    *(Att_sigma_el + i2) = *(Att_el + i2) - sigma*(*(Btt_el + i2));
		}

		if(analysis_flag == 1)
		{

#if 1
/* Assembly of either the global skylined Att matrix or numel_EM of the
   element Att matrices if the Conjugate Gradient method is used */

			if(LU_decomp_flag)
			{
			    /*check = globalKassemble(Att, idiag, Att_el, (lm + k*neqel3),
				neqel3);
			    if(!check) printf( "Problems with globalKassemble \n");*/

			    check = globalKassemble(Att_sigma, idiag, Att_sigma_el,
				(lm + k*neqel3), neqel3);
			    if(!check) printf( "Problems with globalKassemble \n");
			}
			else
			{
			    /*check = globalConjKassemble(Att, EMdof_el, k, Att_diag, Att_el,
				neqel3, neqlsq9, numel_EM_K);
			    if(!check) printf( "Problems with globalConjKassemble \n");*/

			    check = globalConjKassemble(Att_sigma, EMdof_el, k,
				Att_sigma_diag, Att_sigma_el, neqel3, neqlsq9, numel_EM_K);
			    if(!check) printf( "Problems with globalConjKassemble \n");
			}
#endif


/*
                printf("\n Att_el el: %4d\n", k);
                for( i2 = 0; i2 < neqel3; ++i2 )
                {
                        printf("    ");
                        for( i1 = 0; i1 < neqel3; ++i1 )
                        {
                                printf("%12.8f ",*(Att_el + neqel3*i2 + i1));
                        }
                        printf("\n");
                }
                printf("\n");

                printf("\n Btt_el el: %4d\n", k);
                for( i2 = 0; i2 < neqel3; ++i2 )
                {
                        printf("    ");
                        for( i1 = 0; i1 < neqel3; ++i1 )
                        {
                                printf("%12.8f ",*(Btt_el + neqel3*i2 + i1));
                        }
                        printf("\n");
                }
                printf("\n");
*/

/* If there is enough memory, store all the element Att and Btt matrices */

			if(AB_matrix_store)
			{
			   for( j = 0; j < neqlsq9; ++j )
			   {
				*(Btt + neqlsq9*k + j) = *(Btt_el + j);
			   }

			   for( j = 0; j < neqlsq9; ++j )
			   {
				*(Att + neqlsq9*k + j) = *(Att_el + j);
			   }

			}
		}
	}

	return 1;
}

