/*
    This utility function calculates the value of the electric field for each node
    on a 3 node triangle element.  This is done element by element where the node values
    use the already calculated edge values.  Each edge has a tangentinal and normal
    component which will be decomposed into X and Y global directions at each node.
    Because there may be multiple edges which may contain the node, averaging is done
    at the end based on the number of edges which have the node.  This is used
    by the electro-magnetic triangle element.

		Updated 12/6/09

    SLFCEM source file
    Version:  1.1
    Copyright (C) 2009  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "treconst.h"
#include "trestruct.h"

#define SMALL          1.0e-12

extern int dof, numnp, numel, numed;
extern double shg[sosh], shg_node[sosh], shl[sosh], shg_E[sosh_E], shg_node_E[sosh_E],
	curl_E[num_int*epel], shl_node[sosh], shl_node2[sosh_node2], w[num_int], *Area0;


int trEMnode_calculate2( int *node_edge_connect, double *coord, double *EMedge,
	double *EMnode, double *node_counter)
{
	int i, i1, i2, j, k, dof_el[neqel], EMdof_el[neqel3], sdof_el[npel*nsd];;
	int check, counter, node, node1, node2, node_el[npel];
	int edge, edge0, edge1, edge2, edge3;
	double fdum, fdum1, fdum2, fdum3, fdum4;
	double EMedge_el[neqel3], EMnode_el[nsd];
	double length;
	double X1, X2, X3, Y1, Y2, Y3;
	double angle;

/* Because of the problems due to clockwise and counter-clockwise electric fields,
   I tried to avoid this by directly working with the electric fields on the edges
   and breaking them into X-Y components based on the spatial orientation of the edge.
   This does not work, which can be most easily seen by the fact that a diagonal
   edge on a perpendicular or horizontal boundary edge will likely contribute
   a tangentinal component to that edge, violating the zero tangentinal component
   boundary condition.
*/

	memset(EMnode,0,dof*sizeof(double));

	/*printf("\n");*/
	for( k = 0; k < numed; ++k )
	{

/* Create edge_el for one element */

	    node1 = *(node_edge_connect+k*nped);
	    node2 = *(node_edge_connect+k*nped + 1);

	    X1 = *(coord + nsd2*node1);   Y1 = *(coord + nsd2*node1 + 1);
	    X2 = *(coord + nsd2*node2);   Y2 = *(coord + nsd2*node2 + 1);

/* Calculate the edge lengths */

	    fdum  = (X2 - X1) + SMALL;
	    fdum2 = (Y2 - Y1);

	    angle = atan(fdum2/fdum);

	    *(EMnode + ndof*node1)   += *(EMedge + k)*cos(angle);
	    *(EMnode + ndof*node1+1) += *(EMedge + k)*sin(angle);

	    *(EMnode + ndof*node2)   += *(EMedge + k)*cos(angle);
	    *(EMnode + ndof*node2+1) += *(EMedge + k)*sin(angle);

	    printf("%4d %4d\n", node1, node2);
	    printf("%12.8e\n", *(EMedge + k));
	    printf("%12.8e\n", angle);
	    printf("%12.8e %12.8e \n", *(EMnode + ndof*node1), *(EMnode + ndof*node1+1));
	    printf("%12.8e %12.8e \n", *(EMnode + ndof*node2), *(EMnode + ndof*node2+1));

	}

/* Average all the components of EMnode depending based on its node count */

#if 1
	for( i = 0; i < numnp ; ++i )
	{
		/*printf("\nqqqq EMnode %4d %16.8e %16.8e %16.8e",
			i, *(node_counter+i), *(EMnode + ndof*i), *(EMnode + ndof*i+1));*/
		/**(EMnode + ndof*i)   /= *(node_counter+i);
		*(EMnode + ndof*i+1) /= *(node_counter+i);*/
		/*printf("\nqqqq EMnode %4d %16.8e %16.8e %16.8e",
			i, *(node_counter+i), *(EMnode + ndof*i), *(EMnode + ndof*i+1));*/
	}
#endif

	return 1;
}

