/*
     This library function assembles the B matrix in:

            [Btrans][B]

     the [Bedge] matrix in:

            [Bedge trans][Bedge]:

     the [curlBedge] matrix in:

            [curlBedge trans][curlBedge]:

     and the [gradB] matrix in:

            [gradB trans][Bedge], [Bedge trans][gradB], [gradB trans][gradB]

     for triangle elements.  It is for eletromagnetism.  All these
     matrices are in 8.41 to 8.45 on page 246 of "The Finite Element
     Method in Electromagnetics" by Jianming Jin.

     It is a C translation of the subroutine QDCB from the book
     "The Finite Element Method" by Thomas Hughes, page 780.

     SLFCEM source file
     Version:  1.1
     Copyright (C) 1999-2009  San Le

     The source code contained in this file is released under the
     terms of the GNU Library General Public License.
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "treconst.h"

int triangleB_E( double *shl, double *shg, double *shg_E, double *curl_E,
	double *B, double *gradB, double *Bedge, double *curlBedge,
	double *length)
{
/*
 ....  SET UP THE SHAPE FUNCTION MATRICES FOR
       TWO-DIMENSIONAL CONTINUUM ELEMENTS

		Updated 11/20/09
*/
	int i,i2,i2m1;
	for( i = 0; i < npel; ++i )
	{
		*(B+i)              = *(shg+npel*2+i);

		*(gradB+i)          = *(shg+i);
		*(gradB+neqel3*1+i) = *(shg+npel*1+i);
	}

	*(Bedge)            = *(shg_E);
	*(Bedge + 1)        = *(shg_E + 1);
	*(Bedge + 2)        = *(shg_E + 2);
	*(Bedge+neqel3*1)   = *(shg_E + epel*1);
	*(Bedge+neqel3*1+1) = *(shg_E + epel*1 + 1);
	*(Bedge+neqel3*1+2) = *(shg_E + epel*1 + 2);

	*(curlBedge)     = *(curl_E);
	*(curlBedge + 1) = *(curl_E + 1);
	*(curlBedge + 2) = *(curl_E + 2);

	return 1;
}

int triangleB_V( double *shl, double *shg, double *B, double *gradB)
{
/*
 ....  SET UP THE SHAPE FUNCTION MATRICES FOR
       TWO-DIMENSIONAL CONTINUUM ELEMENTS

		Updated 11/20/09
*/
	int i,i2,i2m1;
	for( i = 0; i < npel; ++i )
	{
		*(B+i)              = *(shg+npel*2+i);

		*(gradB+i)          = *(shg+i);
		*(gradB+neqel3*1+i) = *(shg+npel*1+i);
	}

	return 1;
}

