/*
    This utility function assembles the matrices for a finite
    element program which does analysis on a quad.  It is for
    calculating the voltage potetial.

    Unlike the waveguide matrices calculated in "trekasmbl.c",
    the calculation of the potential uses node based interpolation.
    You can still use the reference:

        Jin, Jianming, The Finite Element Method in Electromagnetics,
            John Wiley & Sons, New York, 1993.

                Updated 12/26/09

    SLFCEM source file
    Version:  1.1
    Copyright (C) 1999-2009  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "treconst.h"
#include "trestruct.h"

#define  DEBUG         0
#define  SMALL         1.0e-12

extern int analysis_flag, Vdof, numel, numnp, sof;
extern int AB_matrix_store, Bzz_matrix_store;
extern int LU_decomp_flag, numel_V_K, numel_V_P;
extern double shg[sosh], shg_node[sosh], shl[sosh], shg_E[sosh_E], shg_node_E[sosh_E],
	curl_E[epel], shl_node[sosh], shl_node2[sosh_node2], w[num_int], *Area0, sigma;

int globalConjKassemble(double *, int *, int , double *,
	double *, int , int , int );

int globalKassemble(double *, int *, double *, int *, int );

int matX( double *,double *,double *, int ,int ,int );

int matXT( double *, double *, double *, int, int, int);

int triangleB_V( double *, double *, double *, double *);

int trshg( double *, int , int , double *, double *, double *);

int trVKassemble( double *KV, int *connect, double *coord, int *el_matl, double *charge,
	int *id, int *idiag, double *KV_diag, int *lm, MATL *matl, double *node_counter,
	double *Volt, double *Arean)
{
	int i, i1, i2, j, k, dof_el[neqel], Vdof_el[neqel3], sdof_el[npel*nsd];
	int check, counter, node, node0, node1, node2, edge;
	int matl_num;
	double permit, permea, kosq, G;
	double lamda, mu;
	double B[soB], gradB[soB6], gradB2[soB6];
	double K_temp[Vneqlsq9], K_el[Vneqlsq9];
	double charge_el[Vneqel3], Volt_el[Vneqel3];
	double coord_el[npel*nsd], coord_el_trans[npel*nsd],
		coord_el_local[npel*nsd2], coord_el_local_trans[npel*nsd2];
	double length[epel], lengthsq[epel];
	double X1, X2, X3, Y1, Y2, Y3;
	double det[1], wXdet;
	double fdum, fdum2;

	for( k = 0; k < numel; ++k )
	{

		matl_num = *(el_matl+k);
		permit = matl[matl_num].er;
		permea = matl[matl_num].ur;
		kosq = matl[matl_num].ko*matl[matl_num].ko;

		/*printf("lamda, mu, Emod, Pois  %f %f %f %f \n", lamda, mu, Emod, Pois);*/

/* Create the coord_el vector for one element */

		for( j = 0; j < npel; ++j )
		{
			node = *(connect+npel*k+j);

			*(sdof_el+nsd*j) = nsd*node;
			*(sdof_el+nsd*j+1) = nsd*node+1;

			*(coord_el+nsd*j)=*(coord+*(sdof_el+nsd*j));
			*(coord_el+nsd*j+1)=*(coord+*(sdof_el+nsd*j+1));

			*(coord_el_trans+j)=*(coord+*(sdof_el+nsd*j));
			*(coord_el_trans+npel*1+j)=*(coord+*(sdof_el+nsd*j+1));

			*(dof_el+ndof*j) = ndof*node;
			*(dof_el+ndof*j+1) = ndof*node+1;

			*(Vdof_el+Vndof*j) = Vndof*node;

/* Count the number of times a particular node is part of an element */

			if(analysis_flag == 1)
				*(node_counter + node) += 1.0;
		}

		X1 = *(coord_el_trans);        Y1 = *(coord_el_trans + npel*1);
		X2 = *(coord_el_trans + 1);    Y2 = *(coord_el_trans + npel*1 + 1);
		X3 = *(coord_el_trans + 2);    Y3 = *(coord_el_trans + npel*1 + 2);

		*(coord_el_local_trans) = X1;     *(coord_el_local_trans + 3) = Y1;
		*(coord_el_local_trans + 1) = X2; *(coord_el_local_trans + 4) = Y2;
		*(coord_el_local_trans + 2) = X3; *(coord_el_local_trans + 5) = Y3;

/* Area is simply the the cross product of the sides connecting node 1 to node 0
   and node 2 to node 0, divided by 2.
*/

		fdum = (X2 - X1)*(Y3 - Y1) - (X3 - X1)*(Y2 - Y1);

		if( fdum <= 0.0 )
		{
			printf("the element (%d) is inverted; 2*Area:%f\n", k, fdum);
			return 0;
		}

/* A factor of 0.5 is needed to do the integration.  See Eq. 3.I.34 in
   "The Finite Element Method" by Thomas Hughes, page 174
*/

		*(Arean + k) = .5*fdum;

		check=trshg( det, k, 1, shl, shg, coord_el_trans);
		if(!check) printf( "Problems with trshg \n");

                memset(K_el,0,neqlsq9*sof);

		for( j = 0; j < num_int; ++j )
		{

		    memset(B,0,npel*sof);
		    memset(gradB,0,soB6*sof);
		    memset(K_temp,0,neqlsq9*sof);

/* Assembly of the component matrices */

		    check = triangleB_V((shl+npel*(nsd+1)*j), (shg+npel*(nsd+1)*j), B, gradB);
		    if(!check) printf( "Problems with triangleB_V\n");

		    memcpy(gradB2, gradB, soB6*sizeof(double));

		    check=matXT(K_temp, gradB, gradB2, Vneqel3, Vneqel3, nsd2);
		    if(!check) printf( "Problems with matXT  \n");

		    fdum = *(w+j)*(*(det));

		    for( i2 = 0; i2 < Vneqlsq9; ++i2 )
		    {
			  *(K_el+i2) += *(K_temp+i2)*fdum*permit;
		    }

		}

/*
                printf("\n K_el %4d\n", k);
                for( i2 = 0; i2 < Vneqel3; ++i2 )
                {
                        for( i1 = 0; i1 < Vneqel3; ++i1 )
                        {
                                printf("%12.8f ",*(K_el + Vneqel3*i2 + i1));
                        }
                        printf("\n");
                }
                printf("\n");
*/


		for( j = 0; j < Vneqel3; ++j )
		{
			*(Volt_el + j) = *(Volt + *(Vdof_el+j));
		}

		check = matX(charge_el, K_el, Volt_el, Vneqel3, 1, Vneqel3);
		if(!check) printf( "Problems with matX \n");

		if(analysis_flag == 1)
		{

/* Compute the equivalant nodal charges based on prescribed displacements */

			for( j = 0; j < Vneqel3; ++j )
			{
				*(charge + *(Vdof_el+j)) -= *(charge_el + j);
				/*printf("%4d %12.8f\n", *(Vdof_el+j), *(charge + *(Vdof_el+j)));*/
			}

#if 1
/* Assembly of either the global skylined K matrix or numel_V of the
   element K matrices if the Conjugate Gradient method is used */

			if(LU_decomp_flag)
			{
			    check = globalKassemble(KV, idiag, K_el, (lm + k*Vneqel3), Vneqel3);
			    if(!check) printf( "Problems with globalKassemble \n");
			}
			else
			{
			    check = globalConjKassemble(KV, Vdof_el, k, KV_diag, K_el,
				Vneqel3, Vneqlsq9, numel_V_K);
			    if(!check) printf( "Problems with globalConjKassemble \n");
			}
#endif


		}
		else
		{

/* Compute the equivalant nodal charges based on prescribed displacements */

			for( j = 0; j < Vneqel3; ++j )
			{
				*(charge + *(Vdof_el+j)) += *(charge_el + j);
				/*printf("%4d %12.8f\n", *(Vdof_el+j), *(charge + *(Vdof_el+j)));*/
			}
		}
	}

	if(analysis_flag == 1)
	{

/* Contract the global force matrix using the id array only if LU decomposition
   is used. */

	  if(LU_decomp_flag)
	  {
	     counter = 0;
	     for( i = 0; i < Vdof ; ++i )
	     {
		if( *(id + i ) > -1 )
		{
			*(charge + counter ) = *(charge + i );
			/*printf("%4d %4d %12.8f\n", counter, i,
				*(charge + counter));*/
			++counter;
		}
	     }
	  }
	}

	return 1;
}

