/*
    This program reads in the new input file and prepares it
    for graphical display.
  
   			Last Update 1/24/10

    SLFCEM source file
    Version:  1.1
    Copyright (C) 1999-2010  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
 */

#if WINDOWS
#include <windows.h>
#endif

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../tri_em/treconst.h"
#include "../tri_em/trestruct.h"
#include "trestrcgr.h"
#include "../../common_gr/control.h"

/* glut header files */
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

/********************* These are all the subroutines **************************/

/******** Data management and calculations ********/

void EM_bc_edge02D( int , BOUND , double *);

void EMnode_vectors02D(int , BOUND , double *, XYF *);

void Volt_bc_node02D(int , BOUND , double *);

void agvMakeAxesList(GLuint);

int Eset( BOUND , int *, XYF *, double *, int *, double *, int *, double *,
	int *, double *, int *, double *, int *);

int Eparameter( double *, double *, double *, double *, double *, double * );

int EReGetMemory2_gr( XYF **, int );

int edge_id( BOUND , double *, int *, int );

int tredge( int *, int *, int *, int * );

int Ereader_gr( FILE *, double *, double *);

int Ereader( BOUND , int *, int *, int *, double *, int *, double *, MATL *,
	char *, FILE *, double *, double *);

int EReGetMemory( double **, int, int **, int, MATL **, int , XYI **, int );

int filecheck( char *, char *, FILE **, FILE **, FILE **, char *, int );

/******************************* GLOBAL VARIABLES **************************/

/****** FEA globals ******/
extern int dof, sdof, EMdof, Vdof, nmat, nmode, numed, numel, numnp;
extern int modal_flag, static_flag, edge_connect_read_flag;
extern int flag_3D;
extern XYI *mem_XYI;
extern XYF *mem_XYF;
extern int *mem_int;
extern double *mem_double;
extern double *coord, *coord0;
extern double *EMedge, *EMnode, *EMmag;
extern double *Volt;
extern int *connecter;
extern int *edge_connect, *node_edge_connect;
extern BOUND bc;
extern MATL *matl;
extern int *el_matl;
extern double *charge;
extern double sigma, low_mode;

/* Global variables for the mesh color and nodal data */

extern int *EMnode_color, *EMedge_color, *EMmag_color,
	*charge_color, *Volt_color, *el_matl_color;
extern MATL *matl_crtl;

/* Global variables for drawing the axes */
extern GLuint AxesList, VoltBCList, EMnodeList, EMedgeBCList;   /* Display lists */

/* Global variables for drawing the charge vectors */
extern XYF *EMnode_vec, *EMnode_vec0;

/****** For drawing the Mesh Window ******/
extern double coord_rescale;

extern char NameData[25], ModeData[25];

extern int input_flag, post_flag, matl_choice, node_choice, ele_choice, mode_choice;
extern int Before_flag, After_flag;
extern double amplify_factor;

int trGetNewMesh(void)
{
	int i, j, check, node, edge;
	char *ccheck;
	int dum, dum1, dum2, dum3, dum4;
	double fdum, fdum1, fdum2;
	double fpointx, fpointy, fpointz;
	int sofmi, sofmf, sofmXYI, sofmXYF, ptr_inc;
	char name[50], name2[50], otr_exten[4], buf[ BUFSIZ ], char_dum[25];
	int otr_exten_length = 4;
	FILE *o1, *o2, *o3;
	int numed_mem, numed2;

/* Delete the old display lists */

	glDeleteLists(AxesList,1);
	if(input_flag)
	{
		glDeleteLists(VoltBCList,1);
		glDeleteLists(EMnodeList,1);
		glDeleteLists(EMedgeBCList,1);
	}

/* Initialize filenames */

	memset(name,0,50*sizeof(char));
	memset(name2,0,50*sizeof(char));
	memset(ModeData,0,25*sizeof(char));
	memset(NameData,0,25*sizeof(char));
	memset(otr_exten,0,otr_exten_length*sizeof(char));

/* Initialize old variables */

	input_flag = 1;
	post_flag = 1;
	After_flag = 1;
	Before_flag = 0;
	mode_choice = 0;
	amplify_factor = 1.0;
	matl_choice = 0;
	node_choice = 0;
	ele_choice = 0;

	ccheck = strncpy(otr_exten,".otr",otr_exten_length);
	if(!ccheck) printf( " Problems with strncpy \n");

	printf("What is the name of the input file containing the \n");
	printf("triangle structural data? (example: dipoletr)\n");
	scanf( "%50s",name2);

/*   o1 contains all the structural data for input
     o3 contains all the structural data for postprocessing
     o2 is used to determine the existance of input and post files
*/
	o2 = fopen( name2,"r" );
	if(o2 == NULL ) {
		printf("Can't find file %50s\n", name2);
		exit(1);
	}
	/*printf( "%3d %50s\n ",name2_length,name2);*/

	fgets( buf, BUFSIZ, o2 );
	fscanf( o2, "%d %d %d %d %d %lf %lf\n",
	    &numed, &numel, &numnp, &nmat, &nmode, &sigma, &low_mode);
	dof=numnp*ndof;
	sdof=numnp*nsd;
	Vdof=numnp*Vndof;
	nmode = abs(nmode);

	numed_mem = numel*epel;  /* numed_mem is for the allocation of memory.
	                            I need it because the data file may contain
	                            the edges exactly specified or they may be
	                            generated in the code.  numel*epel is the
	                            greatest numeber of edges possible.  This usually
	                            means I allocate twice the amount I really need. */
	EMdof=numed_mem*edof;    /* I set EMdof temporarily to this maximum value for
	                            memory allocation.  I set it to its true value
	                            aftor "tredge" is potentially called.*/


/* Begin exmaining and checking for the existence of data files */

	check = filecheck( name, name2, &o1, &o2, &o3, otr_exten, otr_exten_length );
	if(!check) printf( " Problems with filecheck \n");

	if( input_flag )
	{
	    fgets( buf, BUFSIZ, o1 );
	    fscanf( o1, "%d %d %d %d %d %lf %lf\n",&dum,&dum1,&dum2,&dum3,&dum4,&fdum1,&fdum2);
	    printf( "%d %d %d %d %d %lf %lf\n",dum,dum1,dum2,dum3,dum4,fdum1,fdum2);
	    /*printf( "name %50s\n ",name);*/
	}
	if( post_flag )
	{
	    fgets( buf, BUFSIZ, o3 );
	    fscanf( o3, "%d %d %d %d %d %lf %lf\n",&dum,&dum1,&dum2,&dum3,&dum4,&fdum1,&fdum2);
	    printf( "%d %d %d %d %d %lf %lf\n",dum,dum1,dum2,dum3,dum4,fdum1,fdum2);
	    /*printf( "out %50s\n ",out);*/
	}

	static_flag = 1;
	modal_flag = 0;
	if(nmode)
	{
		static_flag = 0;
		modal_flag = 1;
	}

/*   Begin allocation of meomory */

/* For the doubles */
	sofmf=2*sdof+2*Vdof+numnp+EMdof+dof+EMdof;

/* For the integers */
	sofmi= numel*npel + numed_mem*nped + numel*epel + numel + numnp+1 +
		numed_mem+1 + nped*numed_mem+1 + 2*numnp+2 + 4 + 3*Vdof + numed_mem +
		dof;

/* For the XYI integers */
	sofmXYI=numnp+1+1;

	check = EReGetMemory( &mem_double, sofmf, &mem_int, sofmi, &matl, nmat,
		&mem_XYI, sofmXYI );
	if(!check) printf( " Problems with EReGetMemory\n");

/* For the doubles */
	                                        ptr_inc=0;
	coord=(mem_double+ptr_inc);             ptr_inc += sdof;
	coord0=(mem_double+ptr_inc);            ptr_inc += sdof;
	charge=(mem_double+ptr_inc);            ptr_inc += Vdof;
	Volt=(mem_double+ptr_inc);              ptr_inc += Vdof;
	EMmag=(mem_double+ptr_inc);             ptr_inc += numnp;
	EMedge=(mem_double+ptr_inc);            ptr_inc += EMdof;
	EMnode=(mem_double+ptr_inc);            ptr_inc += dof;
	bc.EMedge=(mem_double+ptr_inc);         ptr_inc += EMdof;

/* For the materials */

	matl_crtl = matl;

/* For the integers */
	                                        ptr_inc = 0;
	connecter=(mem_int+ptr_inc);            ptr_inc += numel*npel;
	node_edge_connect=(mem_int+ptr_inc);    ptr_inc += numed_mem*nped;
	edge_connect=(mem_int+ptr_inc);         ptr_inc += numel*epel;
	el_matl=(mem_int+ptr_inc);              ptr_inc += numel;
	bc.Volt =(mem_int+ptr_inc);             ptr_inc += numnp+1;
	bc.edge =(mem_int+ptr_inc);             ptr_inc += numed_mem+1;
	bc.edge_node =(mem_int+ptr_inc);        ptr_inc += nped*numed_mem+1;
	bc.charge =(mem_int+ptr_inc);           ptr_inc += numnp+1;
	bc.EMnode =(mem_int+ptr_inc);           ptr_inc += numnp+1;
	bc.num_Volt=(mem_int+ptr_inc);          ptr_inc += 1;
	bc.num_edge=(mem_int+ptr_inc);          ptr_inc += 1;
	bc.num_charge=(mem_int+ptr_inc);        ptr_inc += 1;
	bc.num_EMnode=(mem_int+ptr_inc);        ptr_inc += 1;
	charge_color=(mem_int+ptr_inc);         ptr_inc += Vdof;
	Volt_color=(mem_int+ptr_inc);           ptr_inc += Vdof;
	EMmag_color=(mem_int+ptr_inc);          ptr_inc += Vdof;
	EMedge_color=(mem_int+ptr_inc);         ptr_inc += numed_mem;
	EMnode_color=(mem_int+ptr_inc);         ptr_inc += dof;

	el_matl_color = el_matl;

/* For the XYI integers */
	                                        ptr_inc = 0;
	bc.fix =(mem_XYI+ptr_inc);              ptr_inc += numnp+1;
	bc.num_fix=(mem_XYI+ptr_inc);           ptr_inc += 1;

/* If there is no post file, then set coord to coord0 */

	if( !post_flag )
	{
	    coord = coord0;
	    After_flag = 0;
	    Before_flag = 1;
	}

/* If there is no input file, then set coord0 to coord */

	if( !input_flag )
	{
	    /*coord0 = coord;*/
	    After_flag = 1;
	    Before_flag = 0;
	}

	dum = strlen(name);
	memset(char_dum,0,20*sizeof(char));
	if( dum < 9 )
	{
		ccheck = strncpy(char_dum, name, 8);
		sprintf( NameData, "%s", char_dum);
	}
	else
	{
		ccheck = strncpy(char_dum, name, 5);
		sprintf( NameData, "%s...", char_dum);
	}

	if(modal_flag) if(mode_choice != 0) sprintf( ModeData, "%4d", mode_choice);


	if( post_flag )
	{
		check = Ereader( bc, connecter, node_edge_connect, edge_connect,
			coord, el_matl, charge, matl, name, o3, EMedge, Volt);
		if(!check) printf( " Problems with Ereader \n");

		check = Ereader_gr( o3, EMnode, EMmag);
		if(!check) printf( " Problems with Ereader_gr \n");

		if(modal_flag)
		    if(!edge_connect_read_flag)
			for( i = 0; i < EMdof; ++i )
			    *(EMedge + i) = bc.EMedge[i];

		printf( "\n\n");
	}
	if( input_flag )
	{
		check = Ereader( bc, connecter, node_edge_connect, edge_connect,
			coord0, el_matl, charge, matl, name, o1, EMedge, Volt);
		if(!check) printf( " Problems with Ereader \n");

		/*check = Ereader_gr( o1, EMnode, EMmag);
		if(!check) printf( " Problems with Ereader_gr \n");*/

		printf( "\n\n");
	}

	if(modal_flag)
	{
	    if(!edge_connect_read_flag)
	    {
		memset(node_edge_connect, -1, numed_mem*nped*sizeof(int));
		memset(edge_connect, -1, numel*epel*sizeof(int));

		check = tredge( connecter, edge_connect, node_edge_connect, &numed2 );
		if(!check) printf( " Problems with tredge \n");

		numed2 -= 1;    /* If the edge connectivity is generated by tredge, */
		numed = numed2;     /* set numed to numed2. */

		check = edge_id( bc, EMedge, node_edge_connect, numed2);
		if(!check) printf( " Problems with edge_id\n");
	    }
	    if(edge_connect_read_flag)
	    {
		for( i = 0; i < bc.num_edge[0]; ++i)
		{
		    edge = bc.edge[i];
		    bc.edge_node[nped*i] = *(node_edge_connect + nped*edge);
		    bc.edge_node[nped*i + 1] = *(node_edge_connect + nped*edge + 1);
		}
	    }
	    printf( "\n\n");
	}


/* For the XYF doubles */
	sofmXYF=2*numnp;
/*
   This is allocated seperately from EMemory_gr because usually we don't know the
   number of EMnode vectors needed until we read data from Eread_gr and calculate
   bc.num_charge[0].  But this is true only for SLFFEA, where the analogous quantity
   is bc.num_forec[0].  Here, I know it will be numnp.
*/

	check = EReGetMemory2_gr( &mem_XYF, sofmXYF );
	if(!check) printf( " Problems with EReGetMemory2_gr\n");

	                                        ptr_inc = 0;
	EMnode_vec =(mem_XYF+ptr_inc);          ptr_inc += numnp;
	EMnode_vec0 =(mem_XYF+ptr_inc);         ptr_inc += numnp;

/* Search for extreme values */
 
/* In mesh viewer, search for extreme values of nodal points, the Electric Field
   to obtain viewing parameters and make color assignments.  Also initialize
   variables
*/
	check = Eparameter( coord, EMnode, EMmag, EMedge, charge, Volt );
	if(!check) printf( " Problems with Eparameter \n");

/* Rescale undeformed coordinates */

	if( coord_rescale > 1.01 || coord_rescale < .99 )
	{
	   if( input_flag && post_flag )
	   {
		for( i = 0; i < numnp; ++i )
		{
			*(coord0+nsd*i) /= coord_rescale;
			*(coord0+nsd*i+1) /= coord_rescale;
		}
	   }
	}

	if( !input_flag )
	{
	    for ( i = 0; i < numnp; ++i)
	    {
		*(coord0 + nsd*i) = *(coord + nsd*i) - *(EMnode + ndof*i);
		*(coord0 + nsd*i + 1) = *(coord + nsd*i + 1) - *(EMnode + ndof*i + 1);
	    }
	}

	check = Eset( bc, connecter, EMnode_vec0, EMnode, EMnode_color,
		EMmag, EMmag_color, EMedge, EMedge_color, charge, charge_color,
		Volt, Volt_color );
	if(!check) printf( " Problems with Eset \n");

	AxesList = glGenLists(1);
	agvMakeAxesList(AxesList);

	if( input_flag )
	{

/* create display list for displacement and charge grapics vectors
   on undeformed mesh*/

/* This is for the boundary Voltage nodes. */

	    VoltBCList = glGenLists(1);
	    Volt_bc_node02D(VoltBCList, bc, coord0);

	    for( i = 0; i < numnp; ++i)
	    {
		fpointx = *(coord0+nsd*i);
		fpointy = *(coord0+nsd*i + 1);
		EMnode_vec[i].x = fpointx - EMnode_vec0[i].x;
		EMnode_vec[i].y = fpointy - EMnode_vec0[i].y;
	    }
    
/* This is for the electric field vectors. */

	    EMnodeList = glGenLists(1);
	    EMnode_vectors02D(EMnodeList, bc, coord0, EMnode_vec);

/* This is for the boundary edges. */
    
	    EMedgeBCList = glGenLists(1);
	    EM_bc_edge02D(EMedgeBCList, bc, coord0);
	}

	if( post_flag )
	{
/* create charge grapics vectors for deformed mesh*/

	    for( i = 0; i < numnp; ++i)
	    {
		fpointx = *(coord+nsd*i);
		fpointy = *(coord+nsd*i + 1);
		EMnode_vec[i].x = fpointx - EMnode_vec0[i].x;
		EMnode_vec[i].y = fpointy - EMnode_vec0[i].y;
	    }
	}
	return 1;
}
