/*
    This program shows either the 2 or 3 dimensional model of the finite
    element mesh for triangle elements.
  
   			Last Update 1/24/10

    SLFCEM source file
    Version:  1.1
    Copyright (C) 1999-2010  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
 */

#if WINDOWS
#include <windows.h>
#endif

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <stdarg.h>
#include <string.h>
#include <math.h>
#include "../tri_em/treconst.h"
#include "../tri_em/trestruct.h"
#include "trestrcgr.h"
#include "../../common_gr/control.h"
#include "../../common_gr/color_gr.h"
#include "../../common_gr/egui.h"

/* glut header files */
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#if LINUX
/* X11 header files. */
#include <GL/glx.h>

#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xmu/StdCmap.h>
#include <X11/keysym.h>

#include <X11/X.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#endif

/* The header files below can be used to incorporate
   widgets into your code */
#if 0
/* X11 Widget header files. */
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Form.h>
#include <GL/xmesa.h>
#include <GL/MesaDrawingArea.h>

#include <Xm/PushB.h>
#include <Xm/Form.h>
#include <GL/xmesa.h>
#include <GL/MesaMDrawingArea.h>

#define GLwMakeCurrent GLwMMakeCurrent
#endif

/********************* These are all the subroutines **************************/

/******** Data management and calculations ********/

void EM_bc_edge02D( int , BOUND , double *);

void EMnode_vectors02D(int , BOUND , double *, XYF *);

void Volt_bc_node02D(int , BOUND , double *);

void agvMakeAxesList(GLuint);

int Eset( BOUND , int *, XYF *, double *, int *, double *, int *, double *,
	int *, double *, int *, double *, int *);

int Eparameter( double *, double *, double *, double *, double *, double * );

int EMemory2_gr( XYF **, int );

int edge_id( BOUND , double *, int *, int );

int tredge( int *, int *, int *, int * );

int Ereader_gr( FILE *, double *, double *);

int Ereader( BOUND , int *, int *, int *, double *, int *, double *, MATL *,
	char *, FILE *, double *, double *);

int EMemory( double **, int, int **, int, MATL **, int , XYI **, int );

int filecheck( char *, char *, FILE **, FILE **, FILE **, char *, int );

/******** For the Mesh ********/

void MeshKey_Special(int , int , int );

void EMeshKeys( unsigned char , int , int  );

void trmeshdraw(void);

void trrender(void);

void MeshReshape(int , int );

void EMeshDisplay(void);

void MeshInit(void);

/******** For the Control Panel ********/

void EControlMouse(int , int , int , int );

void ControlReshape(int , int );

void EControlDisplay(void);

void EMenu();

void ControlInit(void);

int ControlDimInit( void );

/******** For the Screen Dump ********/

void ScreenShot(int, int);

/* These are functions provided by Phillip Winston for
   movement of the mesh.*/

void agvHandleButton(int , int , int , int );

void agvHandleMotion(int , int );


/******************************* GLOBAL VARIABLES **************************/

/****** FEA globals ******/
int dof, sdof, EMdof, Vdof, nmat, nmode, numed, numel, numnp;
int modal_flag, static_flag, edge_connect_read_flag;
int flag_3D;
XYI *mem_XYI;
XYF *mem_XYF;
int *mem_int;
double *mem_double;
double *coord, *coord0;
double *EMedge, *EMnode, *EMmag;
double *Volt;
int *connecter;
int *edge_connect, *node_edge_connect;
BOUND bc;
MATL *matl;
int *el_matl;
double *charge;
double sigma, low_mode;

/* Global variables for the mesh color and nodal data */

int *EMnode_color, *EMedge_color, *EMmag_color,
	*charge_color, *Volt_color, *el_matl_color;
MATL *matl_crtl;

/* This used to be in MeshInit */

GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
GLfloat mat_shininess[] = { 50.0 };
GLfloat light_position[] = { 1.0, 1.0, 6.0, 0.0 };

/****** graphics globals ******/

int ControlWindow, MeshWindow;

/* Determines the step size for keyboard and Control panel movement */

double step_sizex = .1, step_sizey = .1, step_sizez = .1;

/****** Translation Variables ******/
double left_right, up_down, in_out, left_right0, up_down0, in_out0;

/****** Rotation Variables ******/
double xAngle = 0.0, yAngle = 0.0, zAngle = 0.0;

/****** Cross Section Plane Translation Variables ******/
double cross_sec_left_right, cross_sec_up_down, cross_sec_in_out,
	cross_sec_left_right0, cross_sec_up_down0, cross_sec_in_out0;

/* Global variables for drawing the axes */
GLuint AxesList, VoltList, EMnodeList, EMedgeList;   /* Display lists */
double AxisMax_x, AxisMax_y, AxisMax_z,
	AxisMin_x, AxisMin_y, AxisMin_z,
	IAxisMin_x, IAxisMin_y, IAxisMin_z;
double AxisLength_x, AxisLength_y, AxisLength_z, AxisLength_max, AxisPoint_step;

/* Global variables for drawing the charge vectors */
XYF *EMnode_vec, *EMnode_vec0;

/****** For drawing the Mesh Window ******/
double left, right, top, bottom, near, far, fscale, coord_rescale;
double ortho_left, ortho_right, ortho_top, ortho_bottom,
	ortho_left0, ortho_right0, ortho_top0, ortho_bottom0;
int mesh_width, mesh_height;
int com_mesh_width0 = mesh_width0;
int com_mesh_height0 = mesh_height0;

/* These Variables are for the Control Panel */

int sofi = sizeof(int);
int row_number = rowdim;
int ControlDiv_y[rowdim + 2], ControlDiv_x[rowdim + 2];
int control_width, control_height;
int com_control_width0 = control_width0;
int com_control_height0 =  control_height0;
int current_width, current_height;

double ratio = scaleFactor, ratio2 = 1.0;
double ratio_width = 1.0;
double ratio_height = 1.0;

int boxMove_x, boxMove_y, boxTextMove_x, textMove_x, textMove_y[rowdim+2];
int Color_flag[rowdim];

char RotateData[3][25] = { "    0.00", "    0.00", "    0.00" };
char MoveData[3][25] = { "    0.00", "    0.00", "    0.00" };
char NameData[25] = { " Name"};
char ModeData[25] = { "  "};
char AmplifyData[25] = { " "};
char BoxData[2*boxnumber+2][25] = { "", "", "", "", "", "", "", "",
	"", "", "", "", "", "", "", "", "", "" };
char BoxText[25];

int del_height = 0;
int del_width = 0;

double matl_choicef = 0, node_choicef = 0, ele_choicef = 0;

int textDiv_xa = textDiv_xa0;
int textDiv_xb = textDiv_xb0;
int boxTextMove_x = boxTextMove_x0;

/* These Variables partition the Electric Field */

double EMnodex_div[boxnumber+1], EMnodey_div[boxnumber+1], EMmag_div[boxnumber+1],
	EMedge_div[boxnumber+1], charge_div[boxnumber+1], Volt_div[boxnumber+1];
double init_right, init_left, init_top,
	init_bottom, init_near, init_far, true_far, dim_max;
double max_EMnodex, min_EMnodex, del_EMnodex, max_EMnodey, min_EMnodey, del_EMnodey,
	absolute_max_EMnode, max_EMmag, min_EMmag, del_EMmag,
	max_EMedge, min_EMedge, del_EMedge,
	max_charge, min_charge, del_charge, max_Volt, min_Volt, del_Volt,
	absolute_max_coord;

/* These are the flags */

int input_flag = 1,          /* Tells whether an input file exists or not */
    post_flag = 1,           /* Tells whether a post file exists or not   */
    color_choice = 3,        /* Set to desired color analysis(range 1 - 24) */
    choice = 0,              /* currently unused */
    matl_choice = -1,        /* Set to which material to view  */
    node_choice = -1,        /* Set to which node to view  */
    ele_choice = -1,         /* Set to which element to view  */
    mode_choice = 0;         /* Set to the desired modal post file */

int input_color_flag = 0;    /* Used with input_flag to determine how to draw input mesh */
int ortho_redraw_flag = 0;   /* calls MeshReshape(currently not used) */
int Solid_flag = 1,          /* Selects between wire frame or solid model */
    Perspective_flag = 1,    /* Selects between orthographic and perspecive views */
    Render_flag = 0,         /* Selects between rendering or analyses */
    EMedgeColor_flag = 0,    /* Turns on and off the wireframe using colors based on the Edge
                                Electric Field values determined in modal analysis. */
    EMfieldVec_flag = 0,     /* Turns Electric Field Vectors on and off */
    Material_flag = 0,       /* Turns material on and off */
    Node_flag = 0,           /* Turns Node ID on and off */
    Element_flag = 0,        /* Turns Element ID on and off */
    Axes_flag = 0,           /* Turns Axes on and off  */
    Boundary_flag = 0,       /* Turns Boundary ID on and off  */
    Outline_flag = 1,        /* Turns Element Outline on and off  */
    Transparent_flag = 0,    /* Turns Element Transparency on and off  */
    CrossSection_flag = 0;   /* Turns CrossSection_flag on and off  */
int Before_flag = 0,         /* Turns before mesh on and off */
    After_flag = 1,          /* Turns after mesh on and off */
    Both_flag = 0,           /* Turns both before and after on and off */
    Amplify_flag = 0;        /* Turns Amplification on and off */

double amplify_factor = 1.0; /* Amplifies deformed mesh for better viewing */
double amplify_step = 0.1;   /* Value of amplification icrement */
double amplify_step0 = 0.1;  /* Value of initial calculated amplification icrement */

int node_edge_quantities_flag = 0;  /* Tells whether viewing of the node or edge quantities
                                       charge, voltage, Electric Field, etc. is on or off */

int disp_flag = 0;           /* Tells whether displacement viewing is on and off.
                                This is a legacy flag which is a global variable
                                in "scalcom.c" so must be retained. */

int main(int argc, char** argv)
{
	int i, j, check;
	char *ccheck;
	int dum, dum1, dum2, dum3, dum4;
	double fdum, fdum1, fdum2;
	double fpointx, fpointy, fpointz;
	int sofmi, sofmf, sofmXYI, sofmXYF, ptr_inc;
	char name[50], name2[50], otr_exten[4], buf[ BUFSIZ ], char_dum[25];
	int otr_exten_length = 4;
	FILE *o1, *o2, *o3;
	int numed_mem, numed2;

	right=0;
	top=0;
	left=1000;
	bottom=1000;
	fscale=0;
	near=1.0;
	far=10.0;
	/*mesh_width=500;
	mesh_height=20;
	control_width=1000;
	control_height=1500;*/

/* Initialize filenames */

	memset(name,0,50*sizeof(char));
	memset(name2,0,50*sizeof(char));
	memset(otr_exten,0,otr_exten_length*sizeof(char));

	ccheck = strncpy(otr_exten,".otr",otr_exten_length);
	if(!ccheck) printf( " Problems with strncpy \n");

	printf("What is the name of the input file containing the \n");
	printf("triangle structural data? (example: diske4atr)\n");
	scanf( "%50s",name2);

/*   o1 contains all the structural data for input
     o3 contains all the structural data for postprocessing
     o2 is used to determine the existance of input and post files
*/
	o2 = fopen( name2,"r" );
	if(o2 == NULL ) {
		printf("Can't find file %50s\n", name2);
		exit(1);
	}
	/*printf( "%3d %50s\n ",name2_length,name2);*/

	fgets( buf, BUFSIZ, o2 );
	fscanf( o2, "%d %d %d %d %d %lf %lf\n",
	    &numed, &numel, &numnp, &nmat, &nmode, &sigma, &low_mode);
	dof=numnp*ndof;
	sdof=numnp*nsd;
	Vdof=numnp*Vndof;
	nmode = abs(nmode);

	numed_mem = numel*epel;  /* numed_mem is for the allocation of memory.
	                            I need it because the data file may contain
	                            the edges exactly specified or they may be
	                            generated in the code.  numel*epel is the
	                            greatest numeber of edges possible.  This usually
	                            means I allocate twice the amount I really need. */
	EMdof=numed_mem*edof;    /* I set EMdof temporarily to this maximum value for
	                            memory allocation.  I set it to its true value
	                            aftor "tredge" is potentially called.*/


/* Begin exmaining and checking for the existence of data files */

	check = filecheck( name, name2, &o1, &o2, &o3, otr_exten, otr_exten_length );
	if(!check) printf( " Problems with filecheck \n");

	if( input_flag )
	{
	    fgets( buf, BUFSIZ, o1 );
	    fscanf( o1, "%d %d %d %d %d %lf %lf\n",&dum,&dum1,&dum2,&dum3,&dum4,&fdum1,&fdum2);
	    printf( "%d %d %d %d %d %lf %lf\n",dum,dum1,dum2,dum3,dum4,fdum1,fdum2);
	    /*printf( "name %50s\n ",name);*/
	}
	if( post_flag )
	{
	    fgets( buf, BUFSIZ, o3 );
	    fscanf( o3, "%d %d %d %d %d %lf %lf\n",&dum,&dum1,&dum2,&dum3,&dum4,&fdum1,&fdum2);
	    printf( "%d %d %d %d %d %lf %lf\n",dum,dum1,dum2,dum3,dum4,fdum1,fdum2);
	    /*printf( "out %50s\n ",out);*/
	}

	static_flag = 1;
	modal_flag = 0;
	if(nmode)
	{
		static_flag = 0;
		modal_flag = 1;
	}

/*   Begin allocation of meomory */

/* For the doubles */
	sofmf=2*sdof+2*Vdof+numnp+EMdof+dof+EMdof;

/* For the integers */
	sofmi= numel*npel + numed_mem*nped + numel*epel + numel + numnp+1 +
		numed_mem+1 + nped*numed_mem+1 + 2*numnp+2 + 4 + 3*Vdof + numed_mem +
		dof;

/* For the XYI integers */
	sofmXYI=numnp+1+1;

	check = EMemory( &mem_double, sofmf, &mem_int, sofmi, &matl, nmat,
		&mem_XYI, sofmXYI );
	if(!check) printf( " Problems with EMemory \n");

/* For the doubles */
	                                        ptr_inc=0;
	coord=(mem_double+ptr_inc);             ptr_inc += sdof;
	coord0=(mem_double+ptr_inc);            ptr_inc += sdof;
	charge=(mem_double+ptr_inc);            ptr_inc += Vdof;
	Volt=(mem_double+ptr_inc);              ptr_inc += Vdof;
	EMmag=(mem_double+ptr_inc);             ptr_inc += numnp;
	EMedge=(mem_double+ptr_inc);            ptr_inc += EMdof;
	EMnode=(mem_double+ptr_inc);            ptr_inc += dof;
	bc.EMedge=(mem_double+ptr_inc);         ptr_inc += EMdof;

/* For the materials */

	matl_crtl = matl;

/* For the integers */
	                                        ptr_inc = 0;
	connecter=(mem_int+ptr_inc);            ptr_inc += numel*npel;
	node_edge_connect=(mem_int+ptr_inc);    ptr_inc += numed_mem*nped;
	edge_connect=(mem_int+ptr_inc);         ptr_inc += numel*epel;
	el_matl=(mem_int+ptr_inc);              ptr_inc += numel;
	bc.Volt =(mem_int+ptr_inc);             ptr_inc += numnp+1;
	bc.edge =(mem_int+ptr_inc);             ptr_inc += numed_mem+1;
	bc.edge_node =(mem_int+ptr_inc);        ptr_inc += nped*numed_mem+1;
	bc.charge =(mem_int+ptr_inc);           ptr_inc += numnp+1;
	bc.EMnode =(mem_int+ptr_inc);           ptr_inc += numnp+1;
	bc.num_Volt=(mem_int+ptr_inc);          ptr_inc += 1;
	bc.num_edge=(mem_int+ptr_inc);          ptr_inc += 1;
	bc.num_charge=(mem_int+ptr_inc);        ptr_inc += 1;
	bc.num_EMnode=(mem_int+ptr_inc);        ptr_inc += 1;
	charge_color=(mem_int+ptr_inc);         ptr_inc += Vdof;
	Volt_color=(mem_int+ptr_inc);           ptr_inc += Vdof;
	EMmag_color=(mem_int+ptr_inc);          ptr_inc += Vdof;
	EMedge_color=(mem_int+ptr_inc);         ptr_inc += numed_mem;
	EMnode_color=(mem_int+ptr_inc);         ptr_inc += dof;

	el_matl_color = el_matl;

/* For the XYI integers */
	                                        ptr_inc = 0;
	bc.fix =(mem_XYI+ptr_inc);              ptr_inc += numnp+1;
	bc.num_fix=(mem_XYI+ptr_inc);           ptr_inc += 1;

/* If there is no post file, then set coord to coord0 */

	if( !post_flag )
	{
	    coord = coord0;
	    After_flag = 0;
	    Before_flag = 1;
	}

/* If there is no input file, then set coord0 to coord */

	if( !input_flag )
	{
	    /*coord0 = coord;*/
	    After_flag = 1;
	    Before_flag = 0;
	}

	dum = strlen(name);
	memset(char_dum,0,20*sizeof(char));
	if( dum < 9 )
	{
		ccheck = strncpy(char_dum, name, 8);
		sprintf( NameData, "%s", char_dum);
	}
	else
	{
		ccheck = strncpy(char_dum, name, 5);
		sprintf( NameData, "%s...", char_dum);
	}

	if(modal_flag) if(mode_choice != 0) sprintf( ModeData, "%4d", mode_choice);


	if( post_flag )
	{
		check = Ereader( bc, connecter, node_edge_connect, edge_connect,
			coord, el_matl, charge, matl, name, o3, EMedge, Volt);
		if(!check) printf( " Problems with Ereader\n");

		check = Ereader_gr( o3, EMnode, EMmag);
		if(!check) printf( " Problems with Ereader_gr\n");

		if(modal_flag)
		    if(!edge_connect_read_flag)
			for( i = 0; i < EMdof; ++i )
			    *(EMedge + i) = bc.EMedge[i];

		printf( "\n\n");
	}
	if( input_flag )
	{
		check = Ereader( bc, connecter, node_edge_connect, edge_connect,
			coord0, el_matl, charge, matl, name, o1, EMedge, Volt);
		if(!check) printf( " Problems with Ereader\n");

		/*check = Ereader_gr( o1, EMnode, EMmag);
		if(!check) printf( " Problems with Ereader_gr\n");*/

		printf( "\n\n");
	}

	if(modal_flag)
	{
	    if(!edge_connect_read_flag)
	    {
		memset(node_edge_connect, -1, numed_mem*nped*sizeof(int));
		memset(edge_connect, -1, numel*epel*sizeof(int));

		check = tredge( connecter, edge_connect, node_edge_connect, &numed2 );
		if(!check) printf( " Problems with tredge \n");

		numed2 -= 1;        /* If the edge connectivity is generated by tedge, */
		numed = numed2;     /* set numed to numed2. */

		check = edge_id( bc, EMedge, node_edge_connect, numed2);
		if(!check) printf( " Problems with edge_id\n");

	    }
	    printf( "\n\n");
	}

/* For the XYF doubles */
	sofmXYF=2*numnp;
/*
   This is allocated seperately from EMemory_gr because we need to know the
   number of EMnode vectors read from Eread_gr.
*/

	check = EMemory2_gr( &mem_XYF, sofmXYF );
	if(!check) printf( " Problems with EMemory2_gr \n");

	                                        ptr_inc = 0;
	EMnode_vec =(mem_XYF+ptr_inc);          ptr_inc += numnp;
	EMnode_vec0 =(mem_XYF+ptr_inc);         ptr_inc += numnp;

/* Search for extreme values */
 
/* In mesh viewer, search for extreme values of nodal points, the Electric Field
   to obtain viewing parameters and make color assignments.  Also initialize
   variables
*/

	check = Eparameter( coord, EMnode, EMmag, EMedge, charge, Volt );
	if(!check) printf( " Problems with Eparameter \n");

/* Rescale undeformed coordinates */

	if( coord_rescale > 1.01 || coord_rescale < .99 )
	{
	   if( input_flag && post_flag )
	   {
		for( i = 0; i < numnp; ++i )
		{
			*(coord0+nsd*i) /= coord_rescale;
			*(coord0+nsd*i+1) /= coord_rescale;
		}
	   }
	}

	if( !input_flag )
	{
	    for ( i = 0; i < numnp; ++i)
	    {
		*(coord0 + nsd*i) = *(coord + nsd*i) - *(EMnode + ndof*i);
		*(coord0 + nsd*i + 1) = *(coord + nsd*i + 1) - *(EMnode + ndof*i + 1);
	    }
	}

	check = Eset( bc, connecter, EMnode_vec0, EMnode, EMnode_color,
		EMmag, EMmag_color, EMedge, EMedge_color, charge, charge_color,
		Volt, Volt_color );
	if(!check) printf( " Problems with Eset \n");

/* Initialize the mesh viewer */

	glutInit(&argc, argv);
	glutInitWindowSize(mesh_width0, mesh_height0);
	glutInitWindowPosition(400, 215);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	MeshWindow = glutCreateWindow("SLFCEM");
	MeshInit ();

	AxesList = glGenLists(1);
	agvMakeAxesList(AxesList);

/* Below,  I calculate EMnode_vec[i].* for the charge vectors graphics.  The reason I
   have coded things like this is because I think it gives me a slight improvement in
   speed.  When glutDisplayFunc displays the mesh, it continuously calls all the
   functions used in displaying the mesh like the subroutines which draw the Electric
   Field vectors and prescribed Voltage nodes.  This doesn't matter for the undeformed
   mesh where everything is drawn from display lists, but for the deformed mesh, it is
   an issue.  So I calculate EMnode_vec[i].* outside those functions, rather than simply
   passing EMnode_vec0[i].* to the particular *EMnode_vectors function and doing something
   like:

                fx = fpointx - EMnode_vec0[node_num].x;
                fy = fpointy - EMnode_vec0[node_num].y;

   There is probably only a small advantage, but that is the reason.
*/

	if( input_flag )
	{

/* create display list for the Electric Field vectors and and prescribed Voltage nodes
   on undeformed mesh*/

	    VoltList = glGenLists(1);
	    Volt_bc_node02D(VoltList, bc, coord0);

	    for( i = 0; i < numnp; ++i)
	    {
		fpointx = *(coord0+nsd*i);
		fpointy = *(coord0+nsd*i + 1);
		EMnode_vec[i].x = fpointx - EMnode_vec0[i].x;
		EMnode_vec[i].y = fpointy - EMnode_vec0[i].y;
	    }
    
	    EMnodeList = glGenLists(1);
	    EMnode_vectors02D(EMnodeList, bc, coord0, EMnode_vec);

	    EMedgeList = glGenLists(1);
	    EM_bc_edge02D(EMedgeList, bc, coord0);
	}

	if( post_flag )
	{
/* create charge grapics vectors for deformed mesh*/

	    for( i = 0; i < numnp; ++i)
	    {
		fpointx = *(coord+nsd*i);
		fpointy = *(coord+nsd*i + 1);
		EMnode_vec[i].x = fpointx - EMnode_vec0[i].x;
		EMnode_vec[i].y = fpointy - EMnode_vec0[i].y;
	    }
	}

/* Initiate variables in Control Panel */

	memset(Color_flag,0,rowdim*sofi);

	check = ControlDimInit();
	if(!check) printf( " Problems with ControlDimInit \n");

/* call display function  */

	glutDisplayFunc(EMeshDisplay);

	glutReshapeFunc(MeshReshape);

/* Initialize Mouse Functions */

	glutMouseFunc(agvHandleButton);
	glutMotionFunc(agvHandleMotion);

/* Initialize Keyboard Functions */

	glutKeyboardFunc(EMeshKeys);
	glutSpecialFunc(MeshKey_Special);

/* Initialize the Control Panel */

	glutInitWindowSize(control_width0, control_height0);
	glutInitWindowPosition(0, 0);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB );
	ControlWindow = glutCreateWindow("SLFCEM Control Panel");

	ControlInit();
	EMenu();
	glutDisplayFunc(EControlDisplay);
	glutReshapeFunc(ControlReshape);

	glutMouseFunc(EControlMouse);

/* call function for hotkeys
 */
#if 0
	glutKeyboardFunc(ControlHandleKeys);
#endif
	glutMainLoop();

	free(matl);
	free(mem_double);
	free(mem_int);
	free(mem_XYI);
	free(mem_XYF);
	return 1;    /* ANSI C requires main to return int. */
}
